var app = angular.module( 'uiApp', [
    'ngRoute',
    'ngCookies',
    'ngIdle',
    'ngSpectrum',
    'login',
    'configure', 'configNetworkedAppliances', 'configUsers',
    'jobs',
    'reports',
    'help',
    'options',
    'mgo-angular-wizard',
    'dynamicdashboardapp',
    "alertWidgetModule",
    'dailyFeedWidgetModule',
    'ngSanitize',
    'nsPopover',
    'angulartics',
    'angulartics.google.analytics',
    'tour-module',
    'inventory',
    'job-calendar',
    'date_range',
    'networked-appliances-module',
    'networked-appliance-customers-module',
    'networked-appliance-options-module',
    'distributed-reports-module',
    'angular-flippy',
    'cookie-manager',
    'connection-manager',
    'angular-tour',
    'report-details-module',
    'gettext',
    'language-selection',
    'credential-manager'
]);

app.factory('Page', ['cookiesvc','$rootScope', function(cookiesvc,$rootScope){
    var title = 'Unitrends Portal';
    return {
        title: function() {
            return location.host + ' - ' + title;
        },
        setTitle: function(newTitle, update_cookies) {
            if (update_cookies === undefined || update_cookies === true) {
                cookiesvc.put('Page', newTitle);
            }
            title = newTitle;
        }
    };
}]);

/*
modal dialog for alert
*/
app.directive('alertDialog', function () {
    return {
        restrict: 'E',
        scope: {
            show: '='
        },
        replace: true, // Replace with the template below
        transclude: true, // we want to insert custom content inside the directive
        link: function (scope, element, attrs) {
            scope.dialogStyle = {};
            if (attrs.width)
                scope.dialogStyle.width = attrs.width;
            if (attrs.height)
                scope.dialogStyle.height = attrs.height;
            scope.hideModal = function () {
                scope.show = false;
            };
        },
        template: "<div class='ng-modal' ng-show='show'><div class='ng-alertmodeldialog-dialog' ng-style='dialogStyle'><div class='ng-alertmodeldialog-dialog-content' ng-transclude></div></div></div>"
    };
});
app.factory('dataToBeShared', function(){
    return { objectList:undefined };
});


/*
 * Items controlled at the global level.
 */
app.controller('GlobalCtrl', ['$scope', '$rootScope', '$window', '$location', '$http', 'Page', 'Idle', 'ngDialog', 'Title', '$timeout', 'cookiesvc',
		function($scope, $rootScope, $window, $location, $http, Page, Idle, ngDialog, Title, $timeout, cookiesvc) {
	
	$scope.Page = Page;

	/*open and close dailog*/
	$scope.modalShown = false;
    document.onkeydown = function (event) {
        var keyCode = event.keyCode;
        if (keyCode == 8 &&
            ((event.target || event.srcElement).tagName !== "TEXTAREA") &&
            ((event.target || event.srcElement).tagName !== "INPUT")) {

            if (navigator.userAgent.toLowerCase().indexOf("msie") === -1) {
                if (typeof(window.PHD.currentDialog) !== 'undefined' && typeof(window.PHD.currentDialog.wizard) !== 'undefined'){
                    try{
                        window.PHD.currentDialog.wizard("close");
                    }
                    catch(err) {

                    }
                }
                ngDialog.close();
                return true;
            } else {
                event.returnValue = false;
            }
            return false;
        }
    };
    $scope.toggleModal = function() {
        if($scope.modalShown === true)
        {
            $('.ng-alertmodeldialog-dialog').css("display","none");
            $scope.modalShown = !$scope.modalShown;
            $timeout(function() {
                $scope.$apply(function() {
                    $("#id-alerts-menu").removeClass("open");
                    $("#id-alerts-menu").blur();
                });
            }, 0);
        }
        else
        { $('.ng-alertmodeldialog-dialog').css("display","block");
            $scope.modalShown = !$scope.modalShown;
        }
    };

    $rootScope.checkRole = function(roleName, subsystem) {
        var disabled = false;
        if (roleName !== undefined) {
            switch (subsystem) {
                case 'configure':
                    disabled = roleName == 'backup_operator' || roleName == 'restore_operator' || roleName == 'operator';
                    break;
                case 'backup':
                    disabled = roleName == 'restore_operator';
                    break;
                case 'recover':
                    disabled = roleName == 'backup_operator';
                    break;
            }
        }
        return disabled;
    };

	// user name should be passed from login
	$rootScope.backupTimer=10000;
    $rootScope.maintenance_mode = false;
    $rootScope.saveTimers = [];
    $rootScope.saveJobTimer = 0;
    $rootScope.saveJobTimer = 0;

    $rootScope.local = {};
    $rootScope.local.id = localID;
    $rootScope.local.name = "portal";

    // Default job history to max of 250 records
    // This could be persisted if desired, but the concern is that the more jobs the more chance to overload the DataTable.
    $rootScope.jobHistoryLimit = 250;
    $rootScope.jobHistoryLimitHit = "";

    // Default audit history to max of 250 records
    $rootScope.systemJobHistoryLimit = 250;
    $rootScope.systemJobHistoryLimitHit = "";

    // is this the portal
    $rootScope.isPortal = true;
    $rootScope.userLanguageCode = "en";  // default langauge for front page.

        // See if the portal is directly connected to an appliance.
    $rootScope.managedAppliance = {'address': '', 'AuthToken': '', 'hostname': ''};
    var managerToken = cookiesvc.get('managedAppliance.AuthToken');
    if (managerToken !== undefined) {
        $rootScope.managedAppliance.AuthToken = managerToken;
        $rootScope.managedAppliance.hostname = cookiesvc.get('managedAppliance.hostname');
        $rootScope.managedAppliance.address = cookiesvc.get('managedAppliance.address');
    }
    // See if the user is logged in.
    var $cookie = {};
    $cookie.username = cookiesvc.get("username");
    $cookie.uid =  cookiesvc.get('uid', true) !== 'undefined' ? cookiesvc.get("uid") : undefined;
    $cookie.password = cookiesvc.get("password");
    $cookie.token = cookiesvc.get('token', true) !== 'undefined' ? cookiesvc.get("token") : undefined;
    $cookie.superuser = cookiesvc.get('superuser', true) !== 'undefined' ? cookiesvc.get("superuser") : undefined;
    $cookie.administrator = cookiesvc.get('administrator', true) !== 'undefined' ? cookiesvc.get("administrator") : undefined;
    $cookie.manager = cookiesvc.get('manager', true) !== 'undefined' ? cookiesvc.get("manager") : undefined;
    $cookie.monitor = cookiesvc.get('monitor', true) !== 'undefined' ? cookiesvc.get("monitor") : undefined;
    $cookie.company_id = cookiesvc.get('company_id', true) !== 'undefined' ? cookiesvc.get("company_id") : undefined;
    $cookie.role_name = cookiesvc.get('role_name', true) !== 'undefined' ? cookiesvc.get("role_name") : undefined;
    $cookie.role_scope = cookiesvc.get('role_scope', true) !== 'undefined' ? cookiesvc.get("role_scope") : undefined;
    $cookie.role_recover_options = cookiesvc.get('role_recover_options', true) !== 'undefined' ? cookiesvc.get("role_recover_options") : undefined;

    if ($cookie.token !== undefined && $cookie.username !== undefined) {
        $scope.user = {};
        $scope.user.loggedIn = true;
        $scope.user.name = $cookie.username;
        $scope.user.id = $cookie.uid;
        $scope.user.token = $cookie.token;
        $scope.user.superuser = $cookie.superuser;
        $scope.user.administrator = $cookie.administrator;
        $scope.user.manager = $cookie.manager;
        $scope.user.monitor = $cookie.monitor;
        $scope.user.company_id = $cookie.company_id;
        $scope.user.role_name = $cookie.role_name;
        $scope.user.role_scope = $cookie.role_scope;
        $scope.user.role_recover_options = $cookie.role_recover_options;

        // Based on role, disable major subsystems.
        $rootScope.backupDisabled = $rootScope.checkRole($scope.user.role_name, 'backup');
        $rootScope.configureDisabled = $scope.user.monitor; //$rootScope.checkRole($scope.user.role_name, 'configure');
        $rootScope.recoverDisabled = $rootScope.checkRole($scope.user.role_name, 'recover');
        $http.defaults.headers.common['xg-token'] = $scope.user.token;
    } else {
        $scope.user = {};
        $scope.user.name = '';
        $scope.user.id = '';
        $scope.user.loggedIn = false;
        $scope.user.token = '';
        $scope.user.superuser = false;
        $scope.user.administrator = false;
        $scope.user.monitor = false;
        $scope.user.role_name = undefined;
        $scope.user.role_scope = undefined;
    }

	// set preferences for user, refresh rates, get user id.
	// track the timezone: (timezone = 'US/Eastern')
	// track username, first and last name, email, active_user
	// is_superuser, is_staff?
	// date_user_created
	// tz string for timezone (tz='-0400')
	
	$rootScope.user = $scope.user;
    global_user = $rootScope.user;

    // Platform-specifics
    $rootScope.isNetworkManager = (showLooselyCoupled === 1);

    /*
     * Set rootScope variable to track view of backup catalog filter (defaults to "show").
     */
    $rootScope.showCatalogFilter = true;

	$scope.user.logout = function() {

        var userout = {};
        userout.user = $scope.user.name;
        userout.token = $scope.user.token;
        /*
         * Turn off the idle timer, if set
         */
        Idle.unwatch();

        $http({
            method: "DELETE",
            url: '/portal/api/login.php',
            data: userout})
            .then(function(result) {
                    $scope.logoutUser();
                },
                function(result) {
                    data = result.data;
                    if (data !== undefined && data.message !== undefined) {
                        console.log("user.logout: error, message = " + data.message);
                    }
                    $scope.logoutUser();
                })
	};

    $(document).on("session-expired", function(event) {
        $scope.user.logout();
    });

    $scope.logoutUser = function() {
        // Send an event on logout.
        var e = $.Event('logout');
        $(document).trigger(e);

        // remove cookies if present.
        cookiesvc.remove('username');
        cookiesvc.remove('password');
        cookiesvc.remove('uid');
        cookiesvc.remove('superuser');
        cookiesvc.remove('administrator');
        cookiesvc.remove('monitor');
        cookiesvc.remove('company_id');
        cookiesvc.remove('remember_me');
        cookiesvc.put('autologin', false);

        $scope.user.loggedIn = false;
        $rootScope.user.loggedIn = false;
        $rootScope.user.token = '';
        cookiesvc.remove('token');

        // force reload on logout to clean up Timers.
        $window.location.href = "#";
        $window.location.reload();
    };
	
	$scope.credentials = {
		username: '',
		password: ''
	};

	$rootScope.setIfDefined = function(obj, widget, variable, value) {
		if (typeof value !== 'undefined') {
            rate = value * 1000;
			obj[variable] = rate;
            $(document).trigger("userpreferencesave." + widget, [variable, rate]);
		}
	};
	
	$scope.bodyIsReady = function() {
		// Send an event to the jQuery listeners on login.

	};

    /*
     * Idle timeout handling.  Set the timer on login, based on CMC Timeout value.  Turn on the timer with "watch()".
     * On logout, turn off the timer with "unwatch()".
     *  When Idle Start is reached, issue a warning.
     *  When Idle timeout is reached, log out
     *  If the user clicks or moves the mouse in the app (IdleEnd), remove the warning dialog.
     */
    $scope.$on('IdleStart', function() {

        ngDialog.open({
            dialogType:'warning',
            dialogMessage: "This session will soon timeout due to inactivity unless you click 'OK' or move your mouse in this window.",
            modelDialogId: 'idle-timeout-dialog',
            scope: $scope,
            onConfirmOkButtonClick:'onConfirmIdle()'
        });
    });

    $scope.onConfirmIdle = function() {
        $timeout(function() {
            Title.value(Page.title());
        }, 0);
        ngDialog.close('idle-timeout-dialog');
    };

    $scope.$on('IdleEnd', function() {
        $scope.onConfirmIdle();
    });

    $scope.$on('IdleTimeout', function() {
        $scope.onConfirmIdle();
        $scope.user.logout();
    });

    //
    // Provides a common validation function for nested dialogs.  The caller must provide the below <div> in the $form
    //
    //  <div id="nested-dialog-errors"><p class="summary error"></p><ul id="nested-dialog-error-list" class="errorlist"></ul></div>
    //
    $rootScope.validateNestedDialog = function($form, options) {
        var wrapper = "#nested-dialog-error-list";
        var errors = "#nested-dialog-errors";
        var dialogDefaults = {
            errorLabelContainer: wrapper,
            errorContainer: errors,
            showErrors: function (errorMap, errorList) {
                $(errors).find(".summary")
                    .html(gettext("Correct the following errors:"));
                this.defaultShowErrors();
            }
        };
        options = jQuery.extend(options, dialogDefaults);
        return $form.validate(options);
    };
}]);

/*
 * Items controlled at the global level.
 */
app.controller('AppCtrl', ['getPreferencesFactory', '$scope', '$rootScope', '$location', 'cookiesvc', '$route', '$http', '$timeout', 'Page', 'ngDialog', 'Idle',
	function(getPreferencesFactory, $scope, $rootScope, $location, cookiesvc, $route, $http, $timeout, Page, ngDialog, Idle) {

	 /*Event to handle preferences*/
	   $scope.$on("update-userPrefrence", function(event, widgetName,refreshKey) {
	 			var dataObj ={};
				var url = '/portal/api/preferences.php?user=' + encodeURI($rootScope.user.name);
				var dropdownKey="#id_"+widgetName.toLowerCase()+"Refresh";
                var my_dropid= "id_"+widgetName.toLowerCase()+'Refresh';
                var dropdown_id= "[id='"+my_dropid+"']";
				dataObj[refreshKey]=$(dropdown_id).val();
		 		$http({
					   url: url, 
					   data: JSON.stringify(dataObj),
					   method: 'PUT'
					}).then(function(result){
					//	console.log("success");
						console.log(result.data);
					},function(response){
						console.log(response);
					}); 
	});
	
	/*
	Initializing the variable at rootScopeLevel to implement Timer and save userPreference for refresh rate
    * 1) Stores function need to be called to refresh data.
	* 2) Stores time period after refresh function need to be called
	* 3) Stores ID return by setInterval function
	* 4) Stores ID of refresh Icon
	* 5) Stores controller of Widgets
	* 6) Stores widget specific refreshKey for saving userPreference
	*/

    $rootScope.listOfFunction=[
        "$rootScope.getAlertsRefereshData",
        "$rootScope.getTopForumPosts",
        "$rootScope.getDailyFeed",
        "$rootScope.getNetworkedApplianceData"
    ];
    $rootScope.listOfTimer=[0,0,0,0];
    $rootScope.listOfWidgetName=[
        "Alerts",
        "Post",
        "Daily Feed",
        "Distributed Status",
    ];
    $rootScope.listOfTimerFunction=[
        "Alerts",
        "Post",
        "Daily Feed",
        "Distributed Status"
    ];
    $rootScope.listOfID=[
        "[id='IDAlerts']",
        "[id='IDCOMMUNITY']",
        "[id='IDDAILY FEED']",
        "[id='IDREMOTE APPLIANCE STATUS']"
    ];
    $rootScope.listOfController=[
        "[ng-controller=AlertWidgetContorller]",
        "[ng-controller=TopForumPostsWidgetCtrl]",
        "[ng-controller=DailyFeedWidgetController]",
        "[ng-controller=networkedAppliancesCtrl]"
    ];
    $rootScope.listOfRefreshKey=[
        "dash_alert_refresh",
        "dash_forum_posts_refresh",
        "dash_daily_feed_refresh",
        "dash_distributed_status_refresh" // just a placeholder - functionality will be added later
    ];

    /* Timer for refresh of Jobs Page */
    $rootScope.JobsTimer = PHD.Timer({
        namespace: "jobspage",
        interval: 0,
        refreshControl: null
    });

    /* Timer for refresh of Protect Page */
    $rootScope.ProtectTimer = PHD.Timer({
        namespace: "protectpage",
        interval: 0,
        refreshControl: null
    });

    /* Timer for refresh of Protect Page */
    $rootScope.RecoverTimer = PHD.Timer({
        namespace: "recoverpage",
        interval: 0,
        refreshControl: null
    });

    $rootScope.disableTimers = function() {
        if (!$rootScope.maintenance_mode) {
            var nTimers = $rootScope.listOfTimer.length;
            $rootScope.saveTimers = $rootScope.listOfTimer;
            for (var i = 0; i < nTimers; i++) {
                $rootScope.listOfTimer[i] = 0;
                clearInterval($rootScope.listOfTimerFunction[i]);
            }
            $rootScope.saveJobTimer = $rootScope.JobsTimer.getInterval();
            $rootScope.saveProtectTimer = $rootScope.ProtectTimer.getInterval();
            $rootScope.saveRecoverTimer = $rootScope.RecoverTimer.getInterval();
            $rootScope.JobsTimer.setInterval(0);
            $rootScope.ProtectTimer.setInterval(0);
            $rootScope.RecoverTimer.setInterval(0);
            $rootScope.maintenance_mode = true;
        }
    };

    $rootScope.enableTimers = function() {
        var nSavedTimers = $rootScope.saveTimers.length;
        var nTimers = $rootScope.listOfTimer.length;
        $rootScope.listOfTimer = $rootScope.saveTimers;
        for (var i = 0; i < nTimers && i < nSavedTimers; i++) {
            $rootScope.listOfTimer[i] = 300 * 1000;
            $rootScope.listOfTimerFunction[i] = setInterval(eval($rootScope.listOfFunction[i]), $rootScope.listOfTimer[i]);
        }
        if ($rootScope.saveJobTimer !== undefined) {
            $rootScope.JobsTimer.setInterval($rootScope.saveJobTimer);
        }
        if ($rootScope.saveProtectTimer !== undefined) {
            $rootScope.ProtectTimer.setInterval($rootScope.saveProtectTimer);
        }
        if ($rootScope.saveRecoverTimer !== undefined) {
            $rootScope.RecoverTimer.setInterval($rootScope.saveRecoverTimer);
        }
        $rootScope.maintenance_mode = false;
    };
	
	/* function to hide/show manual refresh control for widget 
	 * and also refresh widget based on time selected by user
	 * Parameters : refresh_rate and Widget Name
    */
        $rootScope.manualRefreshControl  = function(refresh_rate,widgetName){
            var index = $rootScope.listOfWidgetName.indexOf(widgetName);
            var my_id =$rootScope.listOfID[index];
            var dropDownID='#id_' +widgetName.toLowerCase() +'Refresh';
            var my_dropid= "id_"+widgetName.toLowerCase()+'Refresh';
            var dropdown_id= "[id='"+my_dropid+"']";
           
            /*check for refresh rate , if it is zero we need to show manual refresh icon
             else we need to call function for updating Widget data using setInterval method,
             whereas on change of user Preference we need to clear previous timer Function
             */
            $(dropdown_id).val(refresh_rate); // set the value of dropDown with selected value
            if (refresh_rate === 0){
                $rootScope.listOfTimer[index]=refresh_rate;
                clearInterval($rootScope.listOfTimerFunction[index]);
                if(my_id === "[id='IDCOMMUNITY']" ||
                    my_id === "[id='IDDAILY FEED']" || 
                    my_id === "[id='IDACTIVE JOBS']" || 
                    my_id === "[id='IDSTORAGE']" ||
                    my_id === "[id='IDDISTRIBUTED APPLIANCE STATUS']" ||
                    my_id === "[id='IDRECOVERY ASSURANCE']"){
                    $(my_id).css("display","inline-block");
                }
            } else {
                $(my_id).css("display","none");
                clearInterval($rootScope.listOfTimerFunction[index]);
                refresh_rate = refresh_rate * 1000;
                $rootScope.listOfTimer[index]=refresh_rate;
                $rootScope.listOfTimerFunction[index] = setInterval(eval($rootScope.listOfFunction[index]), $rootScope.listOfTimer[index]);
            }
        };

    /* show or hide user selectable refresh */
    $rootScope.userPreferencesOptionsHidden = function(widgetName) {
        return  widgetName === 'Backup' || widgetName === 'Recover Summary' ||
            widgetName === 'Backup Copy' || widgetName === 'Backup Copy Target'|| widgetName === 'Backup Copy Storage' ||
            widgetName === 'Distributed Status' || widgetName === 'Daily Feed';
    };

    /*cancel setting User Preference*/
    $rootScope.userPreferenceParentCancel = function (refresh_rate, widgetName) {
        var my_dropid= "id_"+widgetName.toLowerCase()+'Refresh';
        var dropdown_id= "[id='"+my_dropid+"']";
        $(dropdown_id).val(refresh_rate); // set the value of dropDown with the original value.
    };

	/*save User Preference*/
	$rootScope.userPreferenceParentSave = function (widgetName) {
        var index = $rootScope.listOfWidgetName.indexOf(widgetName);
        var dropDownID='#id_' +widgetName.toLowerCase() +'Refresh';
        var my_dropid= "id_"+widgetName.toLowerCase()+'Refresh';
        var dropdown_id= "[id='"+my_dropid+"']";
        var refresh_rate= parseInt($(dropdown_id).val());
        var scope = $($rootScope.listOfController[index]).scope();
        scope.toggleModal();
        $rootScope.manualRefreshControl(refresh_rate,widgetName);
        $scope.$broadcast("update-userPrefrence", widgetName,$rootScope.listOfRefreshKey[index]);
	};

	$scope.Page = Page;

	// Initialize navigation to expanded.
	// lmc - TODO load from user preferences.
	cookiesvc.put('nav', 'nav-expanded');
	var navExpanded = {
			cookieNav: 'nav-expanded',
			arrowNav: 'icon-arrowleft',
			navTitle: 'Collapse Left Navigation'
	};
	var navCollapsed = {
			cookieNav: 'nav-collapsed',
			arrowNav: 'icon-arrowright',
			navTitle: 'Expand Left Navigation'
	};

	// set default view, overridden by user preference.
	$scope.navbar = navExpanded;

	$scope.showSelected = function(selectedItem) {
	    var currentLocation = $location.path().substring(1) || 'dynamic-dashboard';
	    var returnClass = '';
	    if (selectedItem === 'jobs' && $rootScope.managedAppliance.AuthToken !== '') {
            returnClass = (currentLocation.indexOf(selectedItem) !== -1) ? 'selected-remote' : '';
        } else {
            returnClass = (currentLocation.indexOf(selectedItem) !== -1) ? 'selected' : '';
        }
	    return returnClass;
	};
	
	/*
	 * Send an event to the rest of the system when a subsystem is selected.
	  */
	$scope.selectNav = function(selectedItem) {
		var e = $.Event('uinav.' + selectedItem);
		$(document).trigger(e);
	};
	
	/*
	 * Collapse or exapand the menu/navigation on the left-hand-side.
	  */
	$scope.toggleNav = function() {
		if ($scope.navbar.cookieNav == 'nav-expanded') {
			$scope.navbar = navCollapsed;
		} else if ($scope.cookieNav = 'nav-collapsed') {
			$scope.navbar = navExpanded;
		}
        cookiesvc.put('nav', $scope.navbar.cookieNav);
	};


    // Login the user, passing in credentials, autologin, or from the cookie store (if remember_me).
    $rootScope.loginUser = function(username, password, auto, remember) {
        var redirectToDash = false;
        var autologin = auto || false;
        var remember_me = remember || false;

        if (username === undefined || password === undefined) {
            if (remember_me) {
                $scope.credentials.username = cookiesvc.get("username");
                $scope.credentials.password = cookiesvc.get("password");
            } else {
                return;
            }
        }  else {
            $scope.credentials.username = username;
            $scope.credentials.password = password;
        }

        var load;
        if (!autologin) {
            var $loginButton = $("#login-button");
            load = PHD.showLoadingIndicator($loginButton, false);
        }
        
        $scope.saveUser = function(global_scope, username, id) {
            global_scope.user.name = username;
            global_scope.user.id = id;
            cookiesvc.put('uid', id);
            cookiesvc.put('username', username);
        };
        
        $scope.savePrivileges = function(global_scope, privilege) {
            var priv_level = parseInt(privilege);
            global_scope.user.privilege = priv_level;
            global_scope.user.superuser = priv_level === 3;
            global_scope.user.administrator = priv_level >= 2;
            global_scope.user.manager = priv_level >= 1;
            global_scope.user.monitor = priv_level === 0;

            cookiesvc.put('superuser', global_scope.user.superuser);
            cookiesvc.put('administrator', global_scope.user.administrator);
            cookiesvc.put('manager', global_scope.user.manager);
            cookiesvc.put('monitor', global_scope.user.monitor);
        };
        
        $scope.saveCompany = function(global_scope, company) {
            global_scope.user.company_id = parseInt(company);
            cookiesvc.put('company_id', global_scope.user.company_id);
        };

        $http.post('/portal/api/login.php', $scope.credentials)
            .then(function(result) {

                var data = result.data;
                $scope.saveUser($rootScope, $scope.credentials.username, data.user_id);
                $scope.savePrivileges($rootScope, data.privilege);
                $scope.saveCompany($rootScope, data.company_id);

                // Role information for role-based access.
                cookiesvc.put('role_name', data.role_name);
                $rootScope.user.role_name = data.role_name;
                cookiesvc.put('role_scope', data.role_scope);
                $rootScope.user.role_scope = data.role_scope;
                cookiesvc.put('role_recover_options', data.role_recover_options);
                $rootScope.user.role_recover_options = data.role_recover_options;

                // Based on role, disable major subsystems.
                $rootScope.backupDisabled = $rootScope.checkRole($rootScope.user.role_name, 'backup');
                $rootScope.configureDisabled = $rootScope.user.monitor; //checkRole($rootScope.user.role_name, 'configure');
                $rootScope.recoverDisabled = $rootScope.checkRole($rootScope.user.role_name, 'recover');

                //$rootScope.user.loggedIn = true;  // set loggin flag

                // See if we should remember or clear the credentials.
                if (remember_me) {
                    cookiesvc.put('password', $scope.credentials.password);
                    cookiesvc.put('remember_me', true);
                } else {
                    cookiesvc.remove('password');
                    cookiesvc.remove('remember_me');
                }

                // set the default headers for the $http service.
                data.auth_token = data['xg-token'];         // portal token
                $http.defaults.headers.common['xg-token'] = data.auth_token;
                cookiesvc.put('token', data.auth_token);

                // get this user's preferences
                getPreferencesFactory.async($scope,$rootScope,cookiesvc).then(function(result) {
                    var data = result.data.preferences;

                    //For QuickStartWizard Flag
                    $rootScope.user.loggedIn = true;  // set loggin flag
                    // clear the logged in status on the login page
                    if (!autologin) {
                        PHD.hideLoadingIndicator(load);
                    }

                    $rootScope.passwd = $scope.credentials.password; // setting current password to root variable for quick start wizard.

                    $scope.setPreferences(result, false);

                });

                // Send an event to the jQuery listeners on login.
                var e = $.Event('login');
                e.name = $scope.user.name;
                e.token = data.auth_token;
                e.version = '1.0';
                // Not great practice, but this is timing between Angular and Ajax calls.
                // We must set rootScope first so that global_user will be set in PHD ajax calls.
                $rootScope.user.token = e.token;
                $(document).trigger(e);

            }, function(result) {
                var data = result.data;
                if (!autologin) {
                    // Remove the loading indicator and tap into PHD.throwError function here.
                    window.PHD.hideLoadingIndicator(load);
                    window.PHD.throwError(data);
                }
            })
    };

    $scope.setPreferences = function(result, isRefresh) {
        var data = result.data.preferences;
        var isRefresh = isRefresh || false;

        if (!isRefresh) {
            // if a refresh, don't need the extra reload.
            var apppage = angular.element(document.getElementById("app"));
            var injector = apppage.injector();
            var route = injector.get('$route');
            route.reload();
        }

        //Set current tour step
        $rootScope.user.tourStep = 1;

        // update the language to be used based on user preference.
        $rootScope.userLanguageCode = (data.language !== undefined) ? data.language : 'en';
        $rootScope.$broadcast("update-language", $rootScope.userLanguageCode);

        $rootScope.setIfDefined($scope.Preferences, "backupwidget", "dash_backup_refresh", data.dash_backup_refresh);
        $rootScope.setIfDefined($scope.Preferences, "alertswidget", "dash_alert_refresh", data.dash_alert_refresh);
        $rootScope.setIfDefined($scope.Preferences, "activejobswidget", "dash_active_job_refresh", data.dash_active_job_refresh);
        $rootScope.setIfDefined($scope.Preferences, "jobspage", "page_jobs_refresh", data.page_jobs_refresh);

        var FIVE_MINUTES = 5 * 60 * 1000;

        refreshRates.dash_active_job_refresh = $scope.Preferences.dash_active_job_refresh;
        refreshRates.dash_alert_refresh = $scope.Preferences.dash_alert_refresh;
        refreshRates.dash_storage_refresh = $scope.Preferences.dash_storage_refresh;
        refreshRates.page_jobs_refresh = $scope.Preferences.page_jobs_refresh;
        $rootScope.listOfTimer[0] = data.dash_alert_refresh !== undefined ? data.dash_alert_refresh * 1000 : refreshRates.dash_alert_refresh;
        $rootScope.listOfTimer[1] = data.dash_forum_posts_refresh !== undefined ? data.dash_forum_posts_refresh * 1000 : FIVE_MINUTES;
        $rootScope.listOfTimer[2] = data.dash_daily_feed_refresh !== undefined ? data.dash_daily_feed_refresh * 1000 : FIVE_MINUTES;
        $rootScope.listOfTimer[3] = data.dash_daily_feed_refresh !== undefined ? data.dash_daily_feed_refresh * 1000 : FIVE_MINUTES;
        if (isRefresh) {
            for (var i = 0; i <= 8; i++) {
                var widgetName = $rootScope.listOfWidgetName[i];
                $rootScope.manualRefreshControl($rootScope.listOfTimer[i]/1000,widgetName);
            }
        }
        var jobsRefresh = data.page_jobs_refresh !== undefined ? data.page_jobs_refresh : refreshRates.page_jobs_refresh;
        $rootScope.JobsTimer.setInterval(jobsRefresh * 1000);
        var protectRefresh = data.page_protect_refresh !== undefined ? data.page_protect_refresh : refreshRates.page_jobs_refresh;
        $rootScope.ProtectTimer.setInterval(protectRefresh * 1000);
        var restoreRefresh = data.page_restore_refresh !== undefined ? data.page_restore_refresh : refreshRates.page_restore_refresh;
        $rootScope.RecoverTimer.setInterval(restoreRefresh * 1000);

        // remember subsystem the user was in last.
        if (cookiesvc.get('Page') !== undefined) {
            var Page = cookiesvc.get('Page');
            var lower_Page = Page.toLowerCase();
            // Redirect if not the dashboard.
            if (lower_Page !== "dashboard") {
                var re = /reports\/\d+\/\d+\/\d+$/;
                // And, redirect if not a direct -report URL from dashboard.
                if (!location.href.match(re)) {
                    re = /configure\/[a]?\d+\/[a-z]+\/\d+$/;
                    if (!location.href.match(re)) {
                        // And, redirect if not a direct -configure URL from dashboard.
                        location.href = "#!/" + lower_Page;
                    }
                }
            }
        }

        /*
         * If the idle timer is set in master.ini, use it.  Otherwise, turn off the timer.
         */
        var logoutTimer = data.timeout;
        if (logoutTimer !== undefined && parseInt(logoutTimer) > 0) {
            Idle.setIdle(parseInt(logoutTimer) * 60);
            Idle.setTimeout(30);    // An additional 30 seconds once the warning is displayed 'til logout.
            Idle.watch();
        } else {
            Idle.unwatch();
        }

        /* Triggering Event once user has logged-in
         * $rootScope.$broadcast is used as there is no parent-child relation
         * between login and alert module
         */
        $timeout(function() {
            $rootScope.$broadcast("logged-in");
        }, 1000);
    };

    angular.element(document).ready(function() {
        // lmc - set as false below to stop initial exceptions until this code is moved to portal.
        if ($rootScope.user.loggedIn && false) {
            // get this user's preferences
            var getPreferences = function() {
                getPreferencesFactory.async($scope,$rootScope,cookiesvc).then(function(result) {
                    $scope.setPreferences(result, true)
                });
            };
            getPreferences();
        } else {
            var autologin = cookiesvc.get('autologin');
            if (autologin === undefined) {
                $http.get("app/login/autologin.json").then(function (result) {
                    var data = result.data;
                    autologin = data.autologin;
                    cookiesvc.put('autologin', autologin);
                    if (autologin) {
                      $rootScope.loginUser(DEFAULT_USER, DEFAULT_PW, true, false);
                    }
                });
            } else if (autologin) {

                $rootScope.loginUser(DEFAULT_USER, DEFAULT_PW, true, false);
            } else {
                var remember_me = cookiesvc.get('remember_me');
                if (remember_me) {
                  $rootScope.loginUser(undefined, undefined, false, true);
                }
            }
        }
    });
}]);

app.controller('AccountCtrl', ['$scope', '$rootScope', '$http', 'ngDialog',
    function($scope, $rootScope, $http, ngDialog) {

    $scope.editUserAccount = function() {
        var url = '/portal/api/users.php?id=' + $rootScope.user.id;
        var load = PHD.showLoadingIndicator("body", true);
        var method = 'GET';
        $http({
            method: method,
            url : url
        }).then(function(result){
            PHD.hideLoadingIndicator(load);
            var data = result.data;
            var users = data.users;
            if (users instanceof Array && users.length > 0) {
                $scope.selectedUser = users[0];
                $scope.userTitle = 'Edit Account Details';
                $scope.isEdit = true;
                ngDialog.open({
                    template: 'app/configure/users/edit.html',
                    scope: $scope,
                    data: {'user': $scope.selectedUser, 'userTitle': $scope.userTitle, 'editDetails': false},
                    overlay: true,
                    modelDialogId: 'User-Manager',
                    name: 'User-Manager',
                    ngDialogStyle: 'width:500px; height:250px;',
                    closeByDocument: false,
                    closeByEscape: true,
                    preCloseCallback: function (value) {
                    }
                });
            } else {
                ngDialog.open({
                    dialogType: 'Error',
                    modelDialogId: 'user-error-dialog',
                    dialogMessage: 'Cannot find user information.'
                });
            }
        }, function(result) {
            PHD.hideLoadingIndicator(load);
            ngDialog.open({
                dialogType: 'Error',
                modelDialogId: 'user-error-dialog',
                dialogMessage: 'Cannot find user information.'
            });
        });
    };
}]);


app.config(['$qProvider', function ($qProvider) {
    $qProvider.errorOnUnhandledRejections(false);
}]);

app.config(function($routeProvider, $analyticsProvider) {
	$routeProvider.when('/', {
        templateUrl: "app/dynamic-dashboard/dynamicDashboard.html",
        controller: 'DynamicDashboardController'
	})
	.when('/jobs', {
		templateUrl: "app/jobs/jobs.html",
		controller: 'JobsCtrl'
	})
	.when('/reports', {
		templateUrl: "app/reports/reports.html",
		controller: 'ReportsCtrl'
	})
	.when('/configure', {
		templateUrl: "app/configure/configure.html",
		controller: 'ConfigCtrl',
        resolve: {
            factory: roleRoutingHandler
        }
	})
	.when('/reports/:id/:days/:sid', {
		templateUrl: "app/reports/reports.html",
		controller: 'ReportsCtrl'
	})
    .when('/configure/:id/:usage/:sid', {
        templateUrl: "app/configure/configure.html",
        controller: 'ConfigCtrl',
        resolve: {
            factory: roleRoutingHandler
        }
    })
	.otherwise({ redirectTo: '/' });
});

/*
 * Checks the incoming route to see if it should be allowed or not.
 * Returns true if so and false if not (and redirects to main page).
 */
var roleRoutingHandler = function($rootScope, $location) {
    var locationPath = $location.path();
    var ret = true;
    if (($rootScope.configureDisabled && (locationPath.indexOf('/configure') !== -1)) ||
        ($rootScope.backupDisabled && (locationPath.indexOf('/protect') !== -1)) ||
        ($rootScope.recoverDisabled && (locationPath.indexOf('/recover') !== -1))) {
        $location.path('/');
        ret = false;
    }
    return ret;
};

app.factory('getPreferencesFactory', function($http) {
    return {
        async: function($scope, $rootScope, cookiesvc) {
            var DEFAULT_REFRESH = 5000;
            $scope.Preferences = {};
            pref = $scope.Preferences;
            pref.dash_backup_refresh = DEFAULT_REFRESH;
            pref.dash_replication_refresh = DEFAULT_REFRESH;
            pref.dash_restore_refresh = DEFAULT_REFRESH;
            pref.dash_active_job_refresh = DEFAULT_REFRESH;
            pref.dash_alert_refresh = DEFAULT_REFRESH;
            pref.dash_storage_refresh = DEFAULT_REFRESH;
            pref.page_protect_refresh = DEFAULT_REFRESH;
            pref.page_restore_refresh = DEFAULT_REFRESH;
            pref.page_jobs_refresh = DEFAULT_REFRESH;
            pref.dash_recovery_assurance_refresh = DEFAULT_REFRESH;

            $url = '/portal/api/preferences.php?user=' + $scope.user.name;
            var promise = $http({
                method: 'GET',
                url: $url
            }).then(function(result) {
                // per the JSON returned, get the first array element.
                return result;
            }, function(result) {
                var data = result.data;
                window.PHD.throwError(data);
            });
            return promise;
        }
    };
});

app.directive('ngElementReady', [function() {
	return {
		priority: -1000,
		restrict: "A",
		link: function(scope, element, attributes) {
			scope.$eval(attributes.ngElementReady);
		}
	}
}]);
