angular.module('configure', ['angulartics', 'angulartics.google.analytics'])

	.controller('ConfigCtrl', ['$scope', '$http', 'Page', '$analytics', '$routeParams', '$timeout', 'cookiesvc',
				function($scope, $http, Page, $analytics, $routeParams, $timeout, cookiesvc) {
								
		Page.setTitle('Admin');

		$scope.tabs = [];
		$scope.currentTabIndex = 0;
		$scope.slugs = ["networked-assets", "customers", "users"];
		var DISTRIBUTED_ASSETS_TAB = 0;
		var CUSTOMERS_TAB = 1;
		var USERS_TAB = 2;

		$scope.networkedAppliance = null;
		$scope.selectedCustomer = null;
		if ($routeParams.id !== undefined) {
			// If route parameters are set, this was called with a pre-selected storage or distributed appliance object, depending on usage.
			if ($routeParams.usage === 'network') {
				$scope.networkedAppliance = {};
				$scope.networkedAppliance.id = $routeParams.id;
			} else if ($routeParams.usage === 'customer') {
				$scope.selectedCustomer = {};
                $scope.selectedCustomer.id = $routeParams.id;
			}
		}

		$scope.configPageLoaded = false;
		$scope.$on('$includeContentLoaded', function(event, url) {
			// Only process configure page once.  Otherwise, it is processed for EVERY included file.
			// We could look for first URL loaded, but in case it changes, instead we use a global flag to load once.
			if (!$scope.configPageLoaded) {
				$scope.configPageLoaded = true;
				processConfigurePage(PHD, window, $analytics);
			}
		});

		/*
		 * Called when the distributed appliance page is loaded.
		 */
		$scope.networkedAppliancesLoaded = function() {
			$scope.tabs[DISTRIBUTED_ASSETS_TAB] = new PHD.NetworkedApplianceTab(DISTRIBUTED_ASSETS_TAB, $("#config-networked-appliances")).init();
			if ($scope.currentTabIndex === DISTRIBUTED_ASSETS_TAB) {
				if ($scope.networkedAppliance !== null) {
					preSelectAppliance($scope.currentTabIndex);
                } else {
					activateTab($scope.currentTabIndex);
				}
			}
		};

		/*
		 * Called when the customers tab is loaded.
		 */
		$scope.customersLoaded = function() {
			$scope.tabs[CUSTOMERS_TAB] = new PHD.CustomersTab(CUSTOMERS_TAB, $("#config-customers")).init();
			if ($scope.currentTabIndex === CUSTOMERS_TAB) {
                if ($scope.networkedAppliance !== null) {
                    preSelectAppliance($scope.currentTabIndex);
                } else if ($scope.selectedCustomer !== null) {
                    preSelectCustomer($scope.currentTabIndex);
                } else {
                    activateTab($scope.currentTabIndex);
                }
			}
		};

		/*
		 * Called when the users tab is loaded.
		 */
		$scope.usersLoaded = function() {
			$scope.tabs[USERS_TAB] = new PHD.UsersTab(USERS_TAB, $("#config-users")).init();
			if ($scope.currentTabIndex === USERS_TAB) {
                if ($scope.networkedAppliance !== null) {
                    preSelectAppliance($scope.currentTabIndex);
                } else if ($scope.selectedCustomer !== null) {
                    preSelectCustomer($scope.currentTabIndex);
                } else {
                    activateTab($scope.currentTabIndex);
                }
			}
		};

		function preSelectAppliance(activeTab) {
			var selected = $scope.networkedAppliance;
			if (activeTab !== DISTRIBUTED_ASSETS_TAB) {
				$('#configure-page-tabs.tabs-wrapper').tabs("option", "active", DISTRIBUTED_ASSETS_TAB);
			} else {
				activateTab(activeTab);
			}
			$scope.networkedAppliance = null;
			$timeout(function () {
				$scope.tabs[DISTRIBUTED_ASSETS_TAB].queueSelection(selected);
			}, 1);
		}

        function preSelectCustomer(activeTab) {
            var selected = $scope.selectedCustomer;
            if (activeTab !== CUSTOMERS_TAB) {
                $('#configure-page-tabs.tabs-wrapper').tabs("option", "active", CUSTOMERS_TAB);
            } else {
                activateTab(activeTab);
            }
            $scope.selectedCustomer = null;
            $timeout(function () {
                $scope.tabs[CUSTOMERS_TAB].queueSelection(selected);
            }, 1);
        }

		function activateTab(activeTab) {
			$scope.tabs[activeTab].activate();
			$(document).trigger("tabchange", "config" + $scope.slugs[activeTab]);
		}

		// wrap existing jQuery functionality.
		function processConfigurePage(PHD, window, $analytics) {
			var $emailEditForm,
				$hvRemoveForm = $("#hypervisor-remove-form"),
				$applianceTable = $("#appliance-table"),
				hypervisorFormEdit = false,
				REFRESH_INTERVAL = 6000;
			var initialTab = cookiesvc.get("initial_tab_config") || 0;
			if (window.location.port !== "" && initialTab >= USERS_TAB) {
				initialTab = 0;
			}

			function initTabs() {
				$scope.tabs[DISTRIBUTED_ASSETS_TAB] = {};
				$scope.tabs[CUSTOMERS_TAB] = {};
				$scope.tabs[USERS_TAB] = {};
			}

			function initForm() {
				/**
				 * Return true if the field value matches the given format RegExp
				 *
				 * @example jQuery.validator.methods.pattern("AR1004",element,/^AR\d{4}$/)
				 * @result true
				 *
				 * @example jQuery.validator.methods.pattern("BR1004",element,/^AR\d{4}$/)
				 * @result false
				 *
				 * @name jQuery.validator.methods.pattern
				 * @type Boolean
				 * @cat Plugins/Validate/Methods
				 */
				$.validator.addMethod("pattern", function (value, element, param) {
					if (this.optional(element)) {
						return true;
					}
					if (typeof param === 'string') {
						param = new RegExp('^(?:' + param + ')$');
					}
					return param.test(value);
				}, gettext("Name cannot contain special characters or a space."));
			}

			/*
			 *   DOM Ready Function
			 */
			$(function () {
				// Initialize form validation and event handlers
				initForm();

				// Quick and dirty way to change what tab opens by default via URL
				if (!isNaN(PHD.util.getHashInt())) {
					initialTab = PHD.util.getHashInt();
				}

				// Initialize the tabs
				$('#configure-page-tabs.tabs-wrapper').tabs({
					active: initialTab,
					create: function (event, ui) {
						$scope.currentTabIndex = initialTab; // lmc - using angular cookie store, and loading once, was: ui.tab.index();
						initTabs();
					},
					activate: function (event, ui) {
						$(".loading").hide();
						$scope.currentTabIndex = ui.newTab.index();
						cookiesvc.put("initial_tab_config", $scope.currentTabIndex, {expires: 365, path: '/'});
						$scope.tabs[ui.oldTab.index()].deactivate();
                        $scope.tabs[$scope.currentTabIndex].activate && $scope.tabs[$scope.currentTabIndex].activate();
						$(document).trigger("tabchange", "config" + $scope.slugs[$scope.currentTabIndex]);
					}
				});
			});

		}

	}]
);