angular.module('credential-manager',['ngDialog'])
    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            showClose: true,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])

    .controller('credentialControl', ['$scope', '$rootScope', '$http','ngDialog', function($scope, $rootScope, $http, ngDialog) {

        var applianceSelect;
        var CredentialsTable = null;
        var allCredentials;
        var credentialRowData = null;
        var base_url = "/portal/api/credentials.php";
        var credentialEdit;
        $scope.selectedCredential = null;

        $scope.initCtrl = function(dlg){
            $scope.credential = {};
            switch (dlg.name){
                case 'Credential-Add':
                    credentialEdit = false;
                    break;
                case 'Credential-Edit':
                    credentialEdit = true;
                    $scope.editProcessing();
                    break;
                case 'Credential-Manager':
                    processDialog(PHD, window);
                    break;
                default:
                    // No action.
                    break;
            }

        };

        $scope.onConfirmError = function () {
            ngDialog.close('ErrorDialog');
            if (credentialEdit) {
                $scope.credential.name = credentialname;
                $scope.credential.username = username;
                $scope.credential.password = password;
                $scope.credential.domain = domain;
                if(isDefault){
                    $('#id_default').prop("checked", true);
                }
                else{
                    $('#id_default').prop("checked", false);
                }
            }
        };

        $scope.onConfirmWarning = function () {
            ngDialog.close('SelectCredential');
        };

        $scope.launchAddCredentials = function() {
            credentialEdit = false;
            ngDialog.open({
                template: 'app/configure/credentials/add-edit-credentials.html',
                scope: $scope,
                overlay:true,
                name: 'Credential-Add',
                ngDialogStyle:'width:500px; height:350px;',
                closeByDocument: false,
                closeByEscape: true,
                modelDialogId:'add-credentials-dialog'
            });
        };

        $scope.launchCredentialEditor = function() {
            if (typeof id !== "undefined" && id !== null && $scope.selectedCredential){
                ngDialog.open({
                    template: 'app/configure/credentials/add-edit-credentials.html',
                    name: 'Credential-Edit',
                    scope: $scope,
                    overlay:true,
                    ngDialogStyle:'width:500px; height:350px;',
                    modelDialogId : 'edit-credentials-dialog',
                    closeByDocument: false,
                    closeByEscape: true,
                    showClose: false
                });
            } else {
                ngDialog.open({
                    dialogType:'Warning',
                    modelDialogId:'SelectCredential',
                    dialogMessage: 'Select a credential to edit.',
                    scope:$scope,
                    onConfirmOkButtonClick:'onConfirmWarning()'
                });
            }
        };

        $scope.editProcessing = function(){
            $("#add-edit-credentials_dialog_title").text("Edit Credential");
            $scope.credential.name = credentialname;
            $scope.credential.username = username;
            $scope.credential.password = password;
            $("#id_domain").val(domain);
            if(isDefault){
                $('#id_default').prop("checked", true);
            }
            else{
                $('#id_default').prop("checked", false);
            }
            if (self_serve_creds) {
                $('#id_display_name').prop("disabled", true);
                $('#id_username').prop("disabled", true);
                $('#id_domain').prop("disabled", true);
                $('#id_default').prop("disabled", true);
            } else {
                $('#id_display_name').prop("disabled", false);
                $('#id_username').prop("disabled", false);
                $('#id_domain').prop("disabled", false);
                $('#id_default').prop("disabled", false);
            }
        };

        $scope.deleteCredential = function() {
            var dialogMsg = "Are you sure you want to delete the selected credential?";
            if ($scope.selectedCredential) {
                if (self_serve_creds) {
                    dialogMsg += "\n\n Self-service information cannot be obtained from the target system, if the selected credentials are deleted."
                }
                ngDialog.open({
                    dialogType: 'Confirmation',
                    dialogMessage: dialogMsg,
                    scope: $scope,
                    overlay:true,
                    modelDialogId:'DeleteCredentialConfirmation',
                    onConfirmOkButtonClick:'onConfirmDelete()'
                });
            } else {
                ngDialog.open({
                    modelDialogId:'NoCredentialSelected',
                    dialogType: 'Information',
                    dialogMessage: "No credential has been selected for deletion.",
                    scope: $scope,
                    overlay:true,
                    onConfirmOkButtonClick:'onConfirmNoneSelected()'
                });
            }
        };

        $scope.onConfirmNoneSelected = function() {
            ngDialog.close('NoCredentialSelected');
        };

        $scope.onConfirmDelete = function() {
            console.log("Delete Credential Confirmation") ;
            ngDialog.close('DeleteCredentialConfirmation');
            var url = "/portal/api/credentials.php?id=";
            console.log("Credential ID being deleted : " + $scope.selectedCredential.id);
            url += $scope.selectedCredential.id;

            $http({
                method: "DELETE",
                url: url
            }).then(function(result) {
                refreshCredentialList();
            }, function(result) {
                var data = result.data;
                var message = data.message;
                ngDialog.open({
                    dialogType: 'ERROR',
                    dialogMessage: message,
                });
            });
        };

        $scope.addCredentials = function() {
            console.log("Adding credentials..");
            saveCredentials();
        };

        function refreshCredentialList() {
            var resp = PHD.Ajax.get(base_url + "?devices=1");
            resp.done(function(data){
                var credentialList = data.credentials;
                credentialList.forEach(function(item, index) {
                    item.id = item.credential_id;
                    item.system_name = item.hosts;
                    if(item.is_default) {
                        item.display_name = item.display_name + " (default)";
                    }
                    // Set the name to empty if an unnamed credential.
                    if (item.name === "(Unnamed)") {
                        item.name = "";
                    }
                });
                allCredentials = credentialList;
                if (CredentialsTable) {
                    if (!$scope.credentialTableLoaded) {
                        CredentialsTable.load(allCredentials);
                        $scope.credentialTableLoaded = true;
                    } else {
                        CredentialsTable.update(allCredentials, true);
                    }
                }
            });
        }

        function processDialog(PHD, window) {

            $scope.credentialTableLoaded = false;

            var $el = $("#credentials-dialog");
            var $credentialsTable;

            var dtOptions = {
                multiselect: false,
                sortable: true,
                sortOnHeader: true,
                sortableCells: [true, true, true, true],
                sortOnLoad: true,
                sortingType : "string",
                cellClasses: ["long-text truncate", "long-text truncate", null, null, null, null],
                placeholder: gettext("There are no credentials")
            };

            if (CredentialsTable === null) {
                $credentialsTable = $el.find(".credentials-table");
                CredentialsTable = PHD.DataTable($credentialsTable, dtOptions);

                refreshCredentialList();

                $credentialsTable.on("rowselect", "tr", function (event, dbid, $el, DataTable, DataRow) {
                    credentialRowData = DataRow.data;
                    id = credentialRowData.id;
                    isDefault = credentialRowData.is_default;
                    credentialname = credentialRowData.display_name;
                    username = credentialRowData.username;
                    password = credentialRowData.password;
                    domain = credentialRowData.domain;
                    $scope.selectedCredential = credentialRowData;
                    self_serve_creds = false;
                    if (credentialRowData.self_service !== undefined && credentialRowData.self_service !== null) {
                        self_serve_creds = true;
                    }
                });

                $credentialsTable.on("loadfinished updatefinished", function () {
                    $credentialsTable.scrollTable({"resize": false});
                    $credentialsTable.sortTable();
                });
            } else {
                refreshCredentialList();
            }

        }

        function saveCredentials() {
            var obj = {};
            var display_name = $("#id_display_name").val();
            if (display_name !== "") {
                if(display_name.indexOf("(default)") > -1){
                    var name = display_name;
                    name = name.replace('(default)','');
                    obj.display_name = name;
                }
                else{
                    obj.display_name = display_name;
                }
            }
            obj.username = $("#id_username").val();
            var password = $("#id_password").val();
            if (password !== "") {
                obj.password = password;
            }
            var domain = $("#id_domain").val();
            obj.domain = domain;
            obj.is_default = $("#id_default").prop("checked");

            var url = "/portal/api/credentials.php";
            if (credentialEdit) {
                url += "?id=" + id;
                obj = JSON.stringify(obj);
            } else {
                obj = JSON.stringify(obj);
            }
            var method = credentialEdit ? 'PUT' : 'POST';
            $http({
                method: method,
                url: url,
                data: obj
            }).then(function(result) {
                refreshCredentialList();
                if(credentialEdit){
                    ngDialog.close('edit-credentials-dialog');
                } else{
                    ngDialog.close('add-credentials-dialog');
                }
            }, function(result) {
                var data = result.data;
                var message = data.message;
                ngDialog.open({
                    dialogType: 'Error',
                    modelDialogId: 'ErrorDialog',
                    scope: $scope,
                    dialogMessage: message,
                    onConfirmOkButtonClick: 'onConfirmError()'
                });
            });
        }

        $scope.$on('ngDialog.opened', function(event,obj) { $scope.initCtrl(obj) });

    }]);
