angular.module('configNetworkedAppliances', ['ngDialog', 'angulartics', 'angulartics.google.analytics',
    'ui.grid', 'ui.grid.resizeColumns', 'ui.grid.autoResize', 'ui.grid.treeView', 'ui.grid.selection'])

    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .controller('NetworkedAppliancesCtrl', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics', '$timeout', 'connectionsvc',
        function($scope, $rootScope, $http, ngDialog, wizardService, $analytics, $timeout, connectionsvc) {

            var $globalScope = $("[ng-controller=GlobalCtrl]").scope();
            $scope.user.monitor = $globalScope.user.monitor;

            $scope.selectedAppliance = null;
            $scope.savedAppliance = null;  // save appliance to restore on cancel

            // Tunnel information.
            $scope.tunnel = 0;
            $scope.managerAddress = "";
            $scope.connectAfterManage = false;

            $scope.authExceptionMessage = "An exception was seen during authentication.  This is likely due to security of the appliance's SSL certificate.  " +
                "<br><br>To fix, click on Manage, then click on the URL to open a tab to the appliance.  You will need to " +
                "confirm the security exception in your browser.  Then, please click on Connect again.";

            var DEM_KB_URL = KB_URL + "2030";
            var DEM_URL_TITLE = 'Click for more information on DEM configuration';

            $scope.initCtrl = function () {
                processAppliances(PHD, window, $analytics);
            };

            $scope.loadDevices = function() {
                $http({
                    method: 'GET',
                    url: '/portal/api/managers.php/?level=devices'
                }).then(function(result) {
                    var data = result.data;
                    $scope.deviceGridOptions.data = data['managers'];
                    // Set the height of the table once loaded.
                    $timeout(function() {
                        $scope.setTableHeight();
                    }, 0);
                }, function(result) {
                    $scope.deviceGridOptions.data = [];
                    var data = result.data;
                    var message = "Error viewing devices; please check manager logs for more information.";
                    if (data !== undefined && data.message !== undefined) {
                        message = data.message;
                    }
                    ngDialog.open({
                        dialogType:'Error',
                        dialogMessage:message
                    })
                });
            };

            function processAppliances(PHD, window, $analytics) {

                var NetworkedAppliancesTab = function(index, $panel) {
                    this.index = index;
                    this.$panel = $panel;
                    this.$applianceArea = this.$panel.children("#appliances-list");
                    this.environmentList = {};
                    this.environments = {};
                    this.activeEnvironment = null;
                    this.isActive = false;
                    this.isBootstrapped = false;
                };

                NetworkedAppliancesTab.prototype.init = function() {
                    var self = this;
                    return this;
                };

                NetworkedAppliancesTab.prototype.activate = function() {
                    this.isActive = true;
                    $scope.loadDevices();
                    return this;
                };

                NetworkedAppliancesTab.prototype.deactivate = function() {
                    this.isActive = false;
                    return this;
                };

                NetworkedAppliancesTab.prototype.queueSelection = function (selected) {
                    var self = this;
                    var dbid = selected.id;
                    var rowIndex = $scope.findRow(dbid);
                    if (rowIndex >= 0) {
                        //$scope.gridApi.grid.modifyRows($scope.deviceGridOptions.data);
                        $scope.gridApi.selection.selectRow($scope.deviceGridOptions.data[rowIndex]);
                    }
                };

                PHD.NetworkedApplianceTab = NetworkedAppliancesTab;
            }

            var nameTemplate = '<div class="ui-grid-cell-contents" title="{{row.entity.hostname}}" ng-style="{ \'margin-left\': (row.treeLevel * 20 + 2) + \'px\' }"><i style="margin-right:4px" class="{{grid.appScope.getIcon(row.entity)}} icon"></i>{{row.entity.hostname}}</div>';

            $scope.handleRowSelection = function(row) {
                if (row.isSelected){
                    $scope.selectedAppliance = row.entity;
                } else {
                    $scope.selectedAppliance = null;
                }
            };

            $scope.deviceGridOptions = {
                enableSorting: true,
                enableFiltering: true,
                enableGridMenu: true,
                //showTreeExpandNoChildren: false,
               // treeRowHeaderBaseWidth: 20,
                enableRowSelection : false,
                enableRowHeaderSelection: true,
                selectionRowHeaderWidth: 28,
                rowHeight: 28,
                enableColumnResizing: true,
                enableHorizontalScrollbar: 0,
                multiSelect: false,
                columnDefs: [
                    {
                        name: 'manager',
                        displayName: 'CUSTOMER',
                        width: '20%',
                        cellTooltip: true
                    },
                    {
                        name: 'location',
                        displayName: 'LOCATION',
                        width: '20%',
                        cellTooltip: true
                    },
                    {
                        name: 'hostname',
                        displayName: 'HOSTNAME',
                        width: '20%',
                        cellTooltip: true
                    },
                    {
                        name: 'credential_name',
                        displayName: 'CREDENTIALS',
                        visible: true,
                        cellTooltip: true
                    },
                    {
                        name: 'version',
                        displayName: 'VERSION',
                        width: '*',
                        cellTooltip: true
                    },
                    {
                        name: 'uuid',
                        displayName: 'ASSET TAG',
                        visible: false,
                        cellTooltip: true
                    }
                ],
                data: [],
                onRegisterApi: function( gridApi ) {
                    $scope.gridApi = gridApi;
                    /*$scope.gridApi.treeBase.on.rowExpanded($scope, function(row) {
                        console.log('expand');
                    });*/
                    $scope.gridApi.selection.on.rowSelectionChanged($scope, $scope.handleRowSelection);
                    $scope.gridApi.selection.on.rowSelectionChangedBatch($scope, $scope.handleRowSelection);
                }
            };

            $scope.findRow = function(id) {
                var index = -1;
                for (var i = 0; i < $scope.deviceGridOptions.data.length; i++) {
                    var obj = $scope.deviceGridOptions.data[i];
                    if (obj.device_id === id) {
                        index = i;
                        break;
                    }
                }
                return index;
            };

            $scope.editAppliance = function(mode) {
                if (!$scope.user.monitor) {
                    $scope.isEdit = mode === 1;
                    if ($scope.isEdit) {
                        if ($scope.selectedAppliance === null) {
                            return;
                        }
                        $scope.savedAppliance = $scope.selectedAppliance;
                        $scope.selectedAppliance = jQuery.extend({}, $scope.selectedAppliance);
                    } else {
                        $scope.savedAppliance = $scope.selectedAppliance;
                        $scope.selectedAppliance = {};
                    }
                    $scope.applianceTitle = ($scope.isEdit ? "Edit" : "Add") + " Appliance";
                    var load = PHD.showLoadingIndicator("body", true, "Loading...");
                    $http({
                        method: 'GET',
                        url: '/portal/api/managers.php'
                    }).then(function (result) {
                        var data = result.data;
                        $scope.managersList = data['managers'];
                        if ($scope.selectedAppliance !== undefined && $scope.selectedAppliance.manager_id !== undefined) {
                            $scope.onManagerChange($scope.selectedAppliance.manager_id, true);
                        }
                        $http({
                            method: 'GET',
                            url: '/portal/api/credentials.php'
                        }).then(function (result) {
                            var data = result.data;
                            PHD.hideLoadingIndicator(load);
                            var credentialArray = [{ "credential_id": "0", "display_name": "None" }];
                            $scope.credentialsList = credentialArray.concat(data['credentials']);
                            ngDialog.open({
                                template: 'app/configure/networked-appliances/appliance-edit.html',
                                scope: $scope,
                                overlay: true,
                                modelDialogId: 'Device-Manager',
                                name: 'Device-Manager',
                                ngDialogStyle: 'width:600px; height:350px;',
                                closeByDocument: false,
                                closeByEscape: true,
                                preCloseCallback: function (value) {
                                }
                            });
                        }, function (result) {
                            PHD.hideLoadingIndicator(load);
                            var data = result.data;
                            var message = data.message;
                            ngDialog.open({
                                dialogType: 'Error',
                                modelDialogId: 'device-error-dialog',
                                scope: $scope,
                                helpArticleLink: DEM_KB_URL,
                                helpArticleTitle: DEM_URL_TITLE,
                                dialogMessage: message,
                                onConfirmOkButtonClick: 'onConfirmError()'
                            });
                        });
                    }, function (result) {
                        PHD.hideLoadingIndicator(load);
                        var data = result.data;
                        var message = data.message;
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'device-error-dialog',
                            scope: $scope,
                            helpArticleLink: DEM_KB_URL,
                            helpArticleTitle: DEM_URL_TITLE,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.onManagerChange = function(manager, initializeValues) {
                var initializeSelected = initializeValues || false;

                $http({
                    method: 'GET',
                    url: '/portal/api/managers.php?level=locations&id=' + manager
                }).then(function(result) {
                    var data = result.data;
                    $scope.locationsList = [];
                    if (data['managers'] !== undefined && data['managers'].length > 0) {
                        var manager = data['managers'][0];
                        if (manager['locations'] !== undefined) {
                            $scope.locationsList = manager['locations'];
                        }
                        // If true, initialize the selectedAppliance manager and location IDs.
                        if (initializeSelected) {
                            $scope.selectedAppliance.manager_id = $scope.managersList[0].id;
                            if ($scope.locationsList.length > 0) {
                                $scope.selectedAppliance.location_id = $scope.locationsList[0].location_id;
                            }
                        }
                    }
                }, function(result) {
                    var data = result.data;
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'device-error-dialog',
                        scope: $scope,
                        helpArticleLink: DEM_KB_URL,
                        helpArticleTitle: DEM_URL_TITLE,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });
            };

            $scope.saveAppliance = function() {
                if ($scope.isEdit) {
                    $scope.updateAppliance();
                } else {
                    $scope.addAppliance();
                }
            };

            /*
             * Update appliance fields.  Ensure if connected to an appliance the user is not changing credentials.
             */
            $scope.updateAppliance = function() {
                if (connectionsvc.isHostConnected($scope.selectedAppliance.hostname) &&
                    $scope.savedAppliance.credential_id !== $scope.selectedAppliance.credential_id) {
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'device-error-dialog',
                        scope: $scope,
                        dialogMessage: "Cannot change credentials while directly connected to this appliance.  Please disconnect before changing appliance credentials.",
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                } else {
                    var url = "/portal/api/devices.php?id=" + $scope.selectedAppliance['device_id'];
                    var load = PHD.showLoadingIndicator("body", true);
                    var payload = {};
                    payload.location_id = $scope.selectedAppliance.location_id;
                    payload.credential_id = $scope.selectedAppliance.credential_id;
                    payload = JSON.stringify(payload);
                    var method = 'PUT';
                    $http({
                        method: method,
                        url: url,
                        data: payload
                    }).then(function(result) {
                        $scope.closeEditDialog();
                        $scope.loadDevices();
                        PHD.hideLoadingIndicator(load);
                    }, function(result) {
                        var data = result.data;
                        PHD.hideLoadingIndicator(load);
                        var message = data.message;
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'device-error-dialog',
                            scope: $scope,
                            helpArticleLink: DEM_KB_URL,
                            helpArticleTitle: DEM_URL_TITLE,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.addAppliance = function() {
                if (!$scope.user.monitor) {
                    // set devtype to an even number when adding (to manage it by default)
                    var url = "/portal/api/devices.php?devtype=2";
                    var load = PHD.showLoadingIndicator("body", true);
                    var payload = $scope.selectedAppliance;
                    payload.asset_tag = payload.uuid;
                    payload = JSON.stringify(payload);
                    var method = 'POST';
                    $http({
                        method: method,
                        url: url,
                        data: payload
                    }).then(function (result) {
                        $scope.closeEditDialog();
                        $scope.loadDevices();
                        PHD.hideLoadingIndicator(load);
                    }, function (result) {
                        var data = result.data;
                        PHD.hideLoadingIndicator(load);
                        var message = data.message;
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'device-error-dialog',
                            scope: $scope,
                            helpArticleLink: DEM_KB_URL,
                            helpArticleTitle: DEM_URL_TITLE,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.onConfirmError = function() {
                ngDialog.close('device-error-dialog');
            };

            $scope.deleteAppliance = function() {
                if (!$scope.user.monitor) {
                    if ($scope.selectedAppliance) {
                        if ($scope.selectedAppliance.hostname !== $rootScope.managedAppliance.hostname) {
                            $scope.deleteTitle = 'Confirm Appliance Removal';
                            $scope.confirmMessage1 = 'Are you sure want to delete appliance ' + $scope.selectedAppliance.hostname + '?';
                            $scope.confirmMessage2 = 'When an appliance is removed, its status is deleted and if open, the management connection is closed.';
                            $scope.confirmMessage3 = 'I understand that appliance removal will delete status and close the management connection';
                            $scope.submitButtonText = 'Remove Appliance';

                            ngDialog.open({
                                template: 'app/common/confirmationDialog.html',
                                scope: $scope,
                                overlay: true,
                                ngDialogStyle: 'width:520px;height:250px',
                                closeByDocument: false,
                                closeByEscape: true
                            });
                        } else {
                            var message = "You are currently directly connected to this appliance.  First disconnect, then you can delete it.";
                            ngDialog.open({
                                dialogType: 'Error',
                                modelDialogId: 'device-error-dialog',
                                scope: $scope,
                                dialogMessage: message,
                                onConfirmOkButtonClick: 'onConfirmError()'
                            });
                        }
                    } else {
                        ngDialog.open({
                            dialogType: 'Warning',
                            dialogMessage: 'You must select one and only one appliance at a time for deletion.'
                        });
                    }
                }
            };

            /*
             * Delete the specified appliance, after checking to see if a tunnel is open.
             * If a tunnel is open, close it, then delete.
             */
            $scope.deleteAsset = function() {
                // close tunnel if open.
                var method = 'PUT';
                var url = "/portal/api/tunnel.php?id=";
                var load = PHD.showLoadingIndicator("body", true, "Closing any management connections to this appliance.  Please wait....");
                url += $scope.selectedAppliance.device_id + "&cmd=close-tunnel&parameter=" + $scope.tunnel;
                var tunnelWasOpen = false;
                $http({
                    method: method,
                    url: url
                }).then(function(result) {
                    PHD.hideLoadingIndicator(load);
                    $scope.tunnel = 0;
                    $scope.local_tunnel = 0;
                    $scope.managerAddress = "";
                    tunnelWasOpen = true;
                    $scope.doDelete(tunnelWasOpen);
                }, function(result) {
                    PHD.hideLoadingIndicator(load);
                    $scope.doDelete(tunnelWasOpen);
                });
            };

            $scope.doDelete = function(tunnelWasOpen) {
                var url = '/portal/api/devices.php?id='+$scope.selectedAppliance.device_id;
                var load = PHD.showLoadingIndicator("body", true);
                var method = 'DELETE';

                $http({
                    method: method,
                    url : url
                }).then(function(result){
                    $scope.loadDevices();
                    PHD.hideLoadingIndicator(load);
                    var message = $scope.selectedAppliance.hostname + " removed successfully.";
                    if (tunnelWasOpen) {
                        message += "  A management connection was closed.  Please close any browser tab that may have been open to " + $scope.selectedAppliance.hostname + ".";
                    }
                    ngDialog.open({
                        dialogType:'information',
                        dialogMessage: message
                    });
                    $scope.selectedAppliance = null;
                }, function(result) {
                    var data = result.data;
                    PHD.hideLoadingIndicator(load);
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'device-error-dialog',
                        scope: $scope,
                        helpArticleLink: DEM_KB_URL,
                        helpArticleTitle: DEM_URL_TITLE,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });

            };

            $scope.manageAppliance = function() {
                if ($scope.selectedAppliance && $scope.selectedAppliance.activity !== DEVICE_ACTIVITY_MANAGE) {
                    var notManaged = "This appliance is not being managed by the portal.  To fix, change the activity in the configuration file ";
                    notManaged += "on the remote appliance to \"manage\" for this host.";
                    ngDialog.open({
                        dialogType: 'Warning',
                        dialogMessage: notManaged,
                        modelDialogId: 'device-error-dialog',
                        scope: $scope,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                    return;
                }
                if (!$scope.user.monitor) {
                    var method = 'PUT';
                    var url = "/portal/api/tunnel.php?id=";
                    var load = PHD.showLoadingIndicator("body", true, "Contacting distributed appliance.  Please wait....");
                    url += $scope.selectedAppliance.device_id + "&cmd=check-tunnel";
                    $http({
                        method: method,
                        url: url
                    }).then(function (result) {
                        var data = result.data;
                        PHD.hideLoadingIndicator(load);
                        $scope.tunnel = data['tunnel'];
                        $scope.managerAddress = data['host'];
                        if ($scope.tunnel === 0) {
                            // No tunnel is open.  Open one.
                            ngDialog.open({
                                template: 'app/configure/networked-appliances/confirm-manage-appliance.html',
                                scope: $scope,
                                overlay: true,
                                modelDialogId: 'confirm-manage-appliance',
                                name: 'Manage-Appliance',
                                ngDialogStyle: 'width:500px; height:210px;',
                                closeByDocument: false,
                                closeByEscape: true,
                                preCloseCallback: function (value) {
                                }
                            });
                        } else {
                            $scope.local_tunnel = data['localTunnel'] !== undefined ? data['localTunnel'] : $scope.tunnel + 1;
                            if ($scope.local_tunnel !== $scope.tunnel + 1) {
                                // A tunnel is open.  Explain how to use or close it.
                                $scope.local_tunnel = data['localTunnel'];
                                var applianceAddress = $scope.managerAddress + ':' + $scope.local_tunnel;
                                if ($scope.connectAfterManage) {
                                    $scope.connectAfterManage = false;
                                    $scope.connectToAppliance($scope.selectedAppliance.hostname, applianceAddress);
                                } else {
                                    ngDialog.open({
                                        template: 'app/configure/networked-appliances/tunnel-open-close.html',
                                        scope: $scope,
                                        overlay: true,
                                        name: 'close-tunnel',
                                        ngDialogStyle: 'width:500px; height:200px;',
                                        closeByDocument: false,
                                        closeByEscape: true
                                    });
                                }
                            } else {
                                // A tunnel is open on the remote, but the local tunnel is bad.
                                ngDialog.open({
                                    template: 'app/configure/networked-appliances/tunnel-no-local.html',
                                    scope: $scope,
                                    overlay: true,
                                    name: 'close-tunnel',
                                    ngDialogStyle: 'width:500px; height:200px;',
                                    closeByDocument: false,
                                    closeByEscape: true
                                });
                            }
                        }
                    }, function (result) {
                        var data = result.data;
                        PHD.hideLoadingIndicator(load);
                        var message = data.message;
                        if (message instanceof Array) {
                            message = message.toString();
                        }
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'device-error-dialog',
                            scope: $scope,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.onConfirmCloseTunnel = function() {
                if ($scope.selectedAppliance) {
                    var method = 'PUT';
                    var url = "/portal/api/tunnel.php?id=";
                    url += $scope.selectedAppliance.device_id + "&cmd=close-tunnel&parameter=" + $scope.tunnel;
                    $http({
                        method: method,
                        url: url
                    }).then(function(result) {
                        ngDialog.open({
                            dialogType: 'Information',
                            dialogMessage: "The management connection has been closed.",
                            preCloseCallback: function (value) {
                                $scope.tunnel = 0;
                                $scope.local_tunnel = 0;
                                $scope.managerAddress = "";
                            }
                        });
                    }, function(result) {
                        var data = result.data;
                        var message = data.message;
                        if (message instanceof Array) {
                            message = message.toString();
                        }
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'device-error-dialog',
                            scope: $scope,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.connectToAppliance = function(hostname, address) {
                if ($rootScope.AuthToken === undefined || $rootScope.AuthToken === '') {
                    if ($scope.selectedAppliance.credential_id !== "0") {
                        $scope.connectUsingCredentials($scope.selectedAppliance, hostname, address);
                    } else {
                        ngDialog.open({
                            template: 'app/configure/networked-appliances/connection-credentials.html',
                            scope: $scope,
                            data: {'connectionObject': { 'hostname': hostname, 'address': address }},
                            overlay:true,
                            modelDialogId: 'connected-appliance-dialog',
                            name: 'connected-appliance-dialog',
                            ngDialogStyle:'width:500px;height:260px;',
                            closeByDocument: false,
                            closeByEscape: true
                        });
                    }
                } else {
                    var message = "You are already directly connected to an appliance.  First disconnect, then you can connect to a different appliance for direct access.";
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'device-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                }
            };

            $scope.connectUsingCredentials = function(appliance, hostname, address) {
                var load = PHD.showLoadingIndicator("body", true, "Connecting to Appliance....");
                $http({
                    method: 'GET',
                    url: '/portal/api/credentials.php?id=' + appliance.credential_id + '&pw=1'
                }).then(function (result) {
                    var data = result.data;
                    var credentialArray = data['credentials'];
                    var thisCredential = credentialArray[0];
                    var username = thisCredential.username;
                    var password = thisCredential.password;

                    connectionsvc.connect({'username': username, 'password': password}, {'hostname': hostname, 'address': address}).then(function(result) {
                        PHD.hideLoadingIndicator(load);
                        if (result.error !== undefined) {
                            var data = result.data;
                            var message = "Unknown appliance error";
                            if (result.status === -1 && data === null) {
                                message = $scope.authExceptionMessage;
                            } else {
                                message = data.message;
                            }
                            ngDialog.open({
                                dialogType: 'Error',
                                modelDialogId: 'credentials-error-dialog',
                                dialogMessage: message,
                                scope: $scope,
                                onConfirmOkButtonClick: 'closeErrorDialog()'
                            });
                        }
                    });
                }, function (result) {
                    PHD.hideLoadingIndicator(load);
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'device-error-dialog',
                        scope: $scope,
                        dialogMessage: "Error getting credential associated with appliance.",
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });
            };

            $scope.closeErrorDialog = function() {
                ngDialog.close('credentials-error-dialog');
            };

            $scope.disconnectFromAppliance = function() {
                connectionsvc.disconnect();
            };

            $scope.directlyManageAppliance = function() {
                if ($rootScope.managedAppliance.AuthToken === '') {
                    $scope.connectAfterManage = true;
                    $scope.manageAppliance();
                } else {
                    $scope.disconnectFromAppliance();
                }
            };

            $scope.createManagementConnection = function() {

                ngDialog.close('confirm-manage-appliance');

                if ($scope.selectedAppliance) {
                    var method = 'PUT';
                    var url = "/portal/api/tunnel.php?id=";
                    url += $scope.selectedAppliance.device_id + "&cmd=open-tunnel";
                    var load = PHD.showLoadingIndicator("body", true, "Opening connection to appliance....");
                    $http({
                        method: method,
                        url: url
                    }).then(function(result) {
                        var data = result.data;
                        PHD.hideLoadingIndicator(load);
                        $scope.tunnel = data['tunnel'];
                        $scope.local_tunnel = data['host_tunnel'];
                        $scope.managerAddress = data['host'];
                        var applianceAddress = $scope.managerAddress + ':' + $scope.local_tunnel;
                        if ($scope.connectAfterManage) {
                            $scope.connectAfterManage = false;
                            $scope.connectToAppliance($scope.selectedAppliance.hostname, applianceAddress);
                        } else {
                            ngDialog.open({
                                template: 'app/configure/networked-appliances/tunnel-open-success.html',
                                scope: $scope,
                                overlay:true,
                                name: 'Manage-Appliance',
                                ngDialogStyle:'width:500px; height:200px;',
                                closeByDocument: false,
                                closeByEscape: true
                            });
                        }
                    }, function(result) {
                        var data = result.data;
                        PHD.hideLoadingIndicator(load);
                        var message = data.message;
                        if (message instanceof Array) {
                            message = message.toString();
                        }
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'device-error-dialog',
                            scope: $scope,
                            helpArticleLink: DEM_KB_URL,
                            helpArticleTitle: DEM_URL_TITLE,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.closeEditDialog = function() {
                $scope.selectedAppliance = $scope.savedAppliance;
                ngDialog.close('Device-Manager');
            };

            $scope.launchPortalConfiguration = function(followFunction) {
                if (!$scope.user.monitor) {
                    ngDialog.open({
                        template: 'app/configure/networked-appliances/options/portal-settings.html',
                        scope: $scope,
                        overlay: true,
                        name: 'Settings-Manager',
                        ngDialogStyle: 'width:700px; height:550px;',
                        closeByDocument: false,
                        closeByEscape: true,
                        preCloseCallback: function (value) {
                            if (followFunction !== undefined) {
                                followFunction();
                            }
                        }
                    });
                }
            };

            $scope.launchCredentialManager = function() {
                ngDialog.open({
                    template: 'app/configure/credentials/credentials.html',
                    scope: $scope,
                    overlay:true,
                    name: 'Credential-Manager',
                    ngDialogStyle:'width:600px; height:410px;',
                    closeByDocument: false,
                    closeByEscape: false,
                    preCloseCallback: function (value) {
                    }
                });
            };

            $scope.setTableHeight = function() {
                var containerHeight = angular.element(document.getElementById('networked-appliance-grid-section')).height();
                // leave some room between the table and the window bottom.
                var gridHeight = containerHeight - 32;
                angular.element(document.getElementById('networkedApplianceGrid')).css('height', gridHeight + 'px');
            };


        }])
    .controller('ConnectionCredentialsCtrl', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics', '$timeout', 'connectionsvc',
        function($scope, $rootScope, $http, ngDialog, wizardService, $analytics, $timeout, connectionsvc) {

            // Used to unbind the watched event on dialog close.
            var unbindHandler = null;

            $scope.appliance = {username: '', password: ''};
            $scope.connectionObject = $scope.ngDialogData.connectionObject;

            $scope.connectCredentials = function() {
                connectionsvc.connect($scope.appliance, $scope.connectionObject).then(function(result) {
                    if (result.error !== undefined) {
                        var data = result.data;
                        var message = "Unknown appliance error";
                        if (result.status === -1 && data === null) {
                            message = $scope.authExceptionMessage;
                        } else {
                            message = data.result[0].message;
                        }
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'credentials-error-dialog',
                            dialogMessage: message,
                            scope: $scope,
                            onConfirmOkButtonClick: 'closeErrorDialog()'
                        });
                    } else {
                        $scope.closeDialog();
                    }
                });

            };

            $scope.closeDialog = function(){
                if ( _.isFunction(unbindHandler)) {
                    unbindHandler();
                }
                ngDialog.close('connected-appliance-dialog');
            };

            $scope.showToolTips = function(form){
                if (form !== undefined) {
                    form.find(".definition").popover({
                        appendTo: form
                    });
                }
            };

            // Initialize the controller, returning the function to turn off the listener.
            unbindHandler = $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

            $scope.initCtrl = function(dlg){
                switch (dlg.name){
                    // When putting an ngDialog above a UI Dialog, we need to adjust the vertical alignment (the default ngDialog top is 25%).
                    case 'connected-appliance-dialog':
                        var $form= $("#credentialsForm");
                        $scope.showToolTips($form);
                        break;
                    default:
                        // No action.
                        break;
                }
            };

        }]);
