angular.module('configUsers', ['ngDialog', 'angulartics', 'angulartics.google.analytics',
    'ui.grid', 'ui.grid.resizeColumns', 'ui.grid.autoResize', 'ui.grid.treeView', 'ui.grid.selection'])

    .config(['ngDialogProvider', function (ngDialogProvider) {
        ngDialogProvider.setDefaults({
            plain: false,
            closeByDocument: true,
            closeByEscape: true,
            appendTo: false,
            preCloseCallback: function (value) {

            }
        });
    }])
    .filter('privilegeFilter', function () {
        return function (value) {
            var name;
            switch (value) {
                case "0":
                    name = "Monitor";
                    break;
                case "1":
                    name = "Manager";
                    break;
                case "2":
                    name = "Administrator";
                    break;
                case "3":
                    name = "Super-user";
                    break;
            }
            return name;
        };
    })
    .controller('UsersCtrl', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics', '$timeout',
        function($scope, $rootScope, $http, ngDialog, wizardService, $analytics, $timeout) {

            var $globalScope = $("[ng-controller=GlobalCtrl]").scope();
            $scope.user.monitor = $globalScope.user.monitor;

            $scope.selectedUser = null;
            $scope.savedUser = null;  // save user to restore on cancel

            $scope.initCtrl = function () {
                processUsers(PHD, window, $analytics);
            };

            $scope.loadUsers = function() {
                $http({
                    method: 'GET',
                    url: '/portal/api/users.php'
                }).then(function(result) {
                    var data = result.data;
                    $scope.userGridOptions.data = data['users'];
                    // Set the height of the table once loaded.
                    $timeout(function() {
                        $scope.setTableHeight();
                    }, 0);
                }, function(result) {
                    console.log('no users');
                    $scope.userGridOptions.data = [];
                });
            };

            function processUsers(PHD, window, $analytics) {

                var UsersTab = function(index, $panel) {
                    this.index = index;
                    this.$panel = $panel;
                    this.$applianceArea = this.$panel.children("#appliances-list");
                    this.environmentList = {};
                    this.environments = {};
                    this.activeEnvironment = null;
                    this.isActive = false;
                    this.isBootstrapped = false;
                };

                UsersTab.prototype.init = function() {
                    var self = this;
                    return this;
                };

                UsersTab.prototype.activate = function() {
                    this.isActive = true;
                    $scope.loadUsers();
                    return this;
                };

                UsersTab.prototype.deactivate = function() {
                    this.isActive = false;
                    return this;
                };

                PHD.UsersTab = UsersTab;
            }

            var nameTemplate = '<div class="ui-grid-cell-contents" title="{{row.entity.hostname}}" ng-style="{ \'margin-left\': (row.treeLevel * 20 + 2) + \'px\' }"><i style="margin-right:4px" class="{{grid.appScope.getIcon(row.entity)}} icon"></i>{{row.entity.hostname}}</div>';

            $scope.handleRowSelection = function(row) {
                if (row.isSelected){
                    $scope.selectedUser = row.entity;
                } else {
                    $scope.selectedUser = null;
                }
            };

            $scope.userGridOptions = {
                enableSorting: true,
                enableFiltering: true,
                //showTreeExpandNoChildren: false,
               // treeRowHeaderBaseWidth: 20,
                enableRowSelection : false,
                enableRowHeaderSelection: true,
                selectionRowHeaderWidth: 28,
                rowHeight: 28,
                enableColumnResizing: true,
                enableHorizontalScrollbar: 0,
                multiSelect: false,
                columnDefs: [
                    {
                        name: 'username',
                        displayName: 'Username',
                        width: '30%',
                        cellTooltip: true
                    },
                    {
                        name: 'email_address',
                        displayName: 'E-mail',
                        width: '30%',
                        cellTooltip: true
                    },
                    {
                        name: 'company',
                        displayName: "Customer",
                        width: '25%',
                        cellTooltip: true
                    },
                    {
                        name: 'privilege',
                        displayName: "Privilege Level",
                        width: '15%',
                        cellFilter: 'privilegeFilter'
                    }
                ],
                data: [],
                onRegisterApi: function( gridApi ) {
                    $scope.gridApi = gridApi;
                    /*$scope.gridApi.treeBase.on.rowExpanded($scope, function(row) {
                        console.log('expand');
                    });*/
                    $scope.gridApi.selection.on.rowSelectionChanged($scope, $scope.handleRowSelection);
                    $scope.gridApi.selection.on.rowSelectionChangedBatch($scope, $scope.handleRowSelection);
                }
            };

            $scope.editUser = function(mode) {
                if (!$scope.user.monitor) {
                    $scope.isEdit = mode === 1;
                    if ($scope.isEdit) {
                        if ($scope.selectedUser === null) {
                            return;
                        }
                        $scope.savedUser = $scope.selectedUser;
                        $scope.selectedUser = jQuery.extend({}, $scope.selectedUser);
                    } else {
                        $scope.savedUser = $scope.selectedUser;
                        $scope.selectedUser = {};
                    }
                    $scope.userTitle = ($scope.isEdit ? "Edit" : "Add") + " User";
                    $http({
                        method: 'GET',
                        url: '/portal/api/managers.php'
                    }).then(function (result) {
                        var data = result.data;
                        $scope.managersList = data['managers'];
                        if ($scope.selectedUser !== undefined && $scope.selectedUser.company_id !== undefined) {
                            $scope.onManagerChange($scope.selectedUser.company_id);
                        }
                        ngDialog.open({
                            template: 'app/configure/users/edit.html',
                            scope: $scope,
                            data: {'user': $scope.selectedUser, 'userTitle': $scope.userTitle, 'editDetails': true},
                            overlay: true,
                            modelDialogId: 'User-Manager',
                            name: 'User-Manager',
                            ngDialogStyle: 'width:500px; height:350px;',
                            closeByDocument: false,
                            closeByEscape: true,
                            preCloseCallback: function (value) {
                            }
                        });
                    }, function (result) {
                        var data = result.data;
                        var message = data.message;
                        ngDialog.open({
                            dialogType: 'Error',
                            modelDialogId: 'user-error-dialog',
                            scope: $scope,
                            dialogMessage: message,
                            onConfirmOkButtonClick: 'onConfirmError()'
                        });
                    });
                }
            };

            $scope.onManagerChange = function(manager) {
                $http({
                    method: 'GET',
                    url: '/portal/api/managers.php?level=locations&id=' + manager
                }).then(function(result) {
                    var data = result.data;
                    $scope.locationsList = [];
                    if (data['managers'] !== undefined && data['managers'].length > 0) {
                        var manager = data['managers'][0];
                        if (manager['locations'] !== undefined) {
                            $scope.locationsList = manager['locations'];
                        }
                    }
                }, function(result) {
                    var data = result.data;
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'user-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });
            };


            $scope.onConfirmError = function() {
                ngDialog.close('user-error-dialog');
            };


            $scope.deleteUser = function() {
                if (!$scope.user.monitor) {
                    if ($scope.selectedUser) {
                        $scope.deleteTitle = 'Confirm User Removal';
                        $scope.confirmMessage1 = 'Are you sure want to delete user ' + $scope.selectedUser.username + '?';
                        $scope.confirmMessage2 = '';
                        $scope.confirmMessage3 = 'I understand that I am removing a user';
                        $scope.submitButtonText = 'Remove User';

                        ngDialog.open({
                            template: 'app/common/confirmationDialog.html',
                            scope: $scope,
                            overlay: true,
                            ngDialogStyle: 'width:600px;height:250px',
                            closeByDocument: false,
                            closeByEscape: true
                        });
                    } else {
                        ngDialog.open({
                            dialogType: 'Warning',
                            dialogMessage: 'You must select one and only one user at a time for deletion.'
                        });
                    }
                }
            };

            $scope.deleteAsset = function() {
                var url = '/portal/api/users.php?id='+$scope.selectedUser.user_id;
                var load = PHD.showLoadingIndicator("body", true);
                var method = 'DELETE';

                $http({
                    method: method,
                    url : url
                }).then(function(result){
                    $scope.loadUsers();
                    PHD.hideLoadingIndicator(load);
                    var message = $scope.selectedUser.username + " removed successfully.";
                    ngDialog.open({
                        dialogType:'information',
                        dialogMessage: message
                    });
                    $scope.selectedUser = null;
                }, function(result) {
                    var data = result.data;
                    PHD.hideLoadingIndicator(load);
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'user-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });

            };

            $scope.closeEditDialog = function() {
                $scope.selectedUser = $scope.savedUser;
                ngDialog.close('User-Manager');
            };

            $scope.setTableHeight = function() {
                var containerHeight = angular.element(document.getElementById('networked-appliance-grid-section')).height();
                // leave some room between the table and the window bottom.
                var gridHeight = containerHeight - 32;
                angular.element(document.getElementById('networkedApplianceGrid')).css('height', gridHeight + 'px');
            };


        }])
    .controller('UserEditCtrl', ['$scope', '$rootScope', '$http', 'ngDialog', 'wizardService', '$analytics', '$timeout',
        function($scope, $rootScope, $http, ngDialog, wizardService, $analytics, $timeout) {

            var $globalScope = $("[ng-controller=GlobalCtrl]").scope();
            $scope.user.monitor = $globalScope.user.monitor;

            $scope.selectedUser = $scope.ngDialogData.user;
            $scope.userTitle = $scope.ngDialogData.userTitle;
            $scope.editDetails = $scope.ngDialogData.editDetails;

            $scope.initCtrl = function () {
                processDialog(PHD, window);
            };

            $scope.privilegeList = [
                { 'privilege': "0", 'name': 'monitor', 'privilege_name': 'monitor' },
                { 'privilege': "1", 'name': 'manager', 'privilege_name': 'manager' },
                { 'privilege': "2", 'name': 'administrator', 'privilege_name': 'administrator' },
                { 'privilege': "3", 'name': 'superuser', 'privilege_name': 'superuser' }
            ];

            function processDialog(PHD, window) {
                $http({
                    method: 'GET',
                    url: '/portal/api/managers.php'
                }).then(function (result) {
                    var data = result.data;
                    $scope.managersList = data['managers'];
                    if ($scope.selectedUser !== undefined && $scope.selectedUser.company_id !== undefined) {
                        $scope.onManagerChange($scope.selectedUser.company_id);
                    }
                }, function (result) {
                    var data = result.data;
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'user-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });
            }

            $scope.saveUser = function() {
                var method = 'POST';
                var url = "/portal/api/users.php";
                if ($scope.isEdit) {
                    method = 'PUT';
                    url += "?id=" + $scope.selectedUser['user_id'];
                }
                var load = PHD.showLoadingIndicator("body", true);
                var payload = $scope.selectedUser;
                payload = JSON.stringify(payload);
                $http({
                    method: method,
                    url: url,
                    data: payload
                }).then(function(result) {
                    PHD.hideLoadingIndicator(load);
                    $scope.closeEditDialog();
                    if ($scope.editDetails) {
                        $scope.loadUsers();
                    }
                }, function(result) {
                    var data = result.data;
                    PHD.hideLoadingIndicator(load);
                    var message = data.message;
                    ngDialog.open({
                        dialogType: 'Error',
                        modelDialogId: 'user-error-dialog',
                        scope: $scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick: 'onConfirmError()'
                    });
                });
            };

            $scope.onConfirmError = function() {
                ngDialog.close('user-error-dialog');
            };

            $scope.closeEditDialog = function() {
                $scope.selectedUser = $scope.savedUser;
                ngDialog.close('User-Manager');
            };

            $scope.setTableHeight = function() {
                var containerHeight = angular.element(document.getElementById('networked-appliance-grid-section')).height();
                // leave some room between the table and the window bottom.
                var gridHeight = containerHeight - 32;
                angular.element(document.getElementById('networkedApplianceGrid')).css('height', gridHeight + 'px');
            };


        }]);