(function($, PHD, window, undefined) {

    var $jobLogTable = $("#backup-copy-table"),
        LogTable, url;
    var $form = $("#status-form");
    var statusForm;

    function initTable() {
        var sid = PHD.appliance_sid;
        var ids = copy_ids;
        if (ids != null && ids[0] !== undefined && ids[0] == ',') {
            ids = ids.slice(1);
        }
        var idJSON = {};
        if (!(ids instanceof String)) {
            // convert to a string for common handling.  On-Demand Backup Copies will have an id,  schedules do not.
            ids = String(ids);
        }
        var pieces = ids.split('.');
        if (pieces.length == 4) {
            // If the ID is of the form x.y.'i'.count, we know we have sid.timestamp.i.count
            // The status API will find return the backups that match the first set closest to this timestamp.
            // Since archiving single threads, we should be able to rely on using hte first set.
            sid = parseInt(pieces[0], 10);
            url = "/api/archive/status/?sid=" + sid;
            idJSON.start_timestamp_archive_set = parseInt(pieces[1], 10);
        } else {
            // If the ID is of the form xa,ya,za, break them apart call the API with the IDs to return.
            url = "/api/archive/status/?sid=" + sid;
            idJSON.archive_ids = ids.split(',').map(function(id) {
                return parseInt(id, 10);
            });
        }
        idJSON = JSON.stringify(idJSON);

        statusForm = PHD.FormController($form, {})
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
            });

        LogTable = PHD.DataGrid($jobLogTable, {
            dynamic: true,
            tableScrollOptions: {"height": "article"},
            select: false,
            sortable: true,
            sortOnLoad: true,
            sortOnHeader: true,
            cellClasses: [null, null, 'long-text truncate', null, null, null, null, null, null, null],
            classifiers: ['success']
        }).registerFormatters(cellFormatter);

        updateTable();

        function updateTable() {
            PHD.Ajax
                .post(url, idJSON)
                .done(function(data) {
                    if (data.status === undefined || data.status.length == 0) {
                        var obj = {};
                        obj.instance_description = "The backup copy information is no longer available.";
                        data.status = [obj];
                    }
                    LogTable.load(data.status);
                });
        }
    }

    var cellFormatter = {
        instance_description: function(value) {
            var title = _.escape(value);
            return '<span class="truncates item-name" title="' + title + '">' + value + '</span>';
        },
        success: function (value, row) {
            var iconClass = value == undefined || value == '--' ? 'warning' : value === true ? "success" : "error";
            return '<i class="icon-phd-'+ iconClass +' icon-large" title="' + iconClass + '"></i>';
        },
        encrypted: function (value, row) {
            var isEncrypted = value;
            if (isEncrypted !== '--') {
                isEncrypted = (value === true) ? "Yes" : "No";
            }
            return(isEncrypted);
        },
        size: function (value, row) {
            var gbSize = value;
            if (gbSize !== '--') {
                gbSize = (value/1024).toFixed(2);
            }
            return(gbSize);
        }
    };

    /*
    *   DOM Ready Function
    */
    $(function() {
        initTable();
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
