(function($, PHD, window, undefined) {

    var $jobLogTable = $("#job-log-table"),
        LogTable, url;
    var $form = $("#status-form");
    var statusForm;
    var allBackups;

    function initTable() {
        var sid = PHD.appliance_sid;
        var bid = backup_ids;
        var showCopies = copies;
        url = "/api/backups/?sid=" + sid + "&bid=" + bid;

        if (showCopies === true) {
            url += "&copies=1";
        }

        statusForm = PHD.FormController($form, {})
            .on("formcancel", function(event) {
                PHD.currentDialog.wizard("close");
            });

        LogTable = PHD.DataGrid($jobLogTable, {
            dynamic: true,
            tableScrollOptions: {"height": "article"},
            select: false,
            sortable: true,
            sortOnLoad: true,
            sortOnHeader: true,
            cellClasses: [null, null, null, null, "full-text"],
            classifiers: ['status']
        }).registerFormatters(cellFormatter);

        updateTable();

        function updateTable() {
            var elem = angular.element(document.querySelector('[ng-controller=GlobalCtrl]'));
            var $rootScope = elem.scope();

            PHD.Ajax
                .get(url)
                .done(function(data) {
                    $(".btn-delete-backup").hide();
                    if (data.BackupStatus === undefined || data.BackupStatus.length == 0) {
                        var obj = {};
                        obj.id = 0;
                        obj.status = 'None';
                        obj.start_time = 'unknown';
                        obj.output = 'The backup(s) no longer exist on the appliance.';
                        data.BackupStatus = [obj];
                    } else {
                        for (var i = 0; i < data.BackupStatus.length; i++) {
                            // Did any backups fail?
                            var backupDetail = data.BackupStatus[i];
                            if (backupDetail.status === 'Failed' && !$rootScope.configureDisabled) {
                                $(".btn-delete-backup").show();
                             break;
                            }
                        }
                    }
                    allBackups = data.BackupStatus;
                    LogTable.load(data.BackupStatus);
                });
        }
    }

    $(".btn-delete-backup").on("click", function() {
        var failedBids = [];
        var failedBackups = [];
        var successCount = 0;
        var warningCount = 0;
        allBackups.forEach(function(obj, index){
            if (obj.status == "Failed"){
                failedBackups.push(obj);
                failedBids.push(obj.id);
            }
            if (obj.status == "Successful"){
                successCount++;
            }
            if (obj.status == "Warning"){
                warningCount++;
            }
        });
        if (failedBids.length !== 0){
           data = failedBackups;
            var message = "The following failed backups will be deleted:";
            message = '<h4 class="text-ok">' + message + '</h4>';
            message += backupTmpl(failedBackups);
            confirm = PHD.confirmDialog(
                message,
                {
                    title: gettext("Confirm Backup Deletion"),
                    open: function (event, ui) {
                        var $dialog = $(this);
                        $dialog.find("table.data-table").scrollTable({"height": 120, "resize": false});
                        $dialog.find(".ui-dialog-buttonset button:last-child").focus();
                        $dialog.dialog("option", "position", {my: "center", at: "center", of: window});
                        $dialog.find(".definition").popover({
                            appendTo: $dialog,
                            position: {
                                my: "left-30 bottom",
                                at: "right bottom+40",
                                within: $dialog
                            }
                        });
                    }
                }
            );
            $('.dialog-confirm').css('border', '1px solid #ccc');

            confirm.done(function () {
                var str_ids = failedBids.toString();
                var sid = (PHD.appliance_sid === null) ? "" : "/?sid=" + PHD.appliance_sid;
                var url = "/api/backups/" + str_ids + sid;
                var resp = PHD.Ajax.delete(url);

                resp.done(function(data) {
                    console.log("done with backup deletion, clearing ids: " + str_ids);
                    PHD.currentDialog.wizard("close");
                    $(document).trigger('failedJobDeleted', [failedBids, successCount, warningCount, PHD.appliance_sid]);

                });
            });

        }
    });

    var cellFormatter = {
        output: function(value, row) {
            var start = 'Start Date: ' + row.start_time + '<br/>';
            var end = row.complete_time != null && row.duration != null  ? ('End Date: ' + row.complete_time + ' (Elapsed: ' + row.duration + ' seconds)' + '<br/>') : "";
            return start + end + value.replace(/\n/g, "<br />");
        }
    };

    var backupTmpl = _.template(
        '<table class="backup-list user-backups data-table dashboard-table">' +
        '<thead><tr>' + '<th> </th>' +
        '<th>' + " Host/Server" + '</th><th>' + "Asset" + '</th><th>' + "Type" + '</th><th>' + 'Status' + '</th><th>' + "Date" + '</th>' +
        '</tr></thead><tbody>' +
        '<% for(var i=0;i<data.length;i++) { %>' +
        '<% data[i].client_name = data[i].client_name ; %>' +
        '<% data[i].asset_name = data[i].asset_name; %>' +
        '<tr>' +
        '<td>' +
        '<% if(data[i].gfs_rp_type !== "") { %>' +
        '<div class="help-trigger"><i class="icon-infosign"></i></div>' +
        '<span class="definition">This backup was kept based<br> on a retention policy.</span>' +
        '<% } %>' + '</td>' +
        '<td><span class="truncates"><%= data[i].client_name %></span></td>' +
        '<td><%= data[i].asset_name %></td>' +
        '<td><%= data[i].type %></td>' +
        '<td><%= data[i].status %></td>' +
        '<td><%= data[i].start_time %></td>' +
        '</tr>' +
        '<% } %>' +
        '</tbody></table>'
    );

    /*
    *   DOM Ready Function
    */
    $(function() {
        initTable();
    });

})(jQuery, window.PHD || (window.PHD = {}), window);
