var jobApp = angular.module('jobs', ['download', 'angulartics', 'angulartics.google.analytics','ngDialog', 'ngTouch', 'ui.grid.treeView','ui.grid.pinning','ui.grid','ui.grid.selection','ui.grid.autoResize','ui.grid.resizeColumns', 'ui.grid.expandable']);

jobApp.factory('jobFactory', function($http, DownloadService) {
    return {
        async: function(url, start_date, end_date, report_name, extra_params) {
            var report_name = report_name || null;
            var extra_params = extra_params || null;
            var params = {};
            params.ts = Date.now();
            if (extra_params != null) {
                params = $.extend(params, extra_params);
            }
            var promise = $http({
                url: url,
                method: "get",
                params: params
            }).then(function(result) {
                var data = result.data;
                var headers = result.headers;
                console.log("success");
                // This should be encapsulated in a reusable function so that other areas (outside reports) can export to CSV or PDF.
                headers = headers();
                var contentType = headers['content-type'];
                if (contentType.indexOf('text/csv') > -1) {
                    var filename = report_name == 'null' ? 'report' + Date.now + ".csv" : report_name.replace(" ", "-") + ".csv";
                    var blob = new Blob([data], {type: contentType});
                    DownloadService.downloadFile(blob, filename);
                }
            }, function(){
                window.PHD.throwError({"result": [{"message": gettext("Requested job data cannot be exported.")}]});
                PHD.hideLoadingIndicator(load);
            });  //1. this returns promise
            return promise;
        }
    };
});

jobApp.controller('JobsCtrl', ['$scope', '$rootScope', '$http', 'Page', 'jobFactory', 'DownloadService', '$analytics', 'ngDialog', '$timeout','$interval','uiGridTreeViewConstants','$q', 'connectionsvc',
        function($scope, $rootScope, $http, Page, jobFactory, DownloadService, $analytics, ngDialog,$timeout,$interval, uiGridTreeViewConstants, $q, connectionsvc) {

            $scope.currentTabIndex = 0;
            $scope.pollEveryFourSeconds = false;
            $scope.showRunMenu = false;
            ngDialogService = ngDialog;
            httpService = $http;
            $scope.Instantlab = $scope.Failover = false;
            $scope.DCAJobName = "Details - Data Copy Access Job";

            Page.setTitle('Jobs');

            $scope.$on('$viewContentLoaded', function() {
                processJobsPage(PHD, window, $rootScope, $scope, $analytics, $http, connectionsvc);
                processJobInventory(PHD, window);
            });

            $scope.exportHistory = function() {
                $analytics.eventTrack('Export History', {  category: 'Jobs', label: 'Jobs' });
                var load = PHD.showLoadingIndicator($(document));
                jobFactory.async("/api/jobs/history/?showFlat=true&sort=desc&format=csv", null, null, 'Job History', null).then(function(result) {
                    PHD.hideLoadingIndicator(load);
                })
            };

            $scope.runMenuClicked = function () {
                if( $(".btn-failover-job").is(":visible")){
                    $(".btn-failover-job").hide();
                    $scope.showRunMenu = false;
                }
                else{
                    $(".btn-failover-job").show();
                    $scope.showRunMenu = true;
                }
            };

            $scope.exportAudits = function() {
                $analytics.eventTrack('Export System', {  category: 'Jobs', label: 'Jobs' });
                var load = PHD.showLoadingIndicator($(document));
                jobFactory.async("/api/jobs/system/?format=csv", null, null, 'System Jobs', null).then(function(result) {
                    PHD.hideLoadingIndicator(load);
                })
            };

            var rootNodeObj={};
            var rootNodeArray=[];
            var finalTreeDataArray = [];

            $scope.openBackupJobDialog = function(){
                $scope.selectedAssetType = INVENTORY_TYPE_FAMILY_CLIENT;
                $scope.selectedNodeList = [];
                //$protectService.openBackupDialog($http,$scope,$rootScope,ngDialog, false);
            };

            $scope.editBackupJobDialog = function() {
                if (!$rootScope.isSelectedJobCopy && !$rootScope.isSelectedDCAJob) {
                    //$protectService.openBackupDialog($http,$scope,$rootScope,ngDialog, true, $rootScope.selectedJobIdToEdit);
                }
                else if($rootScope.isSelectedDCAJob){
                    //$protectService.openCertifyBackupDialog($http,$scope,$rootScope,ngDialog, true, $rootScope.selectedJobIdToEdit);
                }
            };

            window.setInterval(function() {
                if($scope.pollEveryFourSeconds) {
                    $scope.getJobDetails();
                }
            }, 4000);

            $scope.getJobDetails = function() {
                $scope.selectedDCAJob = [];
                $scope.DCAActiveJobDetails = true;  //To let us know if dialog is opened for active job
                var obj;
                var tempTimeString;
                var resp = PHD.Ajax.get("/api/jobs/active/certification/");
                resp.done(function(data) {
                    $scope.activeJobs = data.data;
                    if($scope.activeJobs.length > 0){
                        $scope.activeJobs.forEach(function(job, index) {
                            console.log("job.name "+ job.name);
                            if(job.name == $scope.selectedJobName) {
                                if (job.percent_complete == null || job.percent_complete == 100) {
                                    $scope.pollEveryFourSeconds = false;
                                    $scope.DCAJobName = job.name !== "" ? job.name : "Details - Data Copy Access Job";
                                }

                                if(job !== null){
                                    $scope.selectedDCAJob = [];
                                    $timeout(function() {
                                        $scope.selectedDCAJob.push(job);
                                        $scope.DCAJobName = job.name !== "" ? job.name : "Details - Data Copy Access Job";
                                    });
                                }
                            }
                        });
                    }
                    else if($scope.pollEveryFourSeconds){
                        ngDialog.close('dcaJobDetails-dialog');
                    }
                });
            };

            $scope.openDCAJobDetails = function() {
                $scope.getJobDetails();
                $scope.openDCAJobDetailsNow();
            };

            $scope.openDCAJobDetailsNow = function() {
                var self = this;
                $scope.dcaJobDetails = {};
                $scope.pollEveryFourSeconds = true;
                $scope.title = "Details - Data Copy Access Job : ";
                /*check for screen height*/
                var setTopvalue='top:75px;';
                if (screen.height >1024){
                    setTopvalue='top:15%;'
                }else if(screen.height == 1024){
                    setTopvalue='top:10%;'
                }
                if ($scope.selectedDCAJob != undefined && $scope.selectedDCAJob != null && !$scope.selectedDCAJob.length) {
                    ngDialog.open({
                        template: 'app/joborders/certify-backups/dcaJobDetails.html',
                        scope: $scope,
                        overlay: true,
                        data: $scope.selectedDCAJob,
                        modelDialogId: 'dcaJobDetails-dialog',
                        ngDialogPostionStyle: setTopvalue,
                        ngDialogStyle: 'width:1200px;height:500px',
                        preCloseCallback: function (value) {
                            $scope.pollEveryFourSeconds = false;
                        }
                    });
                } else {
                    var message = "There are no details for this job.";
                    ngDialog.open({
                        dialogType: 'Warning',
                        dialogMessage: message,
                        scope: $scope
                    });
                }
            };

            $scope.openCertifyBackupJobDialog = function(){
                $scope.selectedAssetType = INVENTORY_TYPE_FAMILY_CLIENT;
                $scope.selectedNodeList = [];
                //$protectService.openCertifyBackupDialog($http,$scope,$rootScope,ngDialog, false);
            };

            $scope.openFailoverDialog = function(){
                var loadInstantLabJobindicator = PHD.showLoadingIndicator("body", true, "Loading...");
                $http({
                    method: 'get',
                    url : '/api/joborders/'+$scope.selectedJobId
                }).then(function(result) {
                    var data = result.data;
                    PHD.hideLoadingIndicator(loadInstantLabJobindicator);
                    var dcaJobDetails = data.data[0];
                    console.log("dcaJobDetails "+ dcaJobDetails);
                    $scope.assetListToGetLatestBackups = [];
                    var assetObj;
                    dcaJobDetails.certification_target.instances.forEach(function(instance, index){
                        assetObj = {};
                        assetObj.vm_id = instance.vm_id;
                        assetObj.id = instance.id;
                        assetObj.assetName = instance.name;
                        assetObj.is_wir = instance.is_wir;
                        assetObj.backup = [];
                        assetObj.hyperVisorType = dcaJobDetails.service_profile.server_type;
                        $scope.assetListToGetLatestBackups.push(assetObj);
                    });
                    $scope.getLatestBackups().then(function(){
                        $scope.getWIRSnapshots().then(function(){
                            $scope.populateAssetObjWithBackupsAndWirSnapshots().then(function(){
                                $scope.openFailoverDialogNow();
                            });
                        });
                    });
                    console.log($scope.assetListToGetLatestBackups);
                }, function(response){
                    PHD.hideLoadingIndicator(loadInstantLabJobindicator);
                    ngDialog.open({
                        dialogType:'Error',
                        dialogMessage:response.result[0].message
                    })
                });
            };

            $scope.getLatestBackups = function() {
                var deferred = $q.defer();
                $scope.backupsList = [];
                var certifyJobNameArray = $scope.selectedJobId.split('.');
                var instancesArray = [];
                $scope.assetListToGetLatestBackups.forEach(function (VM) {
                    if(!VM.is_wir){
                        instancesArray.push(VM.id);
                    }
                });
                var instances = '';
                instances = instancesArray.join(',');
                if(instances !== ''){
                    var getlatestBackupsIndicator = PHD.showLoadingIndicator("body", true, "Loading...");
                    $http({
                        method: 'get',
                        url : "/api/backups/latest/?sid=" + certifyJobNameArray[1] + "&iid=" + instances +"&days=7"
                    }).then(function(result){
                        var data = result.data;
                        PHD.hideLoadingIndicator(getlatestBackupsIndicator);
                        $scope.backupHistory = data.data;
                        $scope.backupHistory.forEach(function(backup, index) {
                            $scope.backupsList.push(backup);
                        });
                        deferred.resolve($scope.backupsList);
                    }, function(response){
                        PHD.hideLoadingIndicator(getlatestBackupsIndicator);
                        ngDialog.open({
                            dialogType:'Error',
                            dialogMessage:response.result[0].message
                        })
                    });
                }
                else{
                    deferred.resolve($scope.backupsList);
                }
                return deferred.promise;
            };

            $scope.getWIRSnapshots = function() {
                var deferred = $q.defer();
                var certifyJobNameArray = $scope.selectedJobId.split('.');
                var wirArray = [];
                $scope.WIRSnapshotList = [];
                var wirForSnapshotInfo = [];
                var hypervisor_type = "";
                $scope.wirVMsArray = [];
                $scope.assetListToGetLatestBackups.forEach(function (VM) {
                    if(VM.is_wir && VM.hyperVisorType != "HyperV"){
                        wirArray.push(VM.id);
                    }
                });
                if(wirArray.length > 0){
                    var getSnapshotsIndicator = PHD.showLoadingIndicator("body", true, "Loading...");
                    $http({
                        method: 'get',
                        url : "/api/virtual_clients/?sid=" + certifyJobNameArray[1]
                    }).then(function(result){
                        var data = result.data;
                        var instantRecoveries = data.data;
                        $scope.assetListToGetLatestBackups.forEach(function (VM) {
                            if(VM.is_wir){
                                instantRecoveries.forEach(function(value, index) {
                                    wirArray = value.wir;
                                    wirArray.forEach(function (wir, index) {
                                        if (VM.id == wir.id) {
                                            var obj = {};
                                            obj.vm_id = wir.id;
                                            obj.guid = wir.vm_guid;
                                            $scope.wirVMsArray.push(obj);
                                            wirForSnapshotInfo.push(wir.vm_guid);
                                            if(wir.hypervisor_type == "VMware host"){
                                                hypervisor_type = "VMware";
                                            }
                                            else{
                                                hypervisor_type = "Hyperv";
                                            }
                                        }
                                    });
                                });
                            }
                        });
                        var wirsForSnapshotInfo = wirForSnapshotInfo.join(',');
                        $http({
                            method: 'get',
                            url : "/api/virtual_clients/snapshot_info/?sid=" + certifyJobNameArray[1] + "&type="+ hypervisor_type +"&guid=" + wirsForSnapshotInfo
                        }).success(function(data, status, headers){
                            PHD.hideLoadingIndicator(getSnapshotsIndicator);
                            data = data.vms;
                            data.forEach(function(VMwithSnapshot){
                                $scope.wirVMsArray.forEach(function(VM){
                                    if(VM.guid === VMwithSnapshot.vm_guid){
                                        VMwithSnapshot.vm_id = VM.vm_id;
                                    }
                                });
                                $scope.WIRSnapshotList.push(VMwithSnapshot);
                            });
                            console.log($scope.WIRSnapshotList);
                            deferred.resolve($scope.WIRSnapshotList);
                        }).error(function(response){
                            PHD.hideLoadingIndicator(getSnapshotsIndicator);
                            ngDialog.open({
                                dialogType:'Error',
                                dialogMessage:response.result[0].message
                            })
                        });
                    }).error(function(response){
                        PHD.hideLoadingIndicator(getSnapshotsIndicator);
                        ngDialog.open({
                            dialogType:'Error',
                            dialogMessage:response.result[0].message
                        })
                    });
                }
                else{
                    deferred.resolve($scope.WIRSnapshotList);
                }
                return deferred.promise;
            };

            $scope.populateAssetObjWithBackupsAndWirSnapshots = function() {
                var deferred = $q.defer();
                var obj;
                $scope.assetListToGetLatestBackups.forEach(function(asset, index) {
                    console.log("asset.is_wir "+asset.is_wir);
                    if(asset.is_wir) {
                        if( asset.hyperVisorType == "HyperV"){
                            obj = {};
                            obj.value = null;
                            obj.id = null;
                            obj.name = "Failover to latest recovery point";
                            obj.tooltip = "Failover to latest recovery point";
                            asset.backup.push(obj);
                        }
                        else{
                            $scope.WIRSnapshotList.forEach(function(VMwithSnapshot){
                                if(VMwithSnapshot.vm_id === asset.id){
                                    if(VMwithSnapshot.snapshots.length > 0){
                                        VMwithSnapshot.snapshots = descSortByProperty(VMwithSnapshot.snapshots, "backup_timestamp");
                                        VMwithSnapshot.snapshots.forEach(function(snapshot){
                                            obj = {};
                                            obj.value = snapshot.identifier;
                                            obj.id = snapshot.identifier;
                                            obj.name = snapshot.backup_date;
                                            obj.certified = snapshot.certified;
                                            obj.tooltip = getBackupCertifiedStatus(obj.certified);
                                            asset.backup.push(obj);
                                        })
                                    }
                                }
                            });
                        }
                    }
                    else {
                        $scope.backupsList = descSortByProperty($scope.backupsList, "start_time");
                        $scope.backupsList.forEach(function(backup, index) {
                            if (backup.instance_id == asset.id && backup.backup_id !== "N/A") {
                                obj = {};
                                obj.value = backup.backup_id;
                                obj.id = backup.backup_id;
                                obj.name = backup.start_time_string;
                                obj.certified = backup.certified;
                                obj.tooltip = getBackupCertifiedStatus(obj.certified);
                                asset.backup.push(obj);
                            }
                        });
                    }
                });
                deferred.resolve($scope.assetListToGetLatestBackups);
                return deferred.promise;
            };

            function descSortByProperty(array, propertyName) {
                return array.sort(function (a, b) {
                    return b[propertyName] - a[propertyName];
                });
            }

            function getBackupCertifiedStatus(certifiedStatus){
                switch(certifiedStatus){
                    case null:
                    case -1:
                    case 0:
                    case 2:
                        return "Not Certified";
                        break;
                    case 1:
                        return "Error during Certification";
                        break;
                    case 3:
                        return "Certified";
                        break;
                    case 4:
                        return "Certified with warning";
                        break;
                }
            }


            $scope.openFailoverDialogNow = function() {
                var self = this;
                $scope.title = "Start Failover";
                $scope.Failover = true;
                $scope.Instantlab = false;
                /*check for screen height*/
                var setTopvalue='top:75px;';
                if (screen.height >1024){
                    setTopvalue='top:10%;'
                }else if(screen.height == 1024){
                    setTopvalue='top:6%;'
                }
                $scope.defaultJobName = $scope.selectedJobName + '-Failover';
                ngDialog.open({
                    template: 'app/joborders/failover/failoverJob.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId:'failover-dialog',
                    ngDialogPostionStyle:setTopvalue,
                    ngDialogStyle:'width:800px; height:600px',
                    preCloseCallback: function (value) {}
                });
            };

            $scope.instantLabJob = function(){
                var loadInstantLabJobindicator = PHD.showLoadingIndicator("body", true, "Loading...");
                $http({
                    method: 'get',
                    url : '/api/joborders/'+$scope.selectedJobId
                }).success(function(data, status, headers){
                    PHD.hideLoadingIndicator(loadInstantLabJobindicator);
                    var dcaJobDetails = data.data[0];
                    console.log("dcaJobDetails "+ dcaJobDetails);
                    $scope.assetListToGetLatestBackups = [];
                    var assetObj;
                    dcaJobDetails.certification_target.instances.forEach(function(instance, index){
                        assetObj = {};
                        assetObj.vm_id = instance.vm_id;
                        assetObj.id = instance.id;
                        assetObj.assetName = instance.name;
                        assetObj.is_wir = instance.is_wir;
                        assetObj.hyperVisorType = dcaJobDetails.service_profile.server_type;
                        assetObj.backup = [];
                        $scope.assetListToGetLatestBackups.push(assetObj);
                    });
                    $scope.getLatestBackups().then(function(){
                        $scope.getWIRSnapshots().then(function(){
                            $scope.populateAssetObjWithBackupsAndWirSnapshots().then(function(){
                                $scope.openInstantLabDialogNow();
                            });
                        });
                    });
                    console.log($scope.assetListToGetLatestBackups);
                }).error(function(response){
                    PHD.hideLoadingIndicator(loadInstantLabJobindicator);
                    ngDialog.open({
                        dialogType:'Error',
                        dialogMessage:response.result[0].message
                    })
                });
            };

            $scope.openInstantLabDialogNow = function() {
                var self = this;
                $scope.title = "Instant Lab";
                $scope.Instantlab = true;
                $scope.Failover = false;
                /*check for screen height*/
                var setTopvalue='top:75px;';
                if (screen.height >1024){
                    setTopvalue='top:10%;'
                }else if(screen.height == 1024){
                    setTopvalue='top:6%;'
                }
                $scope.defaultJobName = $scope.selectedJobName + '-InstantLab';
                ngDialog.open({
                    template: 'app/joborders/failover/failoverJob.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId:'failover-dialog',
                    ngDialogPostionStyle:setTopvalue,
                    ngDialogStyle:'width:800px;',
                    preCloseCallback: function (value) {}
                });
            };

            $scope.commitFailover = function() {
                var loadJobindicator = PHD.showLoadingIndicator("body", true, "Committing Job...");
                var jobOrderUrl = '/api/joborders/commit/'+$scope.selectedJobId;
                $http({ method: 'put',
                    url : jobOrderUrl
                }).success(function(data, status, headers){
                    PHD.hideLoadingIndicator(loadJobindicator);
                    var message = "Commit job submitted successfully!";
                    ngDialog.open({
                        dialogType: 'Success',
                        dialogMessage: message,
                        ngDialogStyle:'width:450px;'
                    });
                }).error(function(response) {
                    var message = response.result[0].message;
                    $analytics && $analytics.eventTrack('Failure: ' + message, {  category: 'Failover', label: 'Protect' });
                    PHD.hideLoadingIndicator(loadJobindicator);
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'ErrorDialog',
                        scope:$scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick:'onConfirmError()'
                    });
                });
            };

            $scope.discardFailover = function() {
                var loadJobindicator = PHD.showLoadingIndicator("body", true, "Discarding Job...");
                var jobOrderUrl = '/api/joborders/discard/'+$scope.selectedJobId;
                $http({ method: 'put',
                    url : jobOrderUrl
                }).success(function(data, status, headers){
                    PHD.hideLoadingIndicator(loadJobindicator);
                    var message = "Discard job submitted successfully!";
                    ngDialog.open({
                        dialogType: 'Success',
                        dialogMessage: message,
                        ngDialogStyle:'width:450px;'
                    });
                }).error(function(response) {
                    var message = response.result[0].message;
                    $analytics && $analytics.eventTrack('Failure: ' + message, {  category: 'Failover', label: 'Protect' });
                    PHD.hideLoadingIndicator(loadJobindicator);
                    ngDialog.open({
                        dialogType:'Error',
                        modelDialogId:'ErrorDialog',
                        scope:$scope,
                        dialogMessage: message,
                        onConfirmOkButtonClick:'onConfirmError()'
                    });
                });
            };

            $scope.onConfirmError = function () {
                ngDialog.close('ErrorDialog');
                $scope.jobErrorDialog = false;
            };

            $scope.jobHistorySave = function() {
                    $rootScope.jobHistoryLimit = $('#id_job_history_load').val();
                $scope.toggleModal();
                $(document).trigger("limit-changed", [JOB_TAB_HISTORY, $scope.jobHistoryLimit]);
            };

            $scope.systemJobHistorySave = function() {
                $rootScope.systemJobHistoryLimit = $('#id_system_job_history_load').val();
                $scope.toggleModal();
                $(document).trigger("limit-changed", [JOB_TAB_SYSTEM, $scope.systemJobHistoryLimit]);
            };

            $scope.openSLAPolicyDialog = function(){
                $scope.selectedAssetType = INVENTORY_TYPE_FAMILY_CLIENT;
                $scope.selectedNodeList = [];
                //$protectService.openSLAPolicyDialog($http,$scope,$rootScope,ngDialog, false);
            };

            $scope.editSLAPolicy = function() {
                // last parameter is policy id to edit, set this globally.
                //$protectService.openSLAPolicyDialog($http,$scope,$rootScope,ngDialog, true, $rootScope.selectedPolicyIdToEdit);
            };


            /* Click Event Listener for open configure model dialog box  */
            $scope.modalShown=false;

            $scope.toggleModal = function(){
                if($scope.currentTabIndex === 2){
                    this.toggleHistoryModal();
                } else if ($scope.currentTabIndex === 3){
                    this.toggleSystemHistoryModal();
                }
            };

            $scope.toggleHistoryModal=function(){
                  $('#id_job_history_load').val($scope.jobHistoryLimit);
                  $scope.modalShown=!$scope.modalShown;
            };
            $scope.toggleSystemHistoryModal=function(){
                $('#id_system_job_history_load').val($scope.systemJobHistoryLimit);
                $scope.modalShown=!$scope.modalShown;
            };
        }]
);
