angular.module('language-selection', ['ngDialog','tour-module'])
    .controller('LanguageCtrl', ['$scope', '$rootScope', '$http', 'Page', 'ngDialog', '$timeout', 'tourFactory', 'gettextCatalog',
        function($scope, $rootScope, $http, Page, ngDialog, $timeout, tourFactory, gettextCatalog) {

            /*
             * When the controller initializes, set the initial language set in the $rootScope.
             */
            $scope.initializeDefaultLanguage = function() {
                $scope.setUserLanguage($rootScope.userLanguageCode, true);
            };

            /*
             * Available application themes, the names are used for css styles.
             */
            $scope.availableLanguages = [
                { code: 'en', language: gettext('English') },
                { code: 'fr', language: gettext('French') }
            ];

            $scope.selectLanguage = function() {
                ngDialog.open({
                    modelDialogId:'app-language',
                    name: 'app-language',
                    template: 'app/preferences/language.html',
                    scope:$scope,
                    overlay:true,
                    ngDialogStyle:"width:400px"
                });
            };

            $scope.setCurrentLanguage = function() {
                for (var i = 0; i < $scope.availableLanguages.length; i++) {
                    var language = $scope.availableLanguages[i];
                    language.selected = false;
                    if (language.code === $rootScope.userLanguageCode) {
                        language.selected = true;
                        $scope.selectedLanguage = language;
                    }
                }
            };

            $scope.pickLanguage = function(language) {
                $scope.selectedLanguage = language;
            };

            /*
             * Update the language in the UI (gettext) and in the preferences.
             */
            $scope.saveLanguage = function() {
                // Only update the DOM if the theme changed.
                if ($scope.selectedLanguage !== undefined && ($scope.selectedLanguage.code !== $rootScope.userLanguageCode)) {
                    // Update the UI.
                    $scope.setUserLanguage($scope.selectedLanguage.code);
                    // Persist preferences.
                    $scope.updateLanguagePreference($scope.selectedLanguage.code);
                }

                $scope.closeThisDialog();
            };

            /*
             * Calls the angular-gettext functions to set the language and load the strings JSON file.
             */
            $scope.setUserLanguage = function(code, force) {
                var forceChange = force || false;
                $scope.$evalAsync(function() {
                    if (forceChange || (code !== $rootScope.userLanguageCode)) {
                        $rootScope.userLanguageCode = code;
                        gettextCatalog.setCurrentLanguage(code);
                        if (code !== 'en') {
                            // If not English, load the json file.
                            gettextCatalog.loadRemote("locale/" + code + "/LC_MESSAGES/" + code + ".json");
                        }
                    }
                });
            };

            $scope.$on("update-language", function(event, code) {
                $scope.setUserLanguage(code);
            });

            $scope.updateLanguagePreference = function(code) {
                var prefURL = '/portal/api/preferences.php?user=' ;
                var putData = {};
                putData['language'] = code;
                prefURL = prefURL + encodeURI($rootScope.user.name);
                tourFactory.sync('put', prefURL, $rootScope.local.id, putData).then(function(result){
                    console.log("success >> update language to " + code);
                });
            };

            $scope.closeThisDialog = function() {
                ngDialog.close();
            };

            $scope.initCtrl = function(dlg) {
                switch (dlg.name) {
                    case 'app-language':
                        $scope.setCurrentLanguage();
                        break;
                }
            };

            $scope.$on('ngDialog.opened', function(event, obj) { $scope.initCtrl(obj) });

        }]);