var reportApp = angular.module('reports', ['ui.grid', 'ui.bootstrap','download', 'angulartics', 'angulartics.google.analytics',
                                'ui.grid.cellNav','ngTouch', 'ui.grid', 'ui.grid.pagination', 'ui.grid.edit', 'ui.grid.resizeColumns',
                                'ui.grid.pinning', 'ui.grid.selection', 'ui.grid.moveColumns', 'ui.grid.exporter', 'ui.grid.importer',
                                'ui.grid.grouping','ui.grid.autoResize', 'distributed-reports-module']);
var configParams;
var isSetInfo = false;
var enviornmentId = 0;

/* Creating factory for reportApp
**Input Params:
	*url
	*start_date
	*end_date
**Output 
	*JSON object from requested URL
*/
reportApp.factory('reportFactory', function($http, DownloadService) {
  return {
  async: function(url, start_date, end_date, enviornmentID, report_name, extra_params, analytics) {
	  var report_name = report_name || null;
	  var extra_params = extra_params || null;
	  var $analytics = analytics || false;
	  $analytics && $analytics.eventTrack(report_name, {  category: 'Reports', label: 'Reports' });
	  var params = {};
	  // only add sid if environmentID is not all.
	  if (enviornmentID !== 0) {
		  params.id = enviornmentID;
	  }
	  params.start_date = start_date;
	  params.end_date = end_date;
	  params.ts = Date.now();
	  if (extra_params !== null) {
		  params = $.extend(params, extra_params);
	  }
    var promise = $http({
       url: url, 
       method: "get",
       params: params
    }).then(function(result) {
        var headers = result.headers;
        var data = result.data;
		console.log("success");
		// This should be encapsulated in a reusable function so that other areas (outside reports) can export to CSV or PDF.
		headers = headers();
		var contentType = headers['content-type'];
        console.log(headers);
		if (contentType.indexOf('text/csv') > -1) {
			var filename = report_name == 'null' ? 'report' + Date.now + ".csv" : report_name.replace(" ", "-") + ".csv";
			var blob = new Blob([data], {type: contentType});
			DownloadService.downloadFile(blob, filename);
		}else if (data.pdf_url !== undefined) {
			var filename = report_name == 'null' ? 'report' + Date.now + ".pdf" : report_name.replace(" ", "-") + ".pdf";
			var pdf_location = String(data.pdf_url).indexOf('/');
			var pdf = String(data.pdf_url).slice(pdf_location);
			DownloadService.saveAs(pdf, filename);
			PHD.hideLoadingIndicator(load);
		}
		return result;
	}, function(result){
        var data = result.data;
		if(data && data.result[0].message != null && data.result[0].message != "") {
			window.PHD.throwError(data);
		} else {
			window.PHD.throwError({"result": [{"message": gettext("Requested data can't be retrieved for this report.")}]});
		}
		PHD.hideLoadingIndicator(load);
	});  //1. this returns promise
	 return promise;
	}
  };
});

/* Creating Controller for reportApp
*All the business logic goes here
*/

var ARCHIVE_SET_ID = "Archives in Set";
/* reportApp.controller('ReportsCtrl', ['reportFactory','$scope', '$routeParams', '$http', 'Page', '$analytics',
		function(reportFactory, $scope, $routeParams, $http, Page, $analytics) { */
		
reportApp.controller('ReportsCtrl', ['reportFactory','$scope', '$routeParams', '$location', '$http', 'Page','$analytics','$timeout',
                     '$interval', 'uiGridConstants','$rootScope','uiGridExporterConstants','ngDialog', '$filter', 'distributedReportsSvc',
    function(reportFactory, $scope, $routeParams, $location, $http, Page, $analytics, $timeout, $interval,
             uiGridConstants,$rootScope,uiGridExporterConstants, ngDialog, $filter, distributedReportsSvc) {
         $scope.gridList=[];    
         $scope.reportHeaders=[];
		 $scope.reportFormat={};
		 $scope.reportFormat.pdf={};
		 $scope.dataGridOption={};
       
         var paginationOptions = {
            pageNumber: 1,
            pageSize: 25,
            sort: null
          };
        
        $scope.ROW_HEIGHT = 30;
    
		 $scope.heading= [];
		 Page.setTitle('Reports');
		 $scope.tempDataArray = [];
		 $scope.allDataArray = [];
		 $scope.parsedBackendData = [];
		
		 $scope.totalServerItemsArray=[];
		 
		 $scope.gridData = [];
		 $scope.gridIdList=[];
		 $scope.hideGrid = false;                   // For some graphical reports, we do want to hide the grid itself, but support export to CSV or PDF.
		
		 var cacheParseDataArray = [];
		 var index = 0;
		 var gridReportData = "";
		 var GridItems = [];
         var reportId = 0;
		 var reportDays = 0;
         var report_sid = 0;

		$scope.dataGrids = [];
		$scope.LEGAL_HOLD = 1;
		$scope.UNPROTECTED = 2;
		$scope.PROTECTION_SUMMARY = 3;
		$scope.BACKUP_HISTORY = 4;
		$scope.STORAGE_DETAILS = 5;
		$scope.UPDATES = 6;
		$scope.REPLICATION = 7;
		$scope.ARCHIVE = 8;
		$scope.REPLICATION_HISTORY = 9;
		$scope.SYSTEM_LOAD = 12;
		$scope.REPLICATION_REPORT = 13;
		$scope.RESTORE_HISTORY=14;
		$scope.STORAGE_DATAREDUCTION=15;
		$scope.SYSTEM_CAPACITY = 16;
		$scope.REPLICATION_FOOTPRINT = 17;
		$scope.BACKUP_FAILURE = 18;
        $scope.ALERT_REPORT = 19;
        $scope.AUDIT_REPORT = 20;
        $scope.TRAPS_REPORT = 21;
        $scope.WEEKLY_STATUS = 22;
        $scope.DISTRIBUTED_APPLIANCE_CAPACITY = 23;
        $scope.APPLIANCE_CUSTOMER_LOCATION = 24;
        $scope.PROTECTION_POLICIES = 27;


		/*Main Report Id's.*/
		
		$scope.BACKUP_REPORT = 1;
		$scope.RESTORE_REPORT = 2;	
		$scope.BACKUP_COPY_REPORT = 3;
		$scope.SYSTEM_REPORT = 5;
		$scope.STORAGE_REPORT = 6;
        $scope.COMPLIANCE_REPORT = 7;

		/*
		 * Array to show the Date field, appliance combo and header depends upon report requirement
		 */
		$scope.REPORTS_WITH_NO_DATE_SELECTION = [ // Report with no Date Selection
			$scope.STORAGE_DETAILS,
			$scope.REPLICATION_REPORT,
			$scope.SYSTEM_CAPACITY,
			$scope.REPLICATION,
			$scope.REPLICATION_FOOTPRINT,
            $scope.WEEKLY_STATUS,
            $scope.APPLIANCE_CUSTOMER_LOCATION,
            $scope.PROTECTION_POLICIES,
            $scope.ALERT_REPORT,
            $scope.BACKUP_HISTORY
		];
		
		$scope.REPORTS_WITH_NO_HEADER_INFO = [ // Report with no header info
			$scope.STORAGE_DETAILS,
			$scope.REPLICATION_REPORT
		];
		$scope.REPORTS_WITH_NO_APPLIANCE_SELECTION = [ // Report with appliance combo
			$scope.STORAGE_DETAILS,
			$scope.REPLICATION_REPORT,
            $scope.REPLICATION_FOOTPRINT,
            $scope.DISTRIBUTED_APPLIANCE_CAPACITY,
            $scope.APPLIANCE_CUSTOMER_LOCATION
		];
        $scope.REPORTS_WITH_NO_EXPORTS = [
            $scope.SYSTEM_LOAD,
            $scope.STORAGE_DATAREDUCTION
        ];
        $scope.REPORTS_WITH_CSV_EXPORT_ONLY = [
            $scope.DISTRIBUTED_APPLIANCE_CAPACITY
        ];
        $scope.REPORTS_WITH_MONTH_RANGE = [
            $scope.STORAGE_DATAREDUCTION,
            $scope.DISTRIBUTED_APPLIANCE_CAPACITY,
            $scope.APPLIANCE_CUSTOMER_LOCATION
        ];

		$scope.names = [
			"",
			"Legal Hold Backups",
			"Unprotected VMs",
			"Protection Summary",
			"Backup Status",
			"Storage",
			"Update History",
			"Backup Copy Capacity",
			"Backup Copy – Cold Targets",
			"Backup Copy – Hot Targets",
			"Archive History",
			"VM Storage Report",
			"Load Report",
			"Backup Copies - Past 24 Hours",
			"Recovery History" ,
			"Data Reduction",
			"Capacity Report",
			"Storage Footprint Report",
			"Backup Failures",
            "Alerts",
            "System Notifications",
            "Trap History",
            "Weekly Status",
            "Remote Appliance Capacity",
            "Appliances, Customers, and Locations",
            "Recovery Assurance",
            "Compliance Report",
            "Protection Policies"
		];

		$scope.descriptions = [
			"",
			"A report of all backups currently on legal hold.",
			"Assets that were not backed up within the selected date range.",
			"A graphical and tabular view of protected and unprotected assets within the selected date range.",
			"All backup results by asset over the past 7 days.",
			"Details for all configured storage.",
			"Updates for the specified appliance that occurred during the selected date range. For additional details on update failures, click the report row.",
			"The protected capacity, i.e., the size of the last Full backups per asset on a backup copy appliance target.",
			"The backup copies to cold storage by protected asset within the selected date range.  For additional details, click the report row.",
			"Backups that have been copied to a Unitrends appliance target.  For additional details, click the report row.",
			"All cold backup copies by asset within the selected date range.",
			"Cumulative per-protected asset backup size statistics.",
			"A graphical view of appliance load over the selected date range.",
			"Successful, active, and queued backup copy jobs to hot targets in the last 24 hours.",
			"Recovery operations that occurred within the selected date range. File-level recovery objects created (iSCSI/CIFS) are not included in this report.  For additional details, click the report row." ,
			"A report of storage space saved by the appliance using deduplicaton and compression.",
			"The protected capacity, i.e., the size of the last Full backups for all protected assets.",
			"The amount of space backup copy sources are using on a target.",
			"Failed backups over the selected date range.  For additional details, click the report row.",
            "A list of all alerts over the selected date range.",
            "System notifications from the audit log.",
            "A list of SNMP traps sent from the specified appliance over the selected date range.",
            "A status summary of backups and copies over the last 7 days. Click on a status icon for additional information about the selected day's backups or copies.",
            "A graphical view of protected capacity over time for remote appliances",
            "A report of each appliance's customer and location information",
            "All Recovery Assurance operations that occurred within the selected date range and their respective job results. Click on the job to view additional details.",
            "All Compliance, Recovery Point Objective (RPO) and Recovery Time Objective (RTO) details. Click on the job to view additional details.",
            "A report of each protected asset's policies: jobs, retention, and inclusions/exclusions"
		];
		/*report ID and report title map */
		var reportIdtoReportTitleMap={
            1:"Legal Hold",
            3:"Protection Summary",
            4:"Backup History",
            5:"Storage",
            6:"Update History",
            7:"Replication Capacity",
            8:"Backup Copy – Cold Targets",
            9:"Replication History",
            14:"Restore History",
            16:"System Capacity",
            17:"Storage Footprint",
            18:"Failure",
            19:"Alerts",
            20:"System Notifications",
            21: "Trap History",
            22: "Weekly Status",
            23: "Distributed Appliance Capacity",
            24: "Customers and Locations",
            25:"Recovery Assurance",
            26: "Compliance Report",
            27: "Protection Policies"
        };
        var reportIdtoReportCategory={
            1:"backup",
            3:"backup",
            4:"backup",
            5:"storage",
            6:"system",
            7:"replication",
            8:"archive",
            9:"replication",
            14:"restore",
            16:"system",
            17:"replication",
            18:"backup",
            19:"system",
            20:"system",
            21:"system",
            22:"backup",
            23:"distributed",
            24:"distributed",
            25:"restore",
            26:"compliance",
            27:"backup"
        };

        // template for the weekly status report that produces 1 status icon per day based on backup status.
        var backupColoredStatusTemplate = '<div class="ui-grid-cell-contents"> ' +
            '<i ng-repeat="a in row.entity.last_backups track by $index" ' +
                'ng-click="row.entity.last_backups[$index].disabled || grid.appScope.showStatusDetails(row.entity, 0, $index)" ' +
                'class="{{grid.appScope.showBackupStatus(row.entity.last_backups, $index)}} report-status-icon" ' +
                'title="{{grid.appScope.showDayTitle($index)}}"></i>' +
            '</div>';

        // template for the weekly status report that produces 1 status icon per day based on backup copy status.
        var backupCopyColoredStatusTemplate = '<div class="ui-grid-cell-contents"> ' +
            '<i ng-repeat="a in row.entity.last_backup_copies track by $index" ' +
                'ng-click="row.entity.last_backup_copies[$index].disabled || grid.appScope.showStatusDetails(row.entity, 1, $index)" ' +
                'class="{{grid.appScope.showBackupStatus(row.entity.last_backup_copies, $index)}} report-status-icon" ' +
                'title="{{grid.appScope.showDayTitle($index)}}"></i>' +
            '</div>';

        // Options for the filter dropdowns for the weekly status report.
        var selectStatusFilterOptions = [
            { value: 'successes', label: 'Only Successes in last 7 days' },
            { value: 'nosuccesses', label: 'No Successes in last 7 days' },
            { value: 'failures', label: 'Failed in last 7 days' }
        ];

        // Given the chosen filter (searchTerm), decide which rows should be seen (status of true), or removed from view (status of false).
        function selectStatusFilterFunction (searchTerm, cellValue) {
            var status = searchTerm == 'nosuccesses';
            if (cellValue instanceof Array) {
                for (var i = 0; i < cellValue.length; i++) {
                    var day = cellValue[i];
                    if (searchTerm == 'successes') {
                        if (status === false && day.successes !== undefined && day.successes.count > 0) {
                            status = true;
                        }
                        if ((day.warnings !== undefined && day.warnings.count > 0) || day.failures !== undefined && day.failures.count > 0) {
                            status = false;
                            break;
                        }
                    } else if (searchTerm == 'failures') {
                        if (day.failures !== undefined && day.failures.count > 0) {
                            status = true;
                            break;
                        }
                    } else if (searchTerm == 'nosuccesses') {
                        if (day.successes !== undefined && day.successes.count > 0) {
                            status = false;
                            break;
                        }
                    }
                }
            }
            return status;
        }

        var reportGridColumnDefinition={
          "backup_1": {
              "0": {
                  "system_name": {
                      "displayName": "Appliance Name",
                      "visible": true,
                      "cellFilter": "nullTestFilter",
                      "filter": {placeholder: 'Appliance Name'}
                  },
                  "client_name": {
                      "displayName": "PROTECTED ASSET",
                      "visible": true,

                      "cellFilter": "nullTestFilter",
                      "filter": {placeholder: 'Protected Asset'}
                  },
                  "id": {
                      "displayName": "ID",
                      "visible": true,

                      "cellFilter": "nullTestFilter",
                      "filter": {placeholder: 'Id'}
                  },
                  "app_name": {
                      "displayName": "APPLICATION",
                      "visible": true,

                      "cellFilter": "nullTestFilter",
                      "filter": {placeholder: 'Application'}
                  },
                  "database_name": {
                      "displayName": "DATABASE/VM",
                      "visible": true,

                      "cellFilter": "nullTestFilter",
                      "filter": {placeholder: 'Database/VM'}
                  },
                  "hold_end_date": {
                      "displayName": "LEGAL HOLD EXPIRATION DATE",
                      "visible": true,

                      "cellFilter": "nullTestFilter",
                      "filter": {placeholder: 'Legal Hold Expiration Date'}
                  },
                  "hold_days": {
                      "displayName": "HOLD DAYS",
                      "visible": true,

                      "cellFilter": "colHoldDayFilter | nullTestFilter",
                      "filter": {placeholder: 'Hold Days'}
                  },
                  "start_date": {
                      "displayName": "BACKUP START DATE",
                      "visible": true,
                      "type":"date" ,
                      "cellFilter": "nullTestFilter",
                      "filter": {placeholder: 'Backup Start Date'}
                  },
                  "type": {
                      "displayName": "TYPE",
                      "visible": true,

                      "cellFilter": "nullTestFilter",
                      "filter": {placeholder: 'Type'}
                  },
                  "size": {
                      "displayName": "SIZE (GB)",
                      "visible": true,
                      "type": "number",
                      "cellFilter": "nullTestFilter",
                      "filter": {placeholder: 'Size (GB)'}
                  }
              }
        },        
        "backup_4": {
            "0": {
                "system_name": {
                    "displayName": "Appliance Name",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Appliance Name'}
                },
                "client_name": {
                    "displayName": "Protected Asset",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Protected Asset'}
                },
                "database_name": {
                    "displayName": "Database Name",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Database Name'}
                },
                "id": {
                    "displayName": "ID",
                    "visible": false,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Id'}
                },
                "status": {
                    "displayName": "status",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'status'}
                },
                "start_date": {
                    "displayName": "start_date",
                    "visible": true,
                    "type": "date",
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'start_date'}
                },
                "end_date": {
                    "displayName": "end_date",
                    "visible": false,
                    "cellFilter": "nullTestFilter",
                    "type": "date",
                    "filter": {placeholder: 'end_date'}
                },
                "elapsed_time": {
                    "displayName": "elapsed_time",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'elapsed_time'}
                },
                "type": {
                    "displayName": "type",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'type'}
                },
                "size": {
                    "displayName": "size (GB)",
                    "visible": true,
                    "type": "number",
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'size (GB)'}
                }
            }
        },        
        "backup_3": {
            "0": {
				"system_name": {
                    "displayName": "APPLIANCE NAME",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Appliance Name'}
					},
				"client_name": {
				displayName:"CLIENT NAME",
					"visible": true,
					"filter": {placeholder: 'Client Name'}
					},
				"instance_name": {
				displayName:"INSTANCE NAME",
					"visible": true,
					"filter": {placeholder: 'Instance Name'}
					},
				"asset_type": {
				displayName:"ASSET TYPE",
					"visible": true,
					"filter": {placeholder: 'Asset Type'}
					},
				"protected": {
				displayName:"PROTECTED",
					"visible": true,
					"filter": {placeholder: 'Protected'}
					},
				"protected_by_backup_copy": {
					displayName:"PROTECTED BY BACKUP COPY",
					"visible": true,
					"filter": {placeholder: 'PROTECTED BY BACKUP COPY'}
					}
			},
			"1": {
                "system_name": {
                    "displayName": "APPLIANCE NAME",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Appliance Name'}
                },
                 "client_name": {
					displayName:"CLIENT NAME",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Client Name'}
                },
				"instance_name": {
					displayName:"INSTANCE NAME",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Instance Name'}
                },
                "asset_type": {
                    "displayName": "ASSET TYPE",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Asset Type'}
                },
				"backup_types": {
                    "displayName": "BACKUP TYPES",
                    "visible": true,
                    "cellFilter": "arrayToStringFormatter",
                    "cellTooltip": true,
					"filter": {placeholder: 'Backup Types'}
                },
                "successes": {
                    "displayName": "SUCCESSES",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Successes'}
                },
                "warnings": {
					displayName:"WARNINGS",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Warnings'}
                },
                "failures": {
                    "displayName": "FAILURES",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Failures'}
                },
                "last_backup": {
                    "displayName": "LAST BACKUP",
                    "visible": true,
					"cellFilter": "nullTestFilter",
					"type":"date" ,
					"filter": {placeholder: 'Last Backup'}
                }
            },
            "2": {
                "system_name": {
                    "displayName": "APPLIANCE NAME",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Appliance Name'}
                },
                 "client_name": {
                    "displayName": "CLIENT NAME",
                    "visible": true,
					"cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Client Name'}
                },
                "instance_name": {
                    "displayName": "INSTANCE NAME",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Instance Name'}
                },
                "asset_type": {
                    "displayName": "ASSET TYPE",
                    "visible": true,
					"cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Asset Type'}
                },
				"backup_types": {
                    "displayName": "BACKUP TYPES",
                    "visible": true,
                    "cellFilter": "arrayToStringFormatter",
                    "cellTooltip": true,
					"filter": {placeholder: 'Backup Types'}
                }, 
                "backup_copy_successes": {
                    "displayName": "BACKUP COPY SUCCESSES",
                    "visible": true,
					"cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Backup Copy Successes'}
                },
                "backup_copy_failures": {
                    "displayName": "BACKUP COPY FAILURES",
                    "visible": true,
					"cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Backup Copy Failures'}
                },
                "last_backup_copy": {
                    "displayName": "LAST BACKUP COPY",
                    "visible": true,
					"type":"date",
					"cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Last Backup Copy'}
                },
                "next_backup_copy_to_UEB-Calsoft-240-target_joborder": {
                    "displayName": "LAST BACKUP COPY",
                    "visible": true,
					"cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Last Backup Copy'}
                },
                "success": {
                    "displayName": "SUCCESS",
                    "visible": true,
					"cellFilter": "nullTestFilter",
					"filter": {placeholder: 'Success'}
                }
            }
        },
       "storage_5": {
            "0": {
            "name": {
                "displayName": "NAME",
                "visible": true,
                "filter": {placeholder: 'Name'}
                },
            "mb_size": {
                "displayName": "Allocated Storage (GB)",
                "visible": true,
                "cellFilter": "MBToGBFormater",
                "filter": {placeholder: 'Allocated Storage (GB)'}
                },
            "mb_used": {
                "displayName": "TOTAL BACKUP DATA (GB)",
                "visible": true,
                "cellFilter": "MBToGBFormater",
                "filter": {placeholder: 'Total Backup Data (GB)'}
                },
            "daily_growth_rate": {
                "displayName": "DAILY GROWTH",
                "visible": true,
                "filter": {placeholder: 'Daily Growth'}
                },
            "dedup": {
                "displayName": "DEDUPE RATIO",
                "visible": false,
                "filter": {placeholder: 'Dedupe Ratio'}
                }
                ,
            "system_id": {
                "displayName": "APPLIANCE ID",
                "visible": false,
                "filter": {placeholder: 'Appliance Id'}
                }
                ,
            "system_name": {
                "displayName": "APPLIANCE NAME",
                "visible": true,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Appliance Name'}
                },
            "id": {
                "displayName": "ID",
                "visible": false,
                "filter": {placeholder: 'Id'}
                },
            "type": {
                "displayName": "TYPE",
                "visible": false,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Type'}
                },
            "protocol": {
                "displayName": "PROTOCOL",
                "visible": false,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Protocol'}
                },
            "usage": {
                "displayName": "USAGE",
                "visible": false,
                "filter": {placeholder: 'Usage'}
                },
            "online": {
                "displayName": "ONLINE",
                "visible": false,
                "filter": {placeholder: 'Online'}
                },
            "average_write_speed": {
                "displayName": "AVERAGE WRITE SPEED",
                "visible": false,
                "filter": {placeholder: 'Average_Write Speed'}
                },
            "average_written_daily": {
                "displayName": "AVERAGE_WRITTEN_DAILY",
                "visible": false,
                "filter": {placeholder: 'Average Written Daily'}
                },
            "daily_change_rate": {
                "displayName": "DAILY CHANGE RATE",
                "visible": false,
                "filter": {placeholder: 'Daily Change Rate'}
                },
            "effective_size": {
                "displayName": "EFFECTIVE SIZE",
                "visible": false,
                "filter": {placeholder: 'Effective Size'}
                },
            "size_history": {
                "displayName": "SIZE HISTORY",
                "visible": false,
                "cellFilter": "arrayToStringFormatter",
                "cellTooltip": true,
                "filter": {placeholder: 'Size History'}
                }
            }
        } ,
         "system_6": {
            "0": {
            "system_name": {
                "displayName": "APPLIANCE NAME",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Appliance Name'}
                },
            "start_date": {
                "displayName": "START DATE",
                "visible": true,
               	"type":"date",
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Start Date'}
                },
            "version_from": {
                "displayName": "START VERSION",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Start Version'}
                },
            "end_date": {
                "displayName": "END DATE",
                "visible": true,
               	"type":"date",
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'End Date'}
                },
            "version_to": {
                "displayName": "END VERSION",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'End Version'}
                },
            "status": {
                "displayName": "STATUS",
                "visible": true,
             
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Status'}
                },
            "messages": {
                "displayName": "MESSAGES",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Messages'}
                }
            }
        },    
        "replication_7": {
            "0": {
            "system_name": {
                "displayName": "APPLIANCE NAME",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Appliance Name'}
                },
            "app_name": {
                "displayName": "APPLICATION NAME",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Application Name'}
                },
            "client_name": {
                "displayName": "PROTECTED ASSET",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Protected Asset'}
                },
            "instance_name": {
                "displayName": "INSTANCE",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Instance'}
                },
            "size": {
                "displayName": "LAST FULL (GB)",
                "visible": true,
                "type": "number",
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Last Full (GB)'}
                }
            }
        },
         "archive_8": {
            "0": {
                "system_name": {
                    "displayName": "APPLIANCE NAME",
                    "visible": false,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Appliance Name'}
                },
                "app_name": {
                    "displayName": "APP NAME",
                    "visible": true,

                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'App Name'}
                },
            "archive_time": {
                "displayName": "BACKUP COPY TIME",
                "visible": true,
               "type":"date" ,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Backup Copy Time'}
                },
            "elapsed_time": {
                "displayName": "ELAPSED TIME",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Elapsed Time'}
                },
            "client_name": {
                "displayName": "PROTECTED ASSET",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Protected Asset'}
                },
            "instance_description": {
                "displayName": "INSTANCE",
                "visible": true,
                width:"**",
                cellTooltip: true,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Instance'}
                },
            "backup_time": {
                "displayName": "BACKUP DATE",
                "visible": true,
                "type":"date" ,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Backup Date'}
            },
            "type": {
                "displayName": "TYPE",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Type'}
                },
            "compressed": {
                "displayName": "COMPRESSED?",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Compressed?'}
                },
			"description": {
                "displayName": "JOB",
                "visible": true,
                "filter": {placeholder: 'Job'},
                "cellFilter": "nullTestFilter"
                },
            "target": {
                "displayName": "TARGET",
                "visible": true,
                "filter": {placeholder: 'Target'},
                "cellFilter": "nullTestFilter"
                },
            "deduped": {
                "displayName": "DEDUPLICATED?",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Deduplicated?'}
                },
            "serials": {
                "displayName": "SERIALS",
                "visible": true,
                "cellFilter": "nullTestFilter",
                "cellTooltip": true,
                "filter": {placeholder: 'Serials'}
                },
            "label": {
                "displayName": "LABEL",
                "visible": false,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: "Label"}
                },
            "barcodes": {
                "displayName": "BARCODES",
                "visible": false,
                "cellFilter": "nullTestFilter",
                "cellTooltip": true,
                "filter": {placeholder: 'Barcodes'}
                },
            "retention_days": {
                "displayName": "RETENTION DAYS",
                "visible": true,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Retention Days'}
                }
             }
            
        },
         "replication_9": {
            "0": {
            "system_name": {
                "displayName": "SOURCE APPLIANCE",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Source Appliance'}
                },
            "client_name": {
                "displayName": "PROTECTED ASSET",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Protected Asset'}
                },
         "status": {
                "displayName": "STATUS",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Status'}
                },
            "app_name": {
                "displayName": "APPLICATION",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Application'}
                },
            "database_name": {
                "displayName": "DATABASE/VM",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Database/VM'}
                },
            "replication_elapsed_time": {
                "displayName": "ELAPSED",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Elapsed'}
                },
            "type": {
                "displayName": "TYPE",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Type'}
                },
			"size": {
                "displayName": "SIZE (GB)",
                "visible": true,
                
                "cellFilter": "MBToGBFormater",
                "filter": {placeholder: 'Size (GB)'}
                },
           "replicated_size": {
                "displayName": "COPY SIZE (GB)",
                "visible": true,
                
                "cellFilter": "MBToGBFormater",
                "filter": {placeholder: 'Copy Size (GB)'}
                },
            "size": {
                "displayName": "SIZE (GB)",
                "visible": false,
                "cellFilter": "MBToGBFormater",
                "filter": {placeholder: 'Size (GB)'}
            },
			 "replication_status": {
                "displayName": "COPY STATUS",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Copy Status'}
                },
            "instance_name": {
                "displayName":"INSTANCE NAME",
                    "visible": true,
                   "cellTooltip": function(row){ return row.entity.instance_name; }, 
                    "filter": {placeholder: 'Instance Name'}
                },
            "replication_start_date": {
                "displayName": "Copy Start Date",
                "visible": true,
               "type":"date" ,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Start Date'}

                },
            "replication_end_date": {
                "displayName": "Copy End Date",
                "visible": true,
               "type":"date" ,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'End Date'}

                },
                "speed": {
                    "displayName": "SPEED (MB/s)",
                    "visible": false,
                    "type": "number",
                    "cellFilter": "DecimalFormatter",
                    "filter": {placeholder: 'speed (MB/s)'}
                }
            },
            "1": {
            "system_name": {
                "displayName": "APPLIANCE NAME",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Appliance Name'}
                },
            "client_name": {
                "displayName": "PROTECTED ASSET",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Protected Asset'}
                },
         "status": {
                "displayName": "STATUS",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Status'}
                },
            "app_name": {
                "displayName": "APPLICATION",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Application'}
                },
            "database_name": {
                "displayName": "DATABASE/VM",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Database/VM'}
                },
            "replication_elapsed_time": {
                "displayName": "ELAPSED",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Elapsed'}
                },
            "type": {
                "displayName": "TYPE",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Type'}
                },
           "replicated_size": {
                "displayName": "COPY SIZE (GB)",
                "visible": true,

                "cellFilter": "MBToGBFormater",
                "filter": {placeholder: 'Copy Size (GB)'}
                },
             "replication_status": {
                "displayName": "COPY STATUS",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Copy Status'}
                },
            "instance_name": {
                "displayName":"INSTANCE NAME",
                    "visible": true,
                   "cellTooltip": function(row){ return row.entity.instance_name; }, 
                    "filter": {placeholder: 'Instance Name'}
                },
            "replication_start_date": {
                "displayName": "Copy Start Date",
                "visible": true,
               "type":"date" ,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Start Date'}

                },
            "replication_end_date": {
                "displayName": "Copy End Date",
                "visible": true,
               "type":"date" ,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'End Date'}

                },
            "source_system_name": {
               displayName:"SOURCE APPLIANCE",
               "visible": true,
               "cellTooltip": function(row){ return row.entity.source_system_name; }, 
               "filter": {placeholder: 'Source Appliance'}
                 },
 	     "target_system_name": {
               displayName:"Target APPLIANCE",
               "visible": true,
               "cellTooltip": function(row){ return row.entity.target_system_name; }, 
               "filter": {placeholder: 'Target Appliance'}
                 }        
            }
        },
         "restore_14": {
            "0": {
            "system_name": {
                "displayName": "APPLIANCE NAME",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Appliance Name'}
                },
            "client_name": {
                "displayName": "PROTECTED ASSET",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Protected Asset'}
                },
            "id": {
                "displayName": "ID",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Id'}
                },
            "status": {
                "displayName": "STATUS",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Status'}
                },
            "app_name": {
                "displayName": "APPLICATION",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Application'}
                },
            "database_name": {
                "displayName": "DATABASE/VM",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Database/VM'}
                },
            "start_date": {
                "displayName": "START TIME",
                "visible": true,
               "type":"date" ,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Start Time'}

                },
            "end_date": {
                "displayName": "END TIME",
                "visible": true,
               "type":"date" ,
                "cellFilter": "nullTestFilter", 
                 
                "filter": {placeholder: 'End Time'}
                },
            "elapsed_time": {
                "displayName": "ELAPSED",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Elapsed'}
                },
            "files": {
                "displayName": "# FILES",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: '# Files'}
                }
            }
        },
        "restore_25": {
            "0": {
                "job_name": {
                    "displayName": "Job Name",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Job Name'}
                },
                "system_name": {
                    "displayName": "Appliance Name",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Appliance Name'}
                },
                "start_date": {
                    "displayName": "START TIME",
                    "visible": true,
                    "type": "date",
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Start time'}
                },
                "end_date": {
                    "displayName": "END TIME",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'End time'}
                },
                "result": {
                    "displayName": "RESULT",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Status'}
                },
                "operation": {
                    "displayName": "OPERATION",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Operation'}
                },
                "id": {
                    "displayName": "ID",
                    "visible": false,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'id'}
                },
                "job_id": {
                    "displayName": "JOB ID",
                    "visible": false,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Job id'}
                }
            }
        },
            "compliance_26": {
                "0": {
                    "name": {
                        "displayName": "Job Name",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Job Name'}
                    },
                    "system_name": {
                        "displayName": "Appliance Name",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Appliance Name'}
                    },
                    "profie_name": {
                        "displayName": "PROFILE",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Profile'}
                    },
                    "profie_rpo": {
                        "displayName": "RPO",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'RPO'}
                    },
                    "profie_rto": {
                        "displayName": "RTO",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'RTO'}
                    },
                    "rpo_actual": {
                        "displayName": "Actual RPO",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'actual rpo'}
                    },
                    "rto_actual": {
                        "displayName": "Actual RTO",
                        "visible": false,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'actual rto'}
                    },
                    "compliance_status": {
                        "displayName": "Compliance Status",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'compliance status'}
                    }
                }
            },
        "system_16": {
            "0": {
            "system_name": {
                "displayName": "APPLIANCE NAME",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Appliance Name'}
                },
            "name": {
                "displayName": "NAME",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Name'}
                },
            "host_name": {
                "displayName": "HOST NAME",
                "visible": true,

                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Host name'}
                },
            "hypervisor": {
                "displayName": "HYPERVISOR",
                "visible": false,

                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Hypervisor'}
                },
            "application": {
                "displayName": "APPLICATION",
                "visible": true,

                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Application'}
                },
            "num_fulls": {
                "displayName": "# Fulls",
                "visible": false,

                "cellFilter": "nullTestFilter",
                "filter": {placeholder: '# Fulls'}
                },
            "last_full": {
                "displayName": "LAST FULL (GB)",
                "visible": true,
                "type": "number",               
                "cellFilter": "DecimalFormatter",
                "filter": {placeholder: 'Last Full (GB)'}
                }
            }
        },        
        "replication_17": {
            "0": {
            "local_system_name": {
                "displayName": "TARGET APPLIANCE",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Target Appliance'}
                },
            "local_system_version": {
                "displayName": "TARGET VERSION",
                "visible": false,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Target Version'}
                },
            "report_date": {
                "displayName": "REPORT DATA LAST UPDATE",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Report Data Last Update'}
                },
            "system_name": {
                "displayName": "SOURCE APPLIANCE",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Source Appliance'}
                }
                ,
            "data_size": {
                "displayName": "COPIED DATA SIZE",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Copied Data Size'}
                }
            }
        },
        "backup_18": {
            "0": {
            "system_name": {
                "displayName": "APPLIANCE NAME",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Appliance Name'}
                },
            "client_name": {
                "displayName": "PROTECTED ASSET",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Protected Asset'}
                },
            "database_name": {
                "displayName": "VM NAME",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'VM Name'}
                },
                "joborder_name": {
                    "displayName": "Job Name",
                    "visible": true,
                    "cellFilter": "nullTestFilter",
                    "filter": {placeholder: 'Job Name'}

                },
            "id": {
                "displayName": "ID",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Id'}
                },
            "start_date": {
                "displayName": "DATE",
                "visible": true,
                 "type":"date" ,
                 "cellFilter": "nullTestFilter",
                
                "filter": {placeholder: 'Date'}
                },
            "end_date": {
                "displayName": "END TIME",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                
                "filter": {placeholder: 'End Time'}
                },
            "elapsed_time": {
                "displayName": "ELAPSED",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Elapsed'}
                },
            "type": {
                "displayName": "TYPE",
                "visible": true,
               
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Type'}
                },  
            "size": {
                "displayName": "SIZE (GB)",
                "visible": true,
                "type": "number",
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Size (GB)'}
                }

            }
        },        
        "system_19": {
            "0": {
            "severity": {
                "displayName": "SEVERITY",
                "visible": true,
              	"sortingAlgorithm": function(a, b){
						if(a === b) return 0; if(a === 'notice') return 1;
						if(b === 'notice') return -1;if(a === 'warning') return 1;
						if(b === 'warning') return -1;
				},
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Severity'}
                },
            "resolved": {
                "displayName": "RESOLVED",
                "visible": true,
                "cellFilter": "resolvedFilter | nullTestFilter",
                "filterCellFiltered": true,
                "filter": {placeholder: 'Resolved'}
                },
            "system_name": {
                "displayName": "APPLIANCE",
                "visible": true,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Appliance'}
                },
            "source_name": {
                "displayName": "SOURCE",
                "visible": true,

                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Source'}
                },
            "created": {
                "displayName": "CREATED",
                "visible": true,
                 "type":"date" ,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Time'}
                },
            "updated": {
                "displayName": "UPDATED",
                "visible": false,
                 "type":"date" ,
                "cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Updated'}
                },
            "message": {
                "displayName": "MESSAGE",
                "visible": true,
                "cellTooltip": function(row){ return row.entity.message; }, 
				"cellFilter": "nullTestFilter",
                "filter": {placeholder: 'Message'},
                "width":"**"
                }
            }
        },
            "system_20": {
                "0": {
                    "notification_id": {
                        "displayName": "NOTIFICATION ID",
                        "visible": false,

                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Notification ID'}
                    },
                    "event_time": {
                        "displayName": "NOTIFICATION DATE",
                        "visible": true,
                        "type":"date",
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Notification Date'}
                    },
                    "username": {
                        "displayName": "USER",
                        "visible": true,

                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'User'}
                    },
                    "system": {
                        "displayName": "APPLIANCE NAME",
                        "visible": true,

                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Appliance Name'}
                    },
                    "message": {
                        "displayName": "MESSAGE",
                        "visible": true,
                        "cellTooltip": function(row){ return row.entity.message; },
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Message'},
                        "width":"**"
                    },
                    "category": {
                        "displayName": "CATEGORY",
                        "visible": true,

                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Category'}
                    }
                }
            },
            "system_21": {
                "0": {
                    "severity": {
                        "displayName": "Severity",
                        "visible": true,
                        "width": 100,
                        "cellFilter": "severityFilter | nullTestFilter",
                        "filterCellFiltered": true,
                        "filter": {placeholder: 'Severity'}
                    },
                    "traptype": {
                        "displayName": "Type",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Type'}
                    },
                    "status": {
                        "displayName": "Status",
                        "visible": true,
                        "width": 90,
                        "cellFilter": "statusFilter | nullTestFilter",
                        "filterCellFiltered": true,
                        "filter": {placeholder: 'Status'}
                    },
                    "time_sent": {
                        "displayName": "Date",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Date'}
                    },
                    "destination": {
                        "displayName": "Destination",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Destination'}
                    },
                    "description": {
                        "displayName": "Description",
                        "visible": true,
                        "width": "**",
                        "cellTooltip": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Description'}
                    },
                    "oid": {
                        "displayName": "OID",
                        "visible": false,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'OID'}
                    },
                    "community": {
                        "displayName": "Community",
                        "visible": false,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Community'}
                    },
                    "object": {
                        "displayName": "Object",
                        "visible": false,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Object'}
                    },
                    "system_id": {
                        "displayName": "Appliance ID",
                        "visible": false,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Appliance ID'}
                    },
                    "system_name": {
                        "displayName": "Appliance Name",
                        "visible": false,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Appliance Name'}
                    }
                }
            },
            "backup_22": {
                "0": {
                    "system_name": {
                        "displayName": "Appliance Name",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Appliance Name'}
                    },
                    "name": {
                        "displayName": "Asset Name",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Asset Name'}
                    },
                    "last_backups": {
                        "displayName": "Backups",
                        "visible": true,
                        "cellTemplate": backupColoredStatusTemplate,
                        "filter": {
                            selectOptions: selectStatusFilterOptions,
                            type: uiGridConstants.filter.SELECT,
                            condition: selectStatusFilterFunction
                        }
                    },
                    "last_backup_copies": {
                        "displayName": "Backup Copies",
                        "visible": true,
                        "cellTemplate": backupCopyColoredStatusTemplate,
                        "filter": {
                            selectOptions: selectStatusFilterOptions,
                            type: uiGridConstants.filter.SELECT,
                            condition: selectStatusFilterFunction
                        }
                        /*"filter": {placeholder: 'S    M    T    W    T    F    S'}*/
                    }
                }
            },
            "distributed_23": {
                "0": {
                    "date_string": {
                        "displayName": "DATE",
                        "visible": true,
                        "filter": {placeholder: 'Date'}
                    },
                    "name": {
                        "displayName": "NAME",
                        "visible": true,
                        "filter": {placeholder: 'Name'}
                    },
                    "uuid": {
                        "displayName": "ASSET",
                        "visible": true,
                        "filter": {placeholder: 'Asset'}
                    },
                    "gb_used": {
                        "displayName": "SIZE (GB)",
                        "visible": true,
                        "filter": {placeholder: 'Size (GB)'}
                    }
                }
            },
            "distributed_24": {
                "0": {
                    "hostname": {
                        "displayName": "NAME",
                        "visible": true,
                        "filter": {placeholder: 'Name'}
                    },
                    "uuid": {
                        "displayName": "ASSET",
                        "visible": false,
                        "filter": {placeholder: 'Asset'}
                    },
                    "manager": {
                        "displayName": "CUSTOMER",
                        "visible": true,
                        "filter": {placeholder: 'Customer'}
                    },
                    "location": {
                        "displayName": "LOCATION",
                        "visible": true,
                        "filter": {placeholder: 'Location'}
                    },
                    "street": {
                        "displayName": "STREET",
                        "visible": true,
                        "filter": {placeholder: 'Street'}
                    },
                    "city": {
                        "displayName": "CITY",
                        "visible": true,
                        "filter": {placeholder: 'City'}
                    },
                    "state": {
                        "displayName": "STATE",
                        "visible": true,
                        "filter": {placeholder: 'State'}
                    },
                    "zip": {
                        "displayName": "ZIP CODE",
                        "visible": false,
                        "filter": {placeholder: 'Zip Code'}
                    }
                }
            },
            "backup_27": {
                "0": {
                    "system_name": {
                        "displayName": "Appliance Name",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Appliance Name'}
                    },
                    "name": {
                        "displayName": "Asset Name",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Asset Name'}
                    },
                    "client_name": {
                        "displayName": "Host Name",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Host Name'}
                    },
                    "app_name": {
                        "displayName": "Application",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Application'}
                    },
                    "policy_name": {
                        "displayName": "SLA Policy",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'SLA Policy'}
                    },
                    "backup_job_name": {
                        "displayName": "Backup Job",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Backup Job'}
                    },
                    "archive_job_name": {
                        "displayName": "Cold Backup Copy Job",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Cold Backup Copy Job'}
                    },
                    "replication_job_name": {
                        "displayName": "Hot Backup Copy Job",
                        "visible": true,
                        "cellFilter": "nullTestFilter",
                        "filter": {placeholder: 'Hot Backup Copy Job'}
                    }
                }
            }
     };
        var originalReportGridColumnDefinition = jQuery.extend(true, {}, reportGridColumnDefinition);

      	/*
		 * If a report ID was passed in, set it here.
		 */
		$scope.alreadySelected = false;
		if ($routeParams.id != undefined) {
			reportId = parseInt($routeParams.id);
			console.log("report id is + " + reportId);
			$scope.alreadySelected = true;
			reportDays = $routeParams.days;
            report_sid = $routeParams.sid !== undefined ? $routeParams.sid : 0;
		}

		/*
		angular.element(document).ready(function() {
			processReportsPage();
		});
		*/

		// Process the page when loaded.
        $scope.$on('$viewContentLoaded', function() {
            processReportsPage();
        });

        $scope.onReportRowClick = function(row, reportId){

            //GET Appliance ID for selected report
            $http({
                url: "/api/systems",
                method: "get",
                params: {}
            }).then(function(result) {
                var data = result.data;
                if(data!==undefined && data!==null && data!=""){
                    if(data.appliance!==undefined && data.appliance!==null && data.appliance!=""){
                        var applianceId = 0;
                        for(var i=0; i<data.appliance.length; i++){
                            if(data.appliance[i].name === row.entity.system_name){
                                applianceId = data.appliance[i].id;
                                break;
                            }
                        }
                        // These variables MUST BE set to use status.html.
                        // If we want to get copy information, set copies to true, for the replication history report.
                        PHD.appliance_sid = applianceId;
                        backup_ids = row.entity.id;
                        copies = reportId === $scope.REPLICATION_HISTORY || reportId === $scope.ARCHIVE;
                        var title = reportId === $scope.BACKUP_FAILURE ? "Backup Failure Status: " :
                                    reportId === $scope.RESTORE_HISTORY ? "Recovery History Status: " :
                                    reportId === $scope.BACKUP_HISTORY ? "Backup Status: " :
                                    reportId === $scope.REPLICATION_HISTORY ? "Backup Copy to Appliance Status: " :
                                    reportId === $scope.ARCHIVE ? " Backup Copy Cold Targets Status" : "";

                        // New dialog for details display
                        if (reportId === $scope.BACKUP_HISTORY ||
                            reportId === $scope.REPLICATION_HISTORY ||
                            reportId === $scope.ARCHIVE) {
                            var showCopies = copies;
                            var showArchives = reportId === $scope.ARCHIVE;
                            if ($scope.dialogOpen === false || $scope.dialogOpen === undefined) {
                                $scope.dialogOpen = true;
                                ngDialog.open({
                                    template: 'app/reports/details/backup-drilldown.html',
                                    scope: $scope,
                                    data: {'item': row.entity, 'applianceId': applianceId, 'showCopies': showCopies, 'showArchives': showArchives, 'title': title},
                                    overlay:true,
                                    modelDialogId: 'item-details-dialog',
                                    ngDialogStyle:'width:625px;height:600px;',
                                    preCloseCallback: function (value) {
                                        $scope.dialogOpen = false;
                                    }
                                });
                            }
                        }
                        else {
                            // Old dialog for details display.
                            if (PHD.currentDialog === undefined || PHD.currentDialog === null || !(PHD.currentDialog === "temporary" || (PHD.currentDialog.is("#dialog-view-report-status.dialog-box")))) {
                                PHD.currentDialog = "temporary";  // Fix a timing issue caused when status.html load is delayed.  Block any more dialogs until set by PHD.wizard.
                                var shutdownDialog = PHD.wizard("app/inventory/details/status.html", "view-report-status", {
                                    title: title,
                                    dialogClass: "standalone",
                                    height: 500,
                                    helpArticle: PHD.App.getHelpLink("backup", true),
                                    beforeClose: function(event, ui) {
                                        //console.log(PHD.currentDialog.is("#dialog-view-report-status.dialog-box"));
                                        PHD.currentDialog = undefined;
                                    }
                                });
                            }
                        }
                    }
                }
            });
        };
		/*
		 * Wrap the existing jQuery in a function. Call after Angular loads the page.
		 */
		function processReportsPage() {

			var $reportCategories = $(".report-categories"),
			    //$reportForm = $("#reports-form"),
				$reportCategories = $(".report-categories"),
				$reportCategoriesSubElems = $(".report-categories > div"),
				$reportCategory = $(".report-category"),
				$curCategory = $(".reports-cat-backup"),
				$reportCategorySubElems = $(".report-category h2"),
				$reportContainer = $(".report-container"),
				$reportBack = $("#report-back"),
				$reportBackSpan = $("#report-back-span"),
				$availableReports = $("#available-reports"),
				$availableReportsHeader = $availableReports.children("h1"),
				$availableReportsSubElems = $availableReports.find("div"),
				$currentReport = $("#current-report"),
				$reportListbox = $("#id_name");
				$holderID = $("#flot_holder");
				$divTitleId = $(".divTitle");
				
			
				/* Catching current selection of Appliance drop down */
				   $scope.enviornment = { // for reports which having  "All" as well as other appliances .
						currentActivity: {
							id: 0,
							name: "All" 
						}
					};
				    $scope.enviornmentModelWithoutAll = {// for reports which having no "All" appliances.
				        currentActivityData: {
						
						}
				   }; 
            
				//Catching list of all appliance from Backend.
				$scope.envornments =[{"id" : 0,"name": "All"}];//Adding All as default object in json, From backend data not coming as all.
				$scope.enviornmentDataWithoutAll =[{"id" : 0,"name": "All"}];
                reportFactory.async('/portal/api/managers.php?level=devices').then(function(resultData) {
				// $('#applianceId-'+reportId+'').prepend($('#applianceloadMask').html(''));
                    var holdList = {};
                    $scope.index=0;
                    for(var i = 0; i< resultData.data.managers.length; i++){
                        holdList = {
                                    "id":resultData.data.managers[i].device_id,
                                    "name":resultData.data.managers[i].hostname
                                   };
                        $scope.envornments.push(holdList);//Adding all list from json
						$scope.enviornmentDataWithoutAll.push(holdList);//Adding all list from json
                    }
                    $scope.enviornment.currentActivity = $scope.envornments[0];
					console.log($scope.enviornment.currentActivity);
					$scope.enviornmentModelWithoutAll.currentActivityData = $scope.enviornmentDataWithoutAll[1]; 
					$scope.enviornmentDataWithoutAll.splice(0, 1);
                    console.log($scope.enviornmentModelWithoutAll.currentActivityData);
					
				});
				
				/*************** Main report Category  array of object***************************/
                $scope.categoryArray = [];

                if ($scope.isNetworkManager) {
                    var distributedCategory = {
                        id:$scope.DISTRIBUTED_APPLIANCE_CAPACITY,
                        category: "APPLIANCE",
                        catImgClass: "icon-network distributed-reports icon-large"
                    };
                    var statusCategory = {
                        id: $scope.SYSTEM_REPORT,
                        category: "STATUS",
                        catImgClass: "icon-uui-unitrends-appliance2 icon-large"
                    };
                    $scope.categoryArray.push(distributedCategory);
                    $scope.categoryArray.push(statusCategory);
                }

				
				/**** Intalization of SubReport data for Backup Report at very first time ***/
                $scope.subCategoryArray = [
                    {
                        id:$scope.DISTRIBUTED_APPLIANCE_CAPACITY,
                        description:"The protected capacity of remote appliances over time  (minimum range of 3 days must be selected to display graph)",
                        label: "REMOTE APPLIANCE CAPACITY",
                        subCatImgClass: "icon-barchart icon-large",
                        dropDownWithAll: false,
                        open:false
                    },
                    {
                        id:$scope.APPLIANCE_CUSTOMER_LOCATION,
                        description:"A report of each appliance and its associated customer and location information",
                        label: "APPLIANCE CUSTOMER AND LOCATION",
                        subCatImgClass: "icon-barchart icon-large",
                        dropDownWithAll: false,
                        open:false
                    }
                ];
				
				/**** Intalization of array on click of subreport ************************************* ***/

                $scope.getSubReportCategory = function(id){
                    //Adding and removing CSS on click of report category.
                    $("a.reportCat").removeClass("reportSelected");
                    $("#test-"+id+"").addClass("reportSelected");

                    switch(id) {
                        case $scope.BACKUP_REPORT:
                            $scope.subCategoryArray =[ {id:$scope.PROTECTION_SUMMARY,description:"A summary of protection details and lists of all protected and unprotected assets within the selected date range.",label: 'PROTECTION SUMMARY', subCatImgClass:"icon-clipboard icon-large",dropDownWithAll:false,open: false},
                                {id:$scope.BACKUP_HISTORY,description:"All backup results by protected asset within the selected date range.", label: 'BACKUP HISTORY', subCatImgClass:"icon-uui-backup icon-large",dropDownWithAll:false,open:false},
                                {id:$scope.LEGAL_HOLD,description:"A report of all backups currently on legal hold.", label: 'LEGAL HOLD BACKUPS', subCatImgClass:"icon-backup icon-large",dropDownWithAll:false, open:false},
                                {id:$scope.BACKUP_FAILURE,description:"A report of all failed backups.", label: 'BACKUP FAILURES', subCatImgClass:"icon-phd-error icon-large",dropDownWithAll:false, open:false},
                                {id:$scope.WEEKLY_STATUS,description:"A status summary of backups and copies over the last 7 days", label: 'WEEKLY STATUS', subCatImgClass:"icon-phd-warning icon-large",dropDownWithAll:false, open:false}
                            ];
                            $scope.subCategoryArray.push(createPolicyReportEntry($scope.PROTECTION_POLICIES));
                            break;
                        case $scope.RESTORE_REPORT:
                            $scope.subCategoryArray =[ {id:$scope.RESTORE_HISTORY,description:"Recovery operations that occurred within the selected date range. File-level recovery objects created (iSCSI/CIFS) are not included in this report.",label: 'RECOVERY HISTORY', subCatImgClass:"icon-circlearrowup icon-large",open:false},
                            ];
                            break;
                        case $scope.BACKUP_COPY_REPORT:
                            $scope.subCategoryArray =[ {id:$scope.PROTECTION_SUMMARY,description:"A summary of protection details and lists of all protected and unprotected assets within the selected date range.", label: 'PROTECTION SUMMARY',subCatImgClass:"icon-clipboard icon-large", dropDownWithAll:false,open:false},
                                {id:$scope.REPLICATION,description:"The protected capacity, i.e., the size of the last Full backups per asset on the target appliance.", label: 'BACKUP COPY CAPACITY',subCatImgClass:"icon-clipboardalt icon-large", dropDownWithAll:false,open:false},
                                {id:$scope.REPLICATION_HISTORY,description:"Backups that have been copied to a Unitrends appliance target.", label: 'BACKUP COPY - HOT TARGETS',subCatImgClass:"icon-uui-appliance2 icon-large", dropDownWithAll:false,open:false},
                                {id:$scope.REPLICATION_REPORT,description:"Display a report of successful, active, and queued backup copy jobs to an appliance target over the last 24 hours.<br>This report is generated each morning at 9:00.", label: 'BACKUP COPIES - PAST 24 HOURS',subCatImgClass:"icon-clipboard icon-large", dropDownWithAll:false,open:false},
                                {id:$scope.REPLICATION_FOOTPRINT,description:"A report of the amount of space sources are using on a target.", label: 'STORAGE FOOTPRINT REPORT',subCatImgClass:"icon-phd-replicate icon-large",dropDownWithAll:false, open:false},
                                {id:$scope.ARCHIVE,description:"The backup copies to cold storage by protected asset within the selected date range.", label: 'BACKUP COPY - COLD TARGETS',subCatImgClass:"icon-clipboardalt icon-large",dropDownWithAll:false, open:false},
                                {id:$scope.WEEKLY_STATUS,description:"A status summary of backups and copies over the last 7 days", label: 'WEEKLY STATUS', subCatImgClass:"icon-phd-warning icon-large",dropDownWithAll:false, open:false}
                            ];
                            $scope.subCategoryArray.push(createPolicyReportEntry($scope.PROTECTION_POLICIES));
                            break;
                        case $scope.SYSTEM_REPORT:
                            $scope.subCategoryArray =[
                                //{id:$scope.UPDATES, description:"Updates for specified appliances, based on the selected date range.",label: 'UPDATE HISTORY',subCatImgClass:"icon-circlearrowup icon-large",dropDownWithAll:false, open:false},
                                //{id:$scope.SYSTEM_CAPACITY, description:"The protected capacity, i.e., the size of the last Full backups for all protected assets.",label: 'CAPACITY REPORT',subCatImgClass:"icon-uui-storage icon-large",dropDownWithAll:false, open:false},
                                //{id:$scope.SYSTEM_LOAD, description:"A graphical view of appliance load over the selected date range.",label: 'LOAD REPORT',subCatImgClass:"icon-barchart icon-large",dropDownWithAll:true, open:false},
                                {id:$scope.BACKUP_HISTORY,description:"All backup results for monitored or managed appliances over the last 7 days", label: 'BACKUP STATUS', subCatImgClass:"icon-uui-backup icon-large",dropDownWithAll:false,open:false},
                                {id:$scope.ALERT_REPORT, description:"The list of alerts for one or more remote appliances that are being monitored or managed",label: 'ALERTS REPORT',subCatImgClass:"icon-barchart icon-large",dropDownWithAll:false, open:false},
                                //{id:$scope.TRAPS_REPORT, description:"SNMP trap history over the selected date range.",label: 'TRAP HISTORY',subCatImgClass:"icon-barchart icon-large",dropDownWithAll:false, open:false},
                                //{id:$scope.AUDIT_REPORT, description:"System notifications from the audit log.",label: 'NOTIFICATIONS REPORT',subCatImgClass:"icon-uui-storage icon-large",dropDownWithAll:false, open:false}
                            ];
                            break;
                        case $scope.STORAGE_REPORT:
                            $scope.subCategoryArray =[
                                {id:$scope.STORAGE_DETAILS, description:"Details for all configured storage.", label: 'STORAGE',subCatImgClass:"icon-uui-storage icon-large",dropDownWithAll:false, open:false},
                                {id:$scope.STORAGE_DATAREDUCTION, description:"A report of space saved by deduplication and compression.", label: 'DATA REDUCTION',subCatImgClass:"icon-uui-storage icon-large", dropDownWithAll:true,open:false}
                            ];
                            break;
                        case $scope.DISTRIBUTED_APPLIANCE_CAPACITY:
                            $scope.subCategoryArray = [
                                {
                                    id:$scope.DISTRIBUTED_APPLIANCE_CAPACITY,
                                    description:"The protected capacity of remote appliances over time  (minimum range of 3 days must be selected to display graph)",
                                    label: "REMOTE APPLIANCE CAPACITY",
                                    subCatImgClass: "icon-barchart icon-large",
                                    dropDownWithAll: false,
                                    open:false
                                },
                                {
                                    id:$scope.APPLIANCE_CUSTOMER_LOCATION,
                                    description:"A report of each appliance and its associated customer and location information",
                                    label: "APPLIANCE CUSTOMER AND LOCATION",
                                    subCatImgClass: "icon-barchart icon-large",
                                    dropDownWithAll: false,
                                    open:false
                                }
                            ];
                            break;
                    }
                };
				/*** on click  of individual sub report of main report on right hand side****/
				$scope.selectSubReport = function(reportId,reportExpand,subCategoryArray,index){
					$scope.reportvalidFormFlag = false;
                    if(!angular.isUndefined(index)){
                            if(subCategoryArray[index].open !== true){
                                subCategoryArray[index].open = false;		
                            }else{					

                                for(var i=0; i< subCategoryArray.length; i++){

                                    if(subCategoryArray[i].open){
                                        subCategoryArray[i].open =false;
                                        console.log(subCategoryArray[i].id);
                                        $("a.reportSubCat-"+subCategoryArray[i].id+"").removeClass("reportSelected");
                                        $("a.reportSubCat-"+subCategoryArray[i].id+"").addClass("reportUnSelected");

                                        $("#expandId-"+subCategoryArray[i].id+"").removeClass("reportExpanded");
                                        $("#expandId-"+subCategoryArray[i].id+"").addClass("reportExpand");

                                    } 
                                     subCategoryArray[index].open = true;

                                }
                            }					
                    }
					if(reportExpand){
					    $("a.reportSubCat-"+reportId+"").removeClass("reportUnSelected");
						$("a.reportSubCat-"+reportId+"").addClass("reportSelected");
						
						$("#expandId-"+reportId+"").removeClass("reportExpand");
					    $("#expandId-"+reportId+"").addClass("reportExpanded");
					}else{
					    $("a.reportSubCat-"+reportId+"").removeClass("reportSelected");
						$("a.reportSubCat-"+reportId+"").addClass("reportUnSelected");
						
						$("#expandId-"+reportId+"").removeClass("reportExpanded");
						$("#expandId-"+reportId+"").addClass("reportExpand");
					  
					}
					//Initialization o f objects.
				    $reportListbox = $("#id_name-"+reportId+"");
					$reportForm = $("#reports-form-"+reportId+"");
					$holderID = $("#flot_holder");
					$reportListbox.val(reportId);
					console.log($reportForm);
					// Initialization of date Picker.
					console.log($holderID);
					$(".datepicker input:text").datepicker({
						showOn: "both",
						buttonText: '<img src="assets/images/calendar.png" width="24" height="24">',
						dateFormat: DATE_FORMAT
					}).attr("placeholder", DATE_DISPLAY_FORMAT);
					
					selectReport(reportId,$reportListbox); // call for select report 
					
		
				   /* $reportForm.validate({  // Date field validation 
						rules: {
							start_date: {
								dateFmtNotRequired: true,
								"validate-start-date": true
							},
							end_date: {
								dateFmtNotRequired: true
							}
						}
					}).form();*/
					
			    }
				
			 var loadApplianceData;
			 var reportIndex = 0;
			 function selectReport(reportId,$reportListbox) {
				reportIndex = reportId;
				$("#report-description-"+reportId+"").text($scope.descriptions[reportIndex]);
				$("#report-filter-options-"+reportId+"").css("display", ($.inArray(reportIndex, $scope.REPORTS_WITH_NO_DATE_SELECTION) == -1) ? "" : "none");
				$("#report-appliance-options-"+reportId+"").css("display", ($.inArray(reportIndex, $scope.REPORTS_WITH_NO_APPLIANCE_SELECTION) == -1) ? "" : "none");
                $("#report-certifyJob-options-"+reportId+"").css("display", ($.inArray(reportIndex, $scope.REPORTS_WITH_CERTIFY_JOB_SELECTION) != -1) ? "" : "none");
				$(".report-info").css("display", ($.inArray(reportIndex, $scope.REPORTS_WITH_NO_HEADER_INFO) == -1) ? "" : "none");
                var endDateId ='#id_end_date_'+reportId;
				var startDateId ='#id_start_date_'+reportId;
				if ($.inArray(reportIndex, $scope.REPORTS_WITH_MONTH_RANGE) != -1) {
                    // longer default date range for selected reports.
                    $(startDateId).datepicker('setDate', '-1m +1d');
                } else {
                    $(startDateId).datepicker('setDate', new Date());
                }
				$(endDateId).datepicker('setDate', new Date());
				$(startDateId).datepicker('option',{maxDate: new Date()});
				$(endDateId).datepicker('option',{maxDate: new Date()});
                 if (reportId === $scope.REPLICATION) {
                     // default to "All" appliances for replication (capacity) report.
                     $scope.enviornment.currentActivity[reportId]=$scope.envornments[0];            
                 } else {
                     $scope.enviornment.currentActivity[reportId]=$scope.envornments[$scope.index+1];                   
                 }
				$scope.enviornmentModelWithoutAll.currentActivityData[reportId]=$scope.envornments[$scope.index+1];
            } 
            $reportBackSpan.click(
				function () {
				    //added
					$(".reportCategory").removeClass("hidden");
					$(".report-section-right").removeClass("hidden");
					$reportBack.addClass("hidden");
					$(this).addClass("hidden");
					$("#report-data").addClass("hidden");
					$("#flot_holder").empty();
					 $holderID.addClass("hidden");
								
					$reportCategories.removeClass("hidden");
					$reportContainer.removeClass("hidden");
					$availableReports.removeClass("hidden");
					
					//From dashboard when we select any backup,failure,protected or unprotected report.
					if (!$scope.alreadySelected) {
                        $reportForm.removeClass("hidden");
                    } else {
                        // When going back, clear alreadySelected flag, and re-set the route to reports.
                        $scope.alreadySelected = false;
                        $location.path('/reports');
                        $scope.selectSubReport(reportId);
                        $scope.getSubReportCategory(reportId);
                        if (reportDays !== "0") {
                            var start_date = moment().subtract(reportDays, 'days').format('MM/DD/YYYY');
                            $("#id_start_date_"+reportId+"").val(start_date);
                        } else {
                            $("#id_start_date_"+reportId+"").val(null);
                        }
                    }
					
				}
			);
		
		
			$availableReportsHeader.click(
				function () {
					$availableReportsHeader.removeClass("report-hover");
					$availableReportsSubElems.addClass("hidden");
					$curCategory.removeClass("hidden");
					$reportContainer.addClass("hidden");
				}
			);


            /*Function for initialize all array and variables to default which were
             *used for creation of dynamic ng-grids
             */
            $scope.intializeGrid = function(){
                angular.forEach($scope.dataGrids,function(value,index){
                    //$scope.dataGrids[index].grid ={};
                    $scope.parsedBackendData[index]=[];
                    $scope.tempDataArray[index]=[];//TODO
                    $scope.allDataArray[index] = [];

                });
                $scope.dataGrids=[];
				$scope.gridList=[];
				$scope.reportHeaders=[];
                $scope.parsedBackendData=[];
                $scope.tempDataArray=[];
                $scope.allDataArray = [];
                $scope.pagingOptions = [];
                $scope.totalServerItemsArray=[];
                pagingOptionsObj={};
                cacheParseDataArray = [];
                index = 0;
                gridReportData = "";
            };

            /*Function for calculating date range text which are present in header of each report
             *start_date_val: Selected Start date
             *end_date_val: Selected end date
             */
            $scope.getDateRange = function( start_date_val,end_date_val){
                if(start_date_val == null && end_date_val== null){
                    $scope.report_date_range = "Showing All";
                }else if(start_date_val != null && end_date_val== null){
                    $scope.report_date_range = "equal to or after "+start_date_val;
                }else if(start_date_val == null && end_date_val != null){
                    $scope.report_date_range = "equal to or before "+end_date_val;
                }else{
                    $scope.report_date_range = start_date_val+" - "+end_date_val;
                }
                return $scope.report_date_range;
            };
			/*downloadReports*/

			$scope.exportFiltered = false;
            $scope.getExportOptions = function(type) {
                $scope.exportFormat = type;
                ngDialog.open({
                    template: 'app/reports/exporter.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId: 'exporter-dialog',
                    ngDialogStyle: 'width:400px;height:250px',
                    preCloseCallback: function (value) {
                    }
                });
            };

            var pdfLoad = null;
            $scope.downloadReports = function(output_format, filteredRows){
                var exportFiltered = filteredRows || false;
                var index=$scope.reportHeaders.indexOf($scope.reportFormat.pdf);
                var errorMessage = "Please wait until the report has completed before exporting it.";
                if(output_format == 'pdf'){
                    console.log(index);
                    console.log($scope.gridList);
                    pdfLoad = PHD.showLoadingIndicator($reportContainer, true, "Exporting report to PDF.  Please wait....");
                    $timeout(function() {
                        var gridToExport = eval($scope.gridList[index]);
                        if (gridToExport !== undefined) {
                            // the ui-grid new option exporterPdfCallback has been set to $scope.downloadPDFComplete, so that function runs
                            // when this is done.
                            var rowTypes = exportFiltered ? uiGridExporterConstants.FILTERED : uiGridExporterConstants.ALL;
                            gridToExport.exporter.pdfExport(rowTypes, uiGridExporterConstants.VISIBLE);
                        } else {
                            ngDialog.open({
                                dialogType: 'Error',
                                dialogMessage: errorMessage
                            });
                            PHD.hideLoadingIndicator(pdfLoad);
                        }
                    }, 250);
                }else{
                    var gridToExportCSV = jQuery.extend(true, {}, $scope.gridList[index]);
                    if (gridToExportCSV !== undefined) {
                        // UNIBP-11108 We need to change the way that the Size History Column is sent to csv, because it is unreadable if it is unformatted
                        // It is being edited here to avoid disrupting the logic that surrounds this field elsewhere in the UI
                        if ( $scope.reportTitle == reportIdtoReportTitleMap[$scope.STORAGE_DETAILS] ) {
                            if ( typeof (gridToExportCSV.grid.columns) !== undefined && typeof (gridToExportCSV.grid.rows) !== undefined) {
                                var isSizeHistoryColumnVisible = false;
                                var columnsLength = gridToExportCSV.grid.columns.length;
                                var sizeHistoryColumnIndex = -1;
                                for(var i=0; i < columnsLength; i++){
                                    // If the size column is visible, manipulate the report before exporting via CSV
                                    if ( gridToExportCSV.grid.columns[i].field == "size_history" && gridToExportCSV.grid.columns[i].visible === true  ){
                                        // Copy all of the existing variables so that the pdf functionality is not disrupted
                                        var tempGrid = jQuery.extend(true, {}, gridToExportCSV.grid);
                                        gridToExportCSV.grid = tempGrid;
                                        var tempColumns = jQuery.extend(true, [], gridToExportCSV.grid.columns);
                                        gridToExportCSV.grid.columns = tempColumns;
                                        var tempSizeHistoryColumn = jQuery.extend(true, {}, gridToExportCSV.grid.columns[i]);
                                        gridToExportCSV.grid.columns[i] = tempSizeHistoryColumn;
                                        var tempRenderContainers = jQuery.extend(true, {}, gridToExportCSV.grid.renderContainers);
                                        gridToExportCSV.grid.renderContainers = tempRenderContainers;
                                        var tempRenderContainersBody = jQuery.extend(true, {}, gridToExportCSV.grid.renderContainers.body);
                                        gridToExportCSV.grid.renderContainers.body = tempRenderContainersBody;
                                        var tempVisibleColumnsCache = jQuery.extend(true, [], gridToExportCSV.grid.renderContainers.body.visibleColumnCache);
                                        gridToExportCSV.grid.renderContainers.body.visibleColumnCache = tempVisibleColumnsCache;

                                        // If the size column is visible, add 4 new columns to the CSV gridToExportCSV instead
                                        isSizeHistoryColumnVisible = true;
                                        sizeHistoryColumnIndex = i + 0;
                                        var newColumnNames = ["Date", "Size (MB)", "Used (MB)", "Free (MB)"];
                                        var newColumnKeys = ["day", "day_mb_size", "day_mb_used", "day_mb_free"];
                                        for(var j=0; j < 4; j++){
                                            gridToExportCSV.grid.columns[columnsLength + j] = jQuery.extend(true, {}, gridToExportCSV.grid.columns[i]);
                                            gridToExportCSV.grid.columns[columnsLength + j].displayName = newColumnNames[j];
                                            gridToExportCSV.grid.columns[columnsLength + j].field = newColumnKeys[j];
                                            gridToExportCSV.grid.columns[columnsLength + j].name = newColumnKeys[j];
                                            gridToExportCSV.grid.columns[columnsLength + j].colDef.displayName = newColumnNames[j];
                                            gridToExportCSV.grid.columns[columnsLength + j].colDef.field = newColumnKeys[j];
                                            gridToExportCSV.grid.columns[columnsLength + j].colDef.name = newColumnKeys[j];
                                            gridToExportCSV.grid.columns[columnsLength + j].cellValueGetterCache = false;
                                            gridToExportCSV.grid.renderContainers.body.visibleColumnCache.push( gridToExportCSV.grid.columns[columnsLength + j] );
                                        }
                                        
                                        //Make the size history column no longer visible in the export so that it does not show up
                                        gridToExportCSV.grid.columns[i].visible = false;
                                        for(var k=0; k < gridToExportCSV.grid.renderContainers.body.visibleColumnCache.length; k++) {
                                            if( gridToExportCSV.grid.renderContainers.body.visibleColumnCache[k].field == "size_history" ) {
                                                var tempCachedSizeHistoryColumn = jQuery.extend(true, {}, gridToExportCSV.grid.renderContainers.body.visibleColumnCache[k]);
                                                gridToExportCSV.grid.renderContainers.body.visibleColumnCache[k] = tempCachedSizeHistoryColumn;
                                                gridToExportCSV.grid.renderContainers.body.visibleColumnCache[k].visible = false;
                                                break;
                                            }
                                        }
                                        break;
                                    }
                                }

                                // We need to change the way that the Size History Column is sent to csv.  If it is not visible, it will not be exported.  Only eidt if that column is visible
                                if ( isSizeHistoryColumnVisible === true ) {
                                    var newRows = [];
                                    var tempRows = jQuery.extend(true, [], gridToExportCSV.grid.rows);
                                    gridToExportCSV.grid.rows = tempRows;
                                    for(var m=0; m < gridToExportCSV.grid.rows.length; m++){
                                        if ( typeof (gridToExportCSV.grid.rows[m].entity.size_history.length) !== undefined ) {
                                            for(var n=0; n < gridToExportCSV.grid.rows[m].entity.size_history.length; n++){
                                                // Add a new row for every day in the size history array.  The new row will look like the old row with only one day of information
                                                var tempNewRow = jQuery.extend(true, {}, gridToExportCSV.grid.rows[m]);
                                                tempNewRow.entity.day = gridToExportCSV.grid.rows[m].entity.size_history[n].day;
                                                tempNewRow.entity.day_mb_size = gridToExportCSV.grid.rows[m].entity.size_history[n].mb_size;
                                                tempNewRow.entity.day_mb_used = gridToExportCSV.grid.rows[m].entity.size_history[n].mb_used;
                                                tempNewRow.entity.day_mb_free = gridToExportCSV.grid.rows[m].entity.size_history[n].mb_free;
                                                newRows.push(tempNewRow);
                                            }
                                        }
                                    }
                                    if (newRows.length > 0) {
                                        gridToExportCSV.grid.rows = newRows;
                                    }
                                }
                            }
                        }
                        if ( isSizeHistoryColumnVisible === true ) {
                            gridToExportCSV.exporter.explicitCsvExport( gridToExportCSV.grid, uiGridExporterConstants.ALL, uiGridExporterConstants.VISIBLE );
                        } else {
                            gridToExportCSV.exporter.csvExport(uiGridExporterConstants.ALL, uiGridExporterConstants.VISIBLE);
                        }
                    } else {
                        ngDialog.open({
                            dialogType: 'Error',
                            dialogMessage: errorMessage
                        });
                    }
                }
            };
            /*
             * Callback function passed to the PDF exporter once complete.
             */
            $scope.downloadPDFComplete = function() {
                PHD.hideLoadingIndicator(pdfLoad);
            };
			/*Function for creating report*/
            $scope.generateReport= function (output_format,subReportID){


                var reportFormValidFlag = true;
                var enviornmentName="All";

                $(".reportCategory").addClass("hidden");

                if($scope.alreadySelected){

                    reportFormValidFlag =true;
                }else{
                    reportFormValidFlag = $reportForm.valid();
                }
                if(reportFormValidFlag) {
                    if (typeof output_format === 'undefined' || output_format === null || output_format === '') {
                        $scope.systemCapacityReport = false;
                        $scope.isHide = false;
                        $scope.replicationCapacityReport = false;

                        $scope.intializeGrid(); // For initalizing all grid scope variables.

                        if(output_format != '' ){reportId = $reportListbox.val();
                        }else{reportId = subReportID;}

                        if ($scope.APPLIANCE_CUSTOMER_LOCATION === reportId || $scope.DISTRIBUTED_APPLIANCE_CAPACITY === reportId) {
                            enviornmentId = 0;
                        } else if($scope.STORAGE_DATAREDUCTION == reportId  || reportId == $scope.SYSTEM_LOAD ){// for reports which having no "All" appliances.
                            enviornmentId = $scope.enviornmentModelWithoutAll.currentActivityData[reportId].id;
                            enviornmentName = $scope.enviornmentModelWithoutAll.currentActivityData[reportId].name;
                        }else{ // for reports which having "All" as well as other appliances.
                            if($scope.enviornment.currentActivity[reportId] != undefined){
                                enviornmentId = $scope.enviornment.currentActivity[reportId].id;
                                enviornmentName = $scope.enviornment.currentActivity[reportId].name;
                            }
                        }

                        if(reportId == $scope.SYSTEM_CAPACITY) {
                            $scope.systemCapacityReport = true;
                        }

                        if (reportId == $scope.REPLICATION) {
                            $scope.replicationCapacityReport = true;
                        }

                        if($scope.alreadySelected){
                            if (reportDays !== "0") {
                                var start_date = moment().subtract(reportDays, 'days').format('MM/DD/YYYY');
                                start_date_val = start_date;
                                end_date_val = null;
                            } else {
                                start_date_val = null;
                                end_date_val = null;
                            }
                            enviornmentId = report_sid;
                            if (enviornmentId == $rootScope.local.id) {
                                enviornmentName = $rootScope.local.name;
                            }
                        }else{
                            start_date_val=($("#id_start_date_"+reportId+"").val() == "") ? null : $("#id_start_date_"+reportId+"").val() ;
                            end_date_val=($("#id_end_date_"+reportId+"").val() == "") ? null : $("#id_end_date_"+reportId+"").val() ;
                        }


                        /** Start date and end date validation ***/
                        var start = $("#id_start_date_"+reportId+"").val();
                        var end = 	$("#id_end_date_"+reportId+"").val();
                        $scope.reportvalidFormFlag = false;

                        if(start && end){
                            if(Date.parse(end) < Date.parse(start)){
                                $(".reportCategory").removeClass("hidden");
                                $scope.reportvalidFormFlag = true;
                                return;
                            }else{
                                $(".reportCategory").addClass("hidden");
                            }
                        }
                        /** End of  Start date and end date validation*/

                        if (($.inArray(reportIndex, $scope.REPORTS_WITH_NO_DATE_SELECTION) != -1)) {
                            start_date_val = null;
                            end_date_val = null;
                            $scope.isHide = true;
                        }

                        if (($.inArray(reportIndex, $scope.REPORTS_WITH_NO_EXPORTS) != -1)) {
                            $("#report-export-pdf2").hide();
                            $("#report-export-csv2").hide();
                        }
                        else{
                            if (($.inArray(reportIndex, $scope.REPORTS_WITH_CSV_EXPORT_ONLY) !== -1)) {
                                $("#report-export-pdf2").hide();
                            } else {
                                $("#report-export-pdf2").show();
                            }
                            $("#report-export-csv2").show();
                        }
                        /* For Report header */
                        $scope.reportdesc  = $scope.descriptions[reportId];
                        $scope.reportTitle = $scope.names[reportId];
                        var dateRange = $scope.getDateRange( start_date_val,end_date_val);
                        $scope.report_date_range = dateRange;
                        $scope.current_time = getCurrentTimeAndDate();
                        $scope.applianceName = enviornmentName;
                    }	 /*end*/

                    $reportContainer.removeClass("hidden");
                    $("#report-data").removeClass("hidden");
                    load = PHD.showLoadingIndicator($reportContainer);

                    $scope.hideGrid = false;
                    configParams = getReportConfigs($scope, reportId,$scope.isCE);
                    $scope.heading = configParams[2];
                    if (typeof output_format !== 'undefined') {
                        extra_params = { format: output_format };
                    } else {
                        extra_params = null;
                    }

                    reportFactory.async(configParams[0],start_date_val,end_date_val,enviornmentId,$scope.names[reportId],extra_params,$analytics).then(function(result){
                        PHD.hideLoadingIndicator(load);
                        if (typeof output_format === 'undefined' || output_format === null || output_format === '') {
                            // The backup copy (was) replication activity report is html, otherwise, parse the data.
                            if (reportId != $scope.ARCHIVE
                                && reportId != $scope.REPLICATION_REPORT
                                && reportId != $scope.REPLICATION_HISTORY
                                && reportId != $scope.REPLICATION_FOOTPRINT)
                            {
                                var gbSize=0;
                                for(var i=0; i<result.data.data.length; i++){
                                    if(result.data.data[i].size !==undefined){
                                        gbSize = (result.data.data[i].size / 1024).toFixed(2);
                                        result.data.data[i].size = gbSize;
                                    }
                                }
                                $scope.resultData = result.data.data;
                            }
                            else{
                                $scope.resultData = result.data;
                            }

                            if (configParams[1] == "PieChart" ){ //TODO
                                console.log("pieChart");
                                $holderID.removeClass("hidden");
                                PieChartRenderProtSummary(result, $scope.isCE);
                                $scope.colDefArray = configParams[3]; //getColumnArray
                                $scope.populateGridData($scope.resultData,$scope.colDefArray);
                            }
                            else if (configParams[1] == "Pie-RDRAssuranceChart" ){ //TODO
                                $holderID.removeClass("hidden");
                                createRDRCharts(result.data, $scope.isCE);
                                $scope.colDefArray = configParams[3];
                                $scope.populateGridData($scope.resultData,$scope.colDefArray);
                            }
                            else if (configParams[1] == "Pie-RDRComplianceChart" ){ //TODO
                                $holderID.removeClass("hidden");
                                createRDRComplianceCharts(result.data, $scope.isCE);
                                $scope.colDefArray = configParams[3];
                                $scope.populateGridData($scope.resultData,$scope.colDefArray);
                            }
                            else if (configParams[1] == "NoGrid")
                            {
                                urlToOpen=result.data.html;
                                var wnd = window.open("about:blank", "", "_blank");
                                /*Checking whether pop-up is blocked or not in User browser*/
                                if(!wnd) {
                                    alert('Seems like a pop-up blocker is enabled. Please disable');
                                }
                                else
                                { wnd.document.write(urlToOpen);}
                                $reportBackSpan.trigger( "click" );

                            }
                            else if (configParams[1] == "Pie-BarChart" ){
                                $holderID.removeClass("hidden");
                                PieChartForBackupStorage(result);
                                createBarChartData(result);
                                $scope.colDefArray = configParams[3]; //getColumnArray
                                console.log($scope.colDefArray);
                                $scope.populateGridData($scope.resultData,$scope.colDefArray);
                            }
                            else if (configParams[1] == "Bar-Line" ) { //TODO
                                if (reportId == $scope.SYSTEM_LOAD) {
                                    $holderID.removeClass("hidden");
                                    plotSytemLoadGraph(result.data, start_date_val, end_date_val);
                                }
                                else {
                                    $holderID.removeClass("hidden");
                                    lineOverBarDataReduction(result.data, $scope.isCE);
                                }
                            }
                            else if (configParams[1] == "Line-Graph" ){
                                if (reportId == $scope.DISTRIBUTED_APPLIANCE_CAPACITY)
                                {
                                    $holderID.removeClass("hidden");
                                    $scope.hideGrid = true;
                                    distributedReportsSvc.plotDistributedCapacity(result.data, start_date_val, end_date_val);
                                    $scope.colDefArray = configParams[3];
                                    // Sort the results by date.
                                    $scope.resultData = _.sortBy($scope.resultData, 'date');
                                    $scope.populateGridData($scope.resultData, $scope.colDefArray);
                                }
                            }else{
                                $scope.colDefArray = configParams[3]; //getColumnArray
                                $scope.populateGridData($scope.resultData,$scope.colDefArray);
                            }
                        }
                    });
                    $reportCategories.addClass("hidden");
                    $availableReports.addClass("hidden");
                    $curCategory.addClass("hidden");
                    if(!$scope.alreadySelected) $reportForm.addClass("hidden");
                    $reportBack.removeClass("hidden");
					$reportBackSpan.removeClass("hidden");
                }
            };
					/*function to concatenate  JSON object
                     
                     */
                
                        $.concat||$.extend({concat:function(b,c){var a=[];for(x in arguments)a=a.concat(arguments[x]);return a}});


            /*Function for manipulating JsonData data
             *Note:
             */
            $scope.populateGridData= function (jsonData,columnDefArray){
                 $scope.jsonData=jsonData;
				var listBackups = [];
                if(reportId == $scope.PROTECTION_SUMMARY){
                    
					$scope.tempDataArray.push(jsonData.assets);
					$scope.tempDataArray.push(jsonData.backups.assets);
					$scope.allDataArray.push(jsonData.assets);
					$scope.allDataArray.push(jsonData.backups.assets);
					cacheParseDataArray.push(jsonData.assets);
					cacheParseDataArray.push(jsonData.backups.assets);
					var backupCopyObject =jsonData.backup_copies.assets;
                    console.log(backupCopyObject);
                    console.log(cacheParseDataArray);
					var dataArray=[];
					for (key in backupCopyObject){
						$scope.heading.push("Backup Copies to "+backupCopyObject[key].target);
                      	$scope.tempDataArray.push(backupCopyObject[key].assets);
						$scope.allDataArray.push(backupCopyObject[key].assets);
						cacheParseDataArray.push(backupCopyObject[key].assets);
						
                    }
                    console.log(cacheParseDataArray);
					$scope.parsedBackendData = $scope.tempDataArray;
                }else if (reportId == $scope.ARCHIVE){
                    
                    var archiveList = [];
                    
                    for (var i =0; i<jsonData.data.length; i++){
                        
                        for(var j = 0;j < jsonData.data[i].archives.length;j++){
                            
							jsonData.data[i].archives[j].description 	= jsonData.data[i].description;
							jsonData.data[i].archives[j].target      	= jsonData.data[i].target;
						}
						archiveList.push(jsonData.data[i].archives);
                    }
					var mergeArchivesData=[];
                    for(var i = 0; i < archiveList.length; i++){
						mergeArchivesData = mergeArchivesData.concat(archiveList[i]);
                    }
					$scope.tempDataArray.push(mergeArchivesData);
					$scope.allDataArray.push(mergeArchivesData);
					cacheParseDataArray.push(mergeArchivesData);
					$scope.parsedBackendData = $scope.tempDataArray;
                }
                else if ( reportId == $scope.SYSTEM_CAPACITY && jsonData.length != 0 ){
                    var system_capacityData = [];
                    var capictyReportData=[];
                    var totalLastFull = 0.0;
                    var lastfull =0.0 ;
                    var lastFullArray =[];
                    $scope.usedSpace = 0.0;
                    $scope.availableSpace = 0.0;
                    $scope.irSpace = 0.0;
                    $scope.totalProtectedCapacity = 0.0;
                    for ( i =0 ; i <jsonData.length ; i++)
                    {

                        for (var key in jsonData[i])
                        {
                            if (key === 'used') {
                                $scope.usedSpace += jsonData[i].used;
                                $scope.totalProtectedCapacity += jsonData[i].used;
                            } else if (key === 'available') {
                                $scope.availableSpace += jsonData[i].available;
                            } else if (key === 'ir_used') {
                                $scope.irSpace += jsonData[i].ir_used;
                            } else {
                                if (jsonData[i][key]  instanceof Array){
                                    for(var k=0; k<jsonData[i][key].length;k++){
                                        jsonData[i][key][k]["system_name"] = jsonData[i].system_name;
                                    }
                                }                               
                            }
                        }
                    }
                    $scope.usedSpace = ($scope.irSpace + $scope.usedSpace).toFixed(2);
                    $scope.totalProtectedCapacity = $scope.totalProtectedCapacity.toFixed(2);
                    $scope.availableSpace = $scope.availableSpace.toFixed(2);
                    $scope.irSpace = $scope.irSpace.toFixed(2);
                    for ( i =0 ; i <jsonData.length ; i++)
                    {
                        for (key in jsonData[i])
                        {
                            if (jsonData[i][key]  instanceof Array){
                                // if a protection key -- clients, vms, or apps -- it will be an array, so add to the data.
                                var values = jsonData[i][key];
                                if (values !== undefined) {
                                    for (var v = 0; v < values.length; v++) {
                                        var app = 'File-Level';
                                        if (key == 'vms') {
                                            app = values[v]['hypervisor'];
                                        } else if (key != 'clients') {
                                            app = key;
                                        }
                                        values[v]['application'] = app;
                                    }
                                }
                                system_capacityData = system_capacityData.concat(jsonData[i][key]);
                            }
                        }
                    }

                    for(var i = 0; i < system_capacityData.length; i++)
                    {
                        capictyReportData = capictyReportData.concat(system_capacityData[i]);

                    }

                    /*for (var k  = 0 ; k < capictyReportData.length ; k++  )
                    {
                        lastfull = capictyReportData[k].last_full;
                        totalLastFull = parseFloat(totalLastFull) + parseFloat(lastfull);
                    }
                    parseFloat(totalLastFull) != 0 ? lastFullArray.push(parseFloat(totalLastFull)) : lastFullArray.push("0.0");
                    $scope.lastFullArray = lastFullArray;
                    */
                    cacheParseDataArray = [capictyReportData];
                    $scope.tempDataArray.push(capictyReportData);
                    $scope.parsedBackendData = $scope.tempDataArray;
                    $scope.allDataArray.push(capictyReportData);
                    //$scope.createReportGrid(columnDefArray);

                }else if (reportId == $scope.REPLICATION_FOOTPRINT){
                    for (key in jsonData)
                    {
                        if (jsonData[key]  instanceof Array){
                            for(var k=0; k<jsonData[key].length;k++){
                                jsonData[key][k]["local_system_name"] = jsonData.local_system_name;
                                jsonData[key][k]["local_system_version"] = jsonData.local_system_version;
                                jsonData[key][k]["report_date"] = jsonData.report_date;

                            }
                            $scope.tempDataArray.push(jsonData[key]);
                            $scope.allDataArray.push(jsonData[key]);
                            $scope.parsedBackendData = $scope.tempDataArray;
                            cacheParseDataArray.push(jsonData[key]);
                        }
                    }
                    //$scope.createReportGrid(columnDefArray);
                }
                else if(reportId == $scope.STORAGE_DETAILS){
                    $scope.tempDataArray.push(jsonData);
                    $scope.allDataArray.push(jsonData);
                    $scope.parsedBackendData = $scope.tempDataArray;
                    cacheParseDataArray.push(jsonData);
                    //$scope.createReportGrid(columnDefArray);
                }else if(reportId == $scope.RESTORE_HISTORY){
                    if(jsonData != null && jsonData !=undefined){
                        for(var i =0; i<jsonData.length;i++){

                            if(jsonData[i].instance_name != null &&
                                jsonData[i].instance_name != undefined &&
                                jsonData[i].instance_name != ""){
                                jsonData[i].client_name = jsonData[i].instance_name
                            }
                        }
                    }
                    $scope.allDataArray.push(jsonData);
                    $scope.tempDataArray.push(jsonData);
                    $scope.parsedBackendData = $scope.tempDataArray;
                    cacheParseDataArray.push(jsonData);
                    //$scope.createReportGrid(columnDefArray);
                }
                else if(reportId == $scope.REPLICATION_HISTORY){
                    if(jsonData != null && jsonData !=undefined){
                        if(jsonData.show_source_report){
                            $scope.allDataArray.push(jsonData.source_data);
                            $scope.tempDataArray.push(jsonData.source_data);
                            $scope.parsedBackendData = $scope.tempDataArray;
                            cacheParseDataArray.push(jsonData.source_data);
                        } else {
                            var index = $scope.heading.indexOf("Backup Copies - Source Status");
                            $scope.heading.splice(index, 1);
                        }
                        if(jsonData.show_target_report){
                            $scope.allDataArray.push(jsonData.target_data);
                            $scope.tempDataArray.push(jsonData.target_data);
                            $scope.parsedBackendData = $scope.tempDataArray;
                            cacheParseDataArray.push(jsonData.target_data);
                        } else {
                            var index = $scope.heading.indexOf("Backup Copies - Target Status");
                            $scope.heading.splice(index, 1);
                        }
                        if((!jsonData.show_source_report && !jsonData.show_target_report)){
                            PHD.alert(jsonData.message);
                        }
                    }
                   
                } else if (reportId == $scope.REPLICATION) {
                    $scope.totalProtectedCapacity = 0.0;
                    for (i = 0 ; i < jsonData.length ; i++)
                    {
                        for (var key in jsonData[i])
                        {
                            if (key === 'size') {
                                $scope.totalProtectedCapacity += parseFloat(jsonData[i].size);
                            }
                        }
                    }
                    $scope.totalProtectedCapacity = $scope.totalProtectedCapacity.toFixed(2);
                    $scope.tempDataArray.push(jsonData);
                    $scope.allDataArray.push(jsonData);
                    $scope.parsedBackendData = $scope.tempDataArray;
                    cacheParseDataArray.push(jsonData);
                }
                else{
                    if (jsonData  instanceof Array){
                        console.log("in else11");
                        $scope.allDataArray.push(jsonData);
                        $scope.tempDataArray.push(jsonData);
                        $scope.parsedBackendData = $scope.tempDataArray;
                        cacheParseDataArray.push(jsonData);
                        console.log(cacheParseDataArray);
                    }
                    //$scope.createReportGrid(columnDefArray);
                }
                if($scope.heading[0] == "" || typeof($scope.heading[0]) =='undefined'){
					$scope.reportHeaders=[$scope.names[reportId]]
                }else{
                   for (var i=0;i<  $scope.parsedBackendData.length;i++){
					   if ($scope.heading[i].toString() != 'Backups'){
                        $scope.reportHeaders.push($scope.heading[i]);
                       }
					   
                     }
                }
                $scope.reportFormat.pdf = $scope.reportHeaders[0];
                $scope.getReportColumnDefinitionArray(reportID,reportIdtoReportCategory[reportID]);

            };

            $scope.uiGridProvider = {
                showFailureBackupInfo : function(row) {
                    $scope.onReportRowClick(row, $scope.BACKUP_FAILURE);
                },
                showRecoveryHistoryInfo : function(row) {
                    $scope.onReportRowClick(row, $scope.RESTORE_HISTORY);
                },
                showBackupHistoryInfo : function(row) {
                    $scope.onReportRowClick(row, $scope.BACKUP_HISTORY);
                },
                showReplicationHistoryInfo : function(row) {
                    $scope.onReportRowClick(row, $scope.REPLICATION_HISTORY);
                },
                showBackupStatus: function(element, day) {
                    return $scope.showBackupStatusCell(element, day);
                },
                showStatusDetails: function(row, type, day) {
                    $scope.onBackupStatusDayClick(row, type, day);
                },
                showDayTitle: function(day) {
                    return getDay(day, $scope.dayOfWeek);
                },
                showColdBackupCopyHistoryInfo : function(row) {
                    $scope.onReportRowClick(row, $scope.ARCHIVE);
                },
                showProtectionPolicy : function(row) {
                    $scope.onPolicyRowClick(row, $scope.PROTECTION_POLICIES);
                }
            };
            
            /*
             * Functions for the weekly status report.
             */
            
            // Returns a formatted Day and date, e.g., Sunday, June 13, 2016.
            var dow_weekdays = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];
            function getDay(day, dow) {
                var offset = day - dow;
                if (dow < day) {
                    offset -= 7;    // go to prior week.
                }
                var date = moment().day(dow + offset);
                return dow_weekdays[day] + ", " + date.format(DATE_DISPLAY_FORMAT);
            }
            
            // Returns a status icon for the day, or "none" if on backups (or copies) are available.
            // Sets disabled to false so that the click function will not open a details dialog.
            $scope.showBackupStatusCell = function(element, day) {
                var obj = element[day];
                var status = "none";
                if (obj.successes !== undefined && obj.successes.count > 0) {
                    status = 'success';
                }
                if (obj.warnings !== undefined && obj.warnings.count > 0) {
                    status = 'warning';
                }
                if (obj.failures !== undefined && obj.failures.count > 0) {
                    status = 'error';
                }
                // See if the element should be clickable or not.
                // If the object property is disabled, the ng-click function in the cell template will not open a dialog.
                if (status !== "none") {
                    status += ' day-result'; // so that it is clickable.
                } else {
                    obj.disabled = true;
                }
                return "icon-phd-" + status;
            };
            
            /*
             * If the user clicks on the status icon, this function creates the dialog with summary information.
             */
            $scope.onBackupStatusDayClick = function(row, type, day) {
                var element = type == 0 ? row.last_backups : row.last_backup_copies;
                $scope.dayStatus = element[day];
                if (row.id !== undefined) {
                    var a = row.id.split('_');
                    $scope.dayStatus.sid = a[0];
                } else {
                    $scope.dayStatus.sid = -1;
                }
                $scope.dayStatus.name = row.name;
                $scope.dayStatus.dayString = getDay(day, $scope.dayOfWeek);
                $scope.dayStatus.type = type == 0 ? "Backups" : "Backup Copies";
                
                var dayStatusDialogStyle = "width:auto; height:auto; min-width:260px; min-height:240px";
                if (type != 0) {
                    var targets = $scope.dayStatus.backup_copy_targets;
                    if (targets !== undefined) {
                        for (var targetName in targets) {
                            var obj = targets[targetName];
                            obj.name = targetName;
                        }
                    }
                    dayStatusDialogStyle = "width:auto; height:auto; min-width:300px; min-height:240px";
                }
                ngDialog.open({
                    template: 'app/reports/day-status.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId: 'day-status-dialog',
                    ngDialogStyle: dayStatusDialogStyle
                });
            };

            /*
             * If the user clicks on the details button, this function displays them.  Note: this code is the
             * same as used on the protect page for displaying status.  Over time, this should be moved to a common service.
             */
            $scope.onLoadDailyStatus = function(sid, bid, date, name, type) {
                console.log("ROW Selected");
                ngDialog.close('day-status-dialog');

                PHD.appliance_sid = sid;
                if (type == 'Backups') {
                    backup_ids = bid;
                    copies = false;
                    PHD.wizard("app/inventory/details/status.html", "view-status", {
                        title: gettext(name + " Backup Status: " + date),
                        dialogClass: "standalone",
                        height: 500,
                        helpArticle: PHD.App.getHelpLink("backup", true)
                    });
                } else {
                    console.log("in load backup copy detail status");
                    if (bid.indexOf('a') === -1) {
                        // load backup copies with same API as backups but with an additional option.
                        backup_ids = bid;
                        copies = true;
                        PHD.wizard("app/inventory/details/status.html", "view-status", {
                            title: gettext(name + " Backup Copy Status: " + date),
                            dialogClass: "standalone",
                            height: 500,
                            helpArticle: PHD.App.getHelpLink("backup", true)
                        });
                    } else {
                        copy_ids = bid;
                        copies = true;
                        PHD.wizard("app/inventory/backup-copy-details/status.html", "view-status", {
                            title: gettext(name + " Backup Copy Status: " + date),
                            dialogClass: "standalone",
                            height: 390,
                            helpArticle: PHD.App.getHelpLink("backup", true)
                        });
                    }
                }
            };
            
            /*
             * Before opening the status dialog, this function is called to build up the proper parameters, including the list of
             * backup or backup copy IDs for that day.
             */
            $scope.loadStatusesFromReport = function() {
                console.log("Load Statuses");
                var all_ids = [];
                if ($scope.dayStatus.successes !== undefined && $scope.dayStatus.successes.ids !== undefined) {
                    all_ids = all_ids.concat($scope.dayStatus.successes.ids);
                }
                if ($scope.dayStatus.warnings !== undefined && $scope.dayStatus.warnings.ids !== undefined) {
                    all_ids = all_ids.concat($scope.dayStatus.warnings.ids);
                }
                if ($scope.dayStatus.failures !== undefined && $scope.dayStatus.failures.ids !== undefined) {
                    all_ids = all_ids.concat($scope.dayStatus.failures.ids);
                }
                if ($scope.dayStatus.incomplete !== undefined && $scope.dayStatus.incomplete.ids !== undefined) {
                    all_ids = all_ids.concat($scope.dayStatus.incomplete.ids);
                }
                all_ids = all_ids.toString();
                $scope.onLoadDailyStatus($scope.dayStatus.sid, all_ids, $scope.dayStatus.dayString, $scope.dayStatus.name, $scope.dayStatus.type);
            };

            /*
             * Functions for the weekly status report.
             */

            // Returns a formatted Day and date, e.g., Sunday, June 13, 2016.
            var dow_weekdays = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];
            function getDay(day, dow) {
                var offset = day - dow;
                if (dow < day) {
                    offset -= 7;    // go to prior week.
                }
                var date = moment().day(dow + offset);
                return dow_weekdays[day] + ", " + date.format(DATE_DISPLAY_FORMAT);
            }

            // Returns a status icon for the day, or "none" if on backups (or copies) are available.
            // Sets disabled to false so that the click function will not open a details dialog.
            $scope.showBackupStatusCell = function(element, day) {
                var obj = element[day];
                var status = "none";
                if (obj.successes !== undefined && obj.successes.count > 0) {
                    status = 'success';
                }
                if (obj.warnings !== undefined && obj.warnings.count > 0) {
                    status = 'warning';
                }
                if (obj.failures !== undefined && obj.failures.count > 0) {
                    status = 'error';
                }
                // See if the element should be clickable or not.
                // If the object property is disabled, the ng-click function in the cell template will not open a dialog.
                if (status !== "none") {
                    status += ' day-result'; // so that it is clickable.
                } else {
                    obj.disabled = true;
                }
                return "icon-phd-" + status;
            };

            /*
             * If the user clicks on the status icon, this function creates the dialog with summary information.
             */
            $scope.onBackupStatusDayClick = function(row, type, day) {
                var element = type == 0 ? row.last_backups : row.last_backup_copies;
                $scope.dayStatus = element[day];
                if (row.id !== undefined) {
                    var a = row.id.split('_');
                    $scope.dayStatus.sid = a[0];
                } else {
                    $scope.dayStatus.sid = -1;
                }
                $scope.dayStatus.name = row.name;
                $scope.dayStatus.dayString = getDay(day, $scope.dayOfWeek);
                $scope.dayStatus.type = type == 0 ? "Backups" : "Backup Copies";

                var dayStatusDialogStyle = "width:auto; height:auto; min-width:260px; min-height:240px";
                if (type != 0) {
                    var targets = $scope.dayStatus.backup_copy_targets;
                    if (targets !== undefined) {
                        for (var targetName in targets) {
                            var obj = targets[targetName];
                            obj.name = targetName;
                        }
                    }
                    dayStatusDialogStyle = "width:auto; height:auto; min-width:300px; min-height:240px";
                }
                ngDialog.open({
                    template: 'app/reports/day-status.html',
                    scope: $scope,
                    overlay:true,
                    modelDialogId: 'day-status-dialog',
                    ngDialogStyle: dayStatusDialogStyle
                });
            };

            /*
             * If the user clicks on the details button, this function displays them.  Note: this code is the
             * same as used on the protect page for displaying status.  Over time, this should be moved to a common service.
             */
            $scope.onLoadDailyStatus = function(sid, bid, date, name, type) {
                ngDialog.close('day-status-dialog');

                PHD.appliance_sid = sid;
                if (type == 'Backups') {
                    backup_ids = bid;
                    copies = false;
                    PHD.wizard("app/inventory/details/status.html", "view-status", {
                        title: gettext(name + " Backup Status: " + date),
                        dialogClass: "standalone",
                        height: 500,
                        helpArticle: PHD.App.getHelpLink("backup", true)
                    });
                } else {
                    console.log("in load backup copy detail status");
                    if (bid.indexOf('a') === -1) {
                        // load backup copies with same API as backups but with an additional option.
                        backup_ids = bid;
                        copies = true;
                        PHD.wizard("app/inventory/details/status.html", "view-status", {
                            title: gettext(name + " Backup Copy Status: " + date),
                            dialogClass: "standalone",
                            height: 500,
                            helpArticle: PHD.App.getHelpLink("backup", true)
                        });
                    } else {
                        copy_ids = bid;
                        copies = true;
                        PHD.wizard("app/inventory/backup-copy-details/status.html", "view-status", {
                            title: gettext(name + " Backup Copy Status: " + date),
                            dialogClass: "standalone",
                            height: 390,
                            helpArticle: PHD.App.getHelpLink("backup", true)
                        });
                    }
                }
            };

            /*
             * Before opening the status dialog, this function is called to build up the proper parameters, including the list of
             * backup or backup copy IDs for that day.
             */
            $scope.loadStatusesFromReport = function() {
                var all_ids = [];
                if ($scope.dayStatus.successes !== undefined && $scope.dayStatus.successes.ids !== undefined) {
                    all_ids = all_ids.concat($scope.dayStatus.successes.ids);
                }
                if ($scope.dayStatus.warnings !== undefined && $scope.dayStatus.warnings.ids !== undefined) {
                    all_ids = all_ids.concat($scope.dayStatus.warnings.ids);
                }
                if ($scope.dayStatus.failures !== undefined && $scope.dayStatus.failures.ids !== undefined) {
                    all_ids = all_ids.concat($scope.dayStatus.failures.ids);
                }
                if ($scope.dayStatus.incomplete !== undefined && $scope.dayStatus.incomplete.ids !== undefined) {
                    all_ids = all_ids.concat($scope.dayStatus.incomplete.ids);
                }
                all_ids = all_ids.toString();
                $scope.onLoadDailyStatus($scope.dayStatus.sid, all_ids, $scope.dayStatus.dayString, $scope.dayStatus.name, $scope.dayStatus.type);
            };

            $scope.loadCopyStatusesFromReport = function(targetName) {
                var all_ids = [];
                var targets = $scope.dayStatus.backup_copy_targets;
                for (var thisTarget in targets) {
                    if (targets.hasOwnProperty(thisTarget)) {
                        var target = targets[thisTarget];
                        if (target.name == targetName) {
                            if (target.successes !== undefined && target.successes.ids !== undefined) {
                                all_ids = all_ids.concat(target.successes.ids);
                            }
                            if (target.warnings !== undefined && target.warnings.ids !== undefined) {
                                all_ids = all_ids.concat(target.warnings.ids);
                            }
                            if (target.failures !== undefined && target.failures.ids !== undefined) {
                                all_ids = all_ids.concat(target.failures.ids);
                            }
                            if (target.incomplete !== undefined && target.incomplete.ids !== undefined) {
                                all_ids = all_ids.concat(target.incomplete.ids);
                            }
                            break;
                        }
                    }
                }
                all_ids = all_ids.toString();
                $scope.onLoadDailyStatus($scope.dayStatus.sid, all_ids, $scope.dayStatus.dayString, $scope.dayStatus.name, $scope.dayStatus.type);
            };

            /*
             * If the user clicks on the status icon, this function creates the dialog with summary information.
             */
            $scope.onPolicyRowClick = function(row, type, day) {

                var details = row.entity.details;
                ngDialog.open({
                    template: 'app/reports/details/formatted_details.html',
                    scope: $scope,
                    data: {'details': details, 'row': row.entity, 'specialField': 'calendar_str', 'title': "Protection Policy"},
                    overlay:true,
                    modelDialogId: 'item-details-dialog',
                    ngDialogStyle:'width:625px;height:520px;',
                    preCloseCallback: function (value) {
                        $scope.dialogOpen = false;
                    }
                });

            };

            /*Function for creating grid using manipulated data
             *columnDefArray: column definition array
             */
            $scope.createReportGrid = function (columnDefArray){
                var index = 0;
                $scope.hedeaRforPdf=[];
                $scope.csvFileName=[];
                $scope.headerText=[];
                $scope.pdfFileName=[];
				if (reportId ==$scope.PROTECTION_SUMMARY){
					for (var i=3; i < $scope.parsedBackendData.length ; i++){
					columnDefArray[i]=columnDefArray[2];
					}
				}
                
                if(reportId == $scope.ALERT_REPORT){
                    for(var i=0; i<$scope.parsedBackendData.length; i++){
                        for(var j=0; j<$scope.parsedBackendData[i].length; j++){
                            if($scope.parsedBackendData[i][j].message.length >180){
                                $scope.ROW_HEIGHT = 80;
                            }
                            else if($scope.parsedBackendData[i][j].message.length > 90){
                                $scope.ROW_HEIGHT = 50;
                            }
                        }
                    }
                }else if(reportId !== $scope.ALERT_REPORT || reportId !== $scope.ARCHIVE){
                    $scope.ROW_HEIGHT = 30;
                }
                
                if(reportId == $scope.ARCHIVE){
                      for(var i=0; i<$scope.parsedBackendData.length; i++){
                        for(var j=0; j<$scope.parsedBackendData[i].length; j++){
                             console.log($scope.parsedBackendData[i][j].instance_description);
                             console.log($scope.parsedBackendData[i][j].instance_description.length);
                            if($scope.parsedBackendData[i][j].instance_description.length >80){
                                $scope.ROW_HEIGHT = 60;
                            }
                            else if($scope.parsedBackendData[i][j].instance_description.length > 40){
                                $scope.ROW_HEIGHT = 40;
                            }
                        }
                    }
                }

                if(reportId == $scope.REPLICATION_HISTORY){
                      $scope.ROW_HEIGHT = 60;
			  if($scope.parsedBackendData.length ===1){
                //UNIBP-8306 sometimes for target we get 15 recodes additional param is files, hence we need to make logic >15
			   	if(Object.keys($scope.parsedBackendData[0][0]).length > 15){
					columnDefArray.splice(1, 1);
				}else {
					columnDefArray.splice(0, 1);
				}
			  }	
                 }else{
                     $scope.ROW_HEIGHT = 30;
                }
                
             
	            angular.forEach($scope.parsedBackendData, function(elem) {
                    if((columnDefArray[index]) != undefined){
                        $scope.headerText[index] = ($scope.heading[0] == "" || typeof($scope.heading[0]) =='undefined')?$scope.names[reportId]: $scope.heading[index];
                        $scope.hedeaRforPdf[index] =  $scope.headerText[index] + "\n" + "Created: " + $scope.current_time;
                        $scope.csvFileName[index]=($scope.heading[index] != "")? $scope.heading[index] +".csv" :$scope.names[reportId]+ ".csv";
                        $scope.pdfFileName[index]=($scope.heading[index] != "")? $scope.heading[index] +".pdf" :$scope.names[reportId]+ ".pdf";
                        $scope.dataGridOption={
                            data:$scope.parsedBackendData[index],
                            columnDefs                  : columnDefArray[index],
                            paginationPageSizes         : [10,25, 50, 100],
                            paginationPageSize          : 25,
                            enableFiltering             : true,
                            enableGridMenu              : true,
                            showGridFooter              : true,
							exporterCsvFilename         : $scope.csvFileName[index],
                            exporterPdfFilename         : $scope.pdfFileName[index],
                            exporterPdfCallback         : $scope.downloadPDFComplete,
                            exporterPdfPageSize         : 'LETTER',
                            exporterMenuPdf             : false,
                            exporterMenuCsv             : false,
                            exporterPdfMaxGridWidth     : 500,
                            enableHorizontalScrollbar   : 0,
                            enableVerticalScrollbar     : 0,
                            enableColumnResizing        : true,
                            appScopeProvider            : $scope.uiGridProvider,
                            exporterPdfDefaultStyle     : {fontSize: 8},
                            exporterPdfTableStyle       : {margin: [5, 5, 5, 5]},
                            exporterPdfTableHeaderStyle : {fontSize: 8, bold: true, italics: true, color: 'black'},
                            exporterPdfHeader: { text   : $scope.hedeaRforPdf[index] , style: 'headerStyle' },
                            rowHeight                   : $scope.ROW_HEIGHT,
                            rowTemplate                 :  $scope.getRowTemplate(reportId),
                            exporterPdfFooter : function ( currentPage, pageCount ) {
                                return {
                                    text: currentPage.toString() + ' of ' + pageCount.toString(), style: 'footerStyle'
                                };
                            },
                            exporterPdfCustomFormatter  : function ( docDefinition ) {
                                docDefinition.styles.headerStyle = { fontSize: 10, bold: true, color: 'grey' };
                                docDefinition.styles.footerStyle = { fontSize: 8, bold: true };
                                return docDefinition;
                            },
                            //  exporterCsvLinkElement: angular.element(document.querySelectorAll(".custom-csv-link-location")),

                            onRegisterApi: function(gridApi) {
                                $scope.gridApi = gridApi;
								$scope.gridList.push(gridApi);
								gridApi.core.on.columnVisibilityChanged( $scope, function (column) {
                                    $scope.saveUserPreference(columnDefArray, reportId);
                                });

                                gridApi.core.on.sortChanged($scope, function(grid, sortColumns) {
                                    if (sortColumns.length == 0) {
                                        paginationOptions.sort = null;
                                    } else {
                                        paginationOptions.sort = sortColumns[0].sort.direction;
                                    }
                                });
                                gridApi.pagination.on.paginationChanged($scope, function (newPage, pageSize) {
                                    paginationOptions.pageNumber = newPage;
                                    paginationOptions.pageSize = pageSize;
                                });
                                gridApi.colMovable.on.columnPositionChanged($scope, function (colDef, originalPosition, newPosition) {
                                    if (columnDefArray instanceof Array) {
                                        // Find the column definition array to update, if > 1 (Protection Summary).
                                        // We don't have a grid ID, so we look for the number of columns, which is unique for the multi-grid reports.
                                        // If the same length, we will have to compare the colDefs of each to see if equal (expensive).
                                        for (var i = 0; i < columnDefArray.length; i++) {
                                            var definitions = columnDefArray[i];
                                            if (definitions.length == gridApi.grid.columns.length) {
                                                definitions.splice(originalPosition, 1);
                                                definitions.splice(newPosition, 0, colDef);
                                                // persist new column definitions.
                                                $scope.saveUserPreference(columnDefArray, reportId);
                                                break;
                                            }
                                        }
                                    }
                                });
                            },
                            gridMenuCustomItems: [{
                                title: 'Reset column defaults',
                                action: function($event) {
                                    // clear the current grids, then re-draw as the original.
                                    $scope.dataGrids = [];
                                    $scope.gridList = [];
                                    $scope.saveUserPreference([], reportId, function() {
                                        $scope.getReportColumnDefinitionArray(reportId, reportIdtoReportCategory[reportId], true);
                                    });
                                },
                                order: 99
                            }]
                        };
                        if (reportId == $scope.WEEKLY_STATUS) {
                            /*
                             * Weekly status includes 2 array elements that do not export well, so process them into a textual output.
                             */
                            $scope.dataGridOption['exporterFieldCallback'] = function(grid, row, col, element) {
                                if (col.name === 'last_backups' || col.name === 'last_backup_copies') {
                                    var week_status = "";
                                    if (element instanceof Array) {
                                        for (var day = 0; day < element.length; day++) {
                                            var obj = element[day];
                                            var status = "None";
                                            if (obj.successes !== undefined && obj.successes.count > 0) {
                                                status = 'Success';
                                            }
                                            if (obj.warnings !== undefined && obj.warnings.count > 0) {
                                                status = 'Warning';
                                            }
                                            if (obj.failures !== undefined && obj.failures.count > 0) {
                                                status = 'Error';
                                            }
                                            status = getDay(day, $scope.dayOfWeek) + ': ' + status + '\n';
                                            week_status += status;
                                        }
                                    }
                                    element = week_status;
                                }
                                return element;
                            }
                        } else if (reportId === $scope.REPLICATION_HISTORY || reportID === $scope.STORAGE_DETAILS) {
                            // Any reports that convert MB to GB need to do this on export of data if converted by a formatter.
                            $scope.dataGridOption['exporterFieldCallback'] = function(grid, row, col, element) {
                                if (col.cellFilter !== undefined || col.cellFilter.length > 0) {
                                    var filters = col.cellFilter.split('|');
                                    for (var i = 0; i < filters.length; i++) {
                                        var filterName = filters[i].trim();
                                        if (filterName == 'MBToGBFormater') {
                                            element = $filter(filterName)(element);
                                            break;
                                        }
                                    }
                                }
                                return element;
                            }
                        }

                        $scope.dataGrids.push($scope.dataGridOption);
                        for(var i=0; i<$scope.dataGrids[index].columnDefs.length; i++){
                            $scope.dataGrids[index].columnDefs[i].displayName = angular.uppercase($scope.dataGrids[index].columnDefs[i].displayName.replace("_"," "));
                            
                            if ($scope.dataGrids[index].columnDefs[i].filter.placeholder !== undefined) {
                                $scope.dataGrids[index].columnDefs[i].filter.placeholder = angular.lowercase($scope.dataGrids[index].columnDefs[i].filter.placeholder.replace("_"," "));
                            }
                        }
                        ++index;
                    }
                });
            };

            $scope.getRowTemplate = function(reportId){
                switch (reportId) {
                    case $scope.BACKUP_FAILURE:
                        return failureReportTemplete;
                        break;
                    case $scope.RESTORE_HISTORY:
                        return recoveryHistoryTemplete;
                        break;
                    case $scope.BACKUP_HISTORY:
                        return backupHistoryTemplate;
                        break;
                    case $scope.REPLICATION_HISTORY:
                        return replicationHistoryTemplate;
                        break;
                    case $scope.ARCHIVE:
                        return coldBackupCopyHistoryTemplate;
                        break;
                    case $scope.PROTECTION_POLICIES:
                        return policiesTemplate;
                    default:
                        description = "";
                        break;
                }
            };
			
            $scope.currentSelections= [];
            $scope.showSystemName = function()
            {
                var enviornmentName ="";
                if($scope.STORAGE_DATAREDUCTION == reportId  || reportId == $scope.SYSTEM_LOAD ){
                    enviornmentName = $scope.enviornmentModelWithoutAll.currentActivityData[reportId][reportId].name;
                }else{
                    if($scope.enviornment.currentActivity[reportId] != undefined){
                        enviornmentName = $scope.enviornment.currentActivity[reportId].name;
                    }else{
                        enviornmentName = 'All';
                    }
                }
                console.log(enviornmentName.toUpperCase());

                if (enviornmentName == 'All')
                {
                    return true;

                }
                else{
                    return false;
                }
            };
            /*Report persistence  */
            /*
             *getReportColumnDefinitionArray :- get column list from back-end and compares it from user preference
             *reportID :- which report has been selected
             *category :- Category of the report(i.e. backup,archive,replication,system,restore)
             *return columnDefeinition:- returns column definition for particular report.
             */
             $scope.getReportColumnDefinitionArray = function(reportID,category, reload){
                var columnDefinitionArray=[];
                 var loadOriginalDefinitions = reload || false;
                $http({
                   url: '/portal/api/reports.php?type=list',
                   method: 'get'
                }).then(function(result){
                    var data = result.data;
                   var report_title=reportIdtoReportTitleMap[reportID];
                   var reportColumnList =[];
                   if (data.reports !== undefined && data.reports[category] !== undefined) {
                       for (var i =0;i < data.reports[category].length;i++){
                           if(report_title == data.reports[category][i].title){
                               reportColumnList=data.reports[category][i].cols;
                               console.log("inside category");
                               if(!(data.reports[category][i].extra_cols instanceof Array)){
                                   for (colskey in data.reports[category][i].extra_cols){
                                       reportColumnList[colskey]=data.reports[category][i].extra_cols[colskey];
                                   }
                               }
                               if(reportID ==3){

                               }
                               break;
                           }
                       }
                   }
                    var reportGridColumnDefinitionkey = category+"_"+reportID;
                    console.log(reportGridColumnDefinitionkey);
                    var index=0;
                    var columnDefArray=[];
                    var columnDefArrayList=[];
                    var columListProtectionSummary=["assets", ["backup", "assets"],["backup_copies", "assets", "assets"]];
                    var reportGridColumnDefinitionsToUse = loadOriginalDefinitions ?
                                    jQuery.extend(true, {}, originalReportGridColumnDefinition) : reportGridColumnDefinition;
                    if (parseInt(reportID) == $scope.PROTECTION_SUMMARY){
                        for (var j=0 ; j < columListProtectionSummary.length ;j++){
                            var protectIndex = columListProtectionSummary[j];
                            var colList;
                            // If the index is an array, we need to get the array defined in the first index with properties of the second.
                            if (_.isArray(protectIndex)) {
                                var a = reportColumnList[protectIndex[0]];
                                if (a !== undefined) {
                                    if ( protectIndex.length === 3 ) {
                                        // UNIBP-9347: In the case of ["backup_copies", "assets", "assets"], there are two assets arrays, so this gets reportColumnList["backup_copies"]["assets"]["assets"]
                                        // UNIBP-9347: It is a little messy, but it reduces the chance of regression errors
                                        colList = a[protectIndex[1]][protectIndex[2]];
                                    } else {
                                        colList = a[protectIndex[1]];
                                    }
                                }
                            } else {
                                colList = reportColumnList[protectIndex];
                            }
                            columnDefArray  = $scope.getReportColumnDefinition(reportGridColumnDefinitionsToUse[reportGridColumnDefinitionkey][j],colList);
                            columnDefArrayList.push(columnDefArray);
                        }
                    }else if(parseInt(reportID) === $scope.ALERT_REPORT){
                        reportColumnList={
                            "severity": "string",
                            "resolved": "string",
                            "system_name": "string",
                            "source_name": "string",
                            "created": "date",
                            "updated":"date",
                            "message": "string"
                        }
                        columnDefArray  = $scope.getReportColumnDefinition(reportGridColumnDefinitionsToUse[reportGridColumnDefinitionkey][index],reportColumnList);
                        columnDefArrayList.push(columnDefArray);
                    }else if(parseInt(reportID) === $scope.BACKUP_HISTORY){
                         reportColumnList={
                             "application_name": "string",
                             "client_name": "string",
                             "complete": "boolean",
                             "database_name": "string",
                             "device_id": "string",
                             "elapsed_time": "string",
                             "encrypted": "boolean",
                             "end_date": "string",
                             "instance_name": "string",
                             "size": "long",
                             "start_date": "string",
                             "status": "string",
                             "system_name": "string",
                             "type": "string",
                             "id": "long"
                         }
                         columnDefArray  = $scope.getReportColumnDefinition(reportGridColumnDefinitionsToUse[reportGridColumnDefinitionkey][index],reportColumnList);
                         columnDefArrayList.push(columnDefArray);
                     }
                    else if (parseInt(reportID) === $scope.ARCHIVE) {
                        var columnDefArray=[];
                        var columnDefArrayList=[];
                        //var columListProtectionSummary=["sets","archives"];
						var columListProtectionSummary=["archives"]
                        for (var j=0 ; j < columListProtectionSummary.length ;j++){
						    // Note that the archive report is special and relies on the array defined in reportsUtil.js (not sure why it does this).
                            columnDefArray  = $scope.getReportColumnDefinition(reportGridColumnDefinitionsToUse[reportGridColumnDefinitionkey][j],coldTarget);
							columnDefArrayList.push(columnDefArray);
						}
                    }else if (parseInt(reportID) == $scope.REPLICATION_HISTORY){

                        source_cols= {
                            "app_name": "string",
                            "client_name": "string",
                            "complete": "boolean",
                            "database_name": "string",
                            "elapsed_time": "string",
                            "encrypted": "boolean",
                            "end_date": "string",
                            "instance_name": "string",
                            "replication_status": "string",
                            "size": "long",
                            "replicated_size": "long",
                            "start_date": "string",
                            "replication_start_date": "string",
                            "replication_end_date": "string",
                            "replication_elapsed_time": "string",
                            "speed": "float",
                            "status": "string",
                            "system_name": "string",
                            "type": "string",
                            "certify_status": "string",
                            "id": "long",
                            "verify_status": "string"
                        };

                        target_cols= {
                            "app_name": "string",
                            "client_name": "string",
                            "database_name": "string",
                            "encrypted": "boolean",
                            "instance_name": "string",
                            "replication_status": "string",
                            "replicated_size": "long",
                            "replication_start_date": "string",
                            "replication_end_date": "string",
                            "replication_elapsed_time": "string",
                            "source_system_name": "string",
                            "target_system_name": "string",
                            "type": "string",
                            "id": "long"
                        };

                        columnDefArray  = $scope.getReportColumnDefinition(reportGridColumnDefinitionsToUse[reportGridColumnDefinitionkey][0],source_cols);
                        columnDefArrayList.push(columnDefArray);
                        columnDefArray  = $scope.getReportColumnDefinition(reportGridColumnDefinitionsToUse[reportGridColumnDefinitionkey][1],target_cols);
                        columnDefArrayList.push(columnDefArray);
                    }else if(parseInt(reportID) == $scope.WEEKLY_STATUS) {
                        $scope.dayOfWeek = moment().day();
                        columnDefArray = $scope.getReportColumnDefinition(reportGridColumnDefinitionsToUse[reportGridColumnDefinitionkey][index], reportColumnList);
                        columnDefArrayList.push(columnDefArray);
                    } else if(parseInt(reportID) == $scope.DISTRIBUTED_APPLIANCE_CAPACITY){
                        reportColumnList={
                            "name": "string",
                            "uuid": "string",
                            "gb_used": "number",
                            "date_string": "string"
                        };
                        columnDefArray = $scope.getReportColumnDefinition(reportGridColumnDefinitionsToUse[reportGridColumnDefinitionkey][index],reportColumnList);
                        columnDefArrayList.push(columnDefArray);
                    }else if(parseInt(reportID) == $scope.APPLIANCE_CUSTOMER_LOCATION) {
                        reportColumnList = {
                            "hostname": "string",
                            "uuid": "string",
                            "manager": "string",
                            "location": "string",
                            "street": "string",
                            "city": "string",
                            "state": "string",
                            "zip": "string"
                            //"zip": "string",
                            //"phone": "string"
                        };
                        columnDefArray = $scope.getReportColumnDefinition(reportGridColumnDefinitionsToUse[reportGridColumnDefinitionkey][index], reportColumnList);
                        columnDefArrayList.push(columnDefArray);
                    }
                    else{
                        columnDefArray  = $scope.getReportColumnDefinition(reportGridColumnDefinitionsToUse[reportGridColumnDefinitionkey][index],reportColumnList);
                        columnDefArrayList.push(columnDefArray);
                    }
                    var url = "/portal/api/preferences.php?user=" + encodeURI($rootScope.user.name);
                    $http({
                        url: url,
                        method: 'get'
                    }).then(function(data, status, headers){
                            if(data.data instanceof Array && typeof(data.data[0].dashboard_settings) !='undefined' && data.data[0].dashboard_settings!=""){
                                var jsonUserPrefrence=JSON.parse(data.data[0].dashboard_settings);
                                if (jsonUserPrefrence instanceof Array){
                                    data.data[0].dashboard_settings="";
                                }
                            }
                            if(data.data instanceof Array && data.data[0].dashboard_settings != ""){
                                console.log("data.data[0].dashboard_settings");
                                var stringValue= data.data[0].dashboard_settings;
                                var dataObjprefrence=JSON.parse(stringValue)
                                var dataObj =(typeof(dataObjprefrence.report_preference)!= 'undefined')?dataObjprefrence.report_preference:{};

                                var dataColumnPrefrenceArrayList=dataObj[reportID];
                                if (typeof(dataColumnPrefrenceArrayList) != "undefined") {
                                    for (var i =0 ;i < columnDefArrayList.length ; i++){
                                        var colDefintion=columnDefArrayList[i][1];
                                        var prefrenceObject =dataColumnPrefrenceArrayList[i];
                                        if(typeof(prefrenceObject) != "undefined"){
                                            var processedColumnDefArray =[];
                                            for (var j=0 ; j < prefrenceObject.length  ; j++){
                                                for ( key in prefrenceObject[j]){
                                                    var columnDefinition = colDefintion[key];
                                                    if (typeof(columnDefinition) != "undefined"){
                                                        columnDefinition.field=key;
                                                        columnDefinition.visible=prefrenceObject[j][key];
                                                        processedColumnDefArray.push(columnDefinition);
                                                        delete colDefintion[key];
                                                    }
                                                }
                                            }
                                            for (key in colDefintion){
                                                var columnDefinition = colDefintion[key];
                                                columnDefinition.field=key;
                                                processedColumnDefArray.push(columnDefinition);
                                            }
                                            columnDefinitionArray.push(processedColumnDefArray);
                                        }
                                        else{
                                            columnDefinitionArray.push(columnDefArrayList[i][0]);
                                        }
                                    }
                                } else{
                                    console.log("No dashboard_settings for particular Report");
                                    for (var i=0; i< columnDefArrayList.length ; i++){
                                        columnDefinitionArray.push(columnDefArrayList[i][0]);
                                    }
                                }
                            }else{
                                console.log("No dashboard_settings", columnDefinitionArray);
                                for (var i=0; i< columnDefArrayList.length ; i++){
                                    columnDefinitionArray.push(columnDefArrayList[i][0]);
                                }
                            }
                            isColumnLoaded=true;
                            $scope.createReportGrid(columnDefinitionArray);


                        }
                    );

                }, function(response){
                    console.log(response);
                    var report_title=reportIdtoReportTitleMap[reportID];
                    var reportColumnList =[];
                    console.log("No dashboard_settings", columnDefinitionArray);
                    for (var i=0; i< columnDefArrayList.length ; i++){
                        columnDefinitionArray.push(columnDefArrayList[i][0]);
                    }
                });
             };

            /*
             *getReportColumnDefinitionArray :- creates column definition based on default columnList and
             column definition sent by REST API
             *gridColumnDefinition :-default columnList
             *reportColumnList :- column list sent by REST API
             *return columnDefeinition:- returns column definition for particular report.
             */
            $scope.getReportColumnDefinition = function(gridColumnDefinition,reportColumnList){
                console.log(reportColumnList);
                var coumnList=[];
                var columnDefArray =[];
                var columNameArray=[];
                var columNameobj=[];
                var mainObject = {};
                var displayname ="";
                for (key in reportColumnList){
				    var value_column = gridColumnDefinition[key];
                    var value_column_key = gridColumnDefinition[key];
                    if (typeof(value_column) != "undefined"){

                        mainObject[key] = value_column_key;
                        value_column.field=key;
                        value_column.headerTooltip = true;
                        columnDefArray.push(value_column);

                    }
                    else{
                        if(key == "size"){
                            displayname = key+" (GB)";
                            displayname =displayname.toUpperCase();
                        }
                        else{
                            displayname = key.toUpperCase();
                        }

                        var arrayOBj ={
                            "field":key,
                            "displayName":displayname,
                            "visible":false,
                            "cellFilter": "nullTestFilter",
                            "filter" : {"placeholder":displayname},
                            "headerTooltip": true
                        };
                        var arrayOBjkey ={
                            "displayName":displayname,
                            "visible":false,
                            "cellFilter": "nullTestFilter",
                            "filter" : {"placeholder":displayname}
                        };
                        var str = key;
                        if(key == "size"){
                            arrayOBj.type = 'number';
                        }
                        mainObject[str] = arrayOBjkey;
                        columnDefArray.push(arrayOBj);


                    }
                }
				
                coumnList.push(columnDefArray);
			    coumnList.push(mainObject);
                return coumnList;
            };
			/*
			*saveUserPreference :- saves user preference for report columns
			*columnDefintion :- new column definition created by user
			*reportId :- particular report
			*gridIndexChanged:- which grid has been modified.
			*/
            $scope.saveUserPreference = function(columnDefintion,reportId, followFunction){
                console.log("saveUserPreference...."+reportId);
                console.log(columnDefintion);
                var columnDefintionList=[];
                var columnObject={};
                var follow = followFunction || null;
                for(var j=0; j<columnDefintion.length;j++){
                    columnDefintionList[j]=[];
                    for (var i =0 ; i < columnDefintion[j].length ; i++ ){
                        var columnObj={};
                        columnObj[columnDefintion[j][i].field] = columnDefintion[j][i].visible;
                        columnDefintionList[j].push(columnObj);
                    }
                    columnObject[j]=columnDefintionList[j];
                    console.log("After modified obj");
                }
                console.log(columnObject);
                var dataObj ={};
                dataObj["report_preference"]={};
                dataObj["report_preference"][reportId]={};
                var url = '/portal/api/preferences.php?user=' + encodeURI($rootScope.user.name);
                $http({
                    url: url,
                    method: 'get'
                }).then(function(data, status, headers){

                    if(data.data[0].dashboard_settings != ""){
                        dataObj=JSON.parse(data.data[0].dashboard_settings);
                        // preferenceArray=prefrenceObject.dashboard_prefrence;
                        dataObj["report_preference"]=(typeof(dataObj.report_preference) != "undefined")?dataObj.report_preference:{};
                        console.log(dataObj);
                        dataObj["report_preference"][reportId]=columnObject;
                    }
                    else{
                        console.log("in else1 ++++");
                        console.log(dataObj);
                        dataObj["report_preference"][reportId]=columnObject;
                    }
                    var dataObject={};
                    dataObject["dashboard_settings"]=JSON.stringify(dataObj);
                    $http({
                        url: url,
                        data: dataObject,
                        method: 'PUT'
                    }).then(function(data, status, headers){
                        console.log("success");
                        console.log(data);
                        if (follow != null) {
                            follow();
                        }
                    }), (function(response){
                        console.log(response);
                    });

                }), (function(response){
                    console.log(response);
                });
            };
				

			if ($scope.alreadySelected) {
				$(".report-section-right").addClass("hidden");
				$reportCategories.addClass("hidden");
				$reportCategory.addClass("hidden");
				$availableReports.addClass("hidden");
				//$reportForm.addClass("hidden");
				$reportBack.removeClass("hidden");
				$reportBackSpan.removeClass("hidden");
				$reportContainer.removeClass("hidden");
				//$currentReport.html($("[data-report-id='" + $reportListbox.val() + "']").html());
				// lmc - TODO, need report name and icon.
				//$currentReport.html($scope.names[reportId]);
				$availableReportsSubElems.removeClass("hidden");
				$availableReportsHeader.addClass("report-hover");
				/* alert("in alreadySelected"+reportId); */
				$scope.generateReport('',reportId);
			}
			selectReport();
		}
	}]
);

