
(function ($, PHD, window, undefined) {

    /*
     *
     *   Defines logic for page-level buttons
     *   @param $but - the button element that was clicked
     *   @param $row - the element or element(s) that are considered "selected" by the user
     *   @param tab - the object that owns $but and $row - typically this is the object that called buttonlogic()
     *
     */

    var jobTypeRegex = /[\s|^]job-type-([a-z]+)(?:\s)?/;

    var jobEditSelections = function(rowData, data) {
        var selections = "";
        console.log("in job edit selections, data = ", data);
        if (rowData != null) {
            var inventory = "inventory=";
            var selection = "";
            var system_id = (rowData.sid != undefined) ? rowData.sid : 1;
            var client_id = 0;
            if (data.client_id != undefined) {
                client_id = data.client_id;
            }
            var app_id = 1;
            if (data.app_id !== undefined) {
                app_id = data.app_id;
            }
            var added = false;
            var items = 0;
            if (data.instances != null) {
                added = true;
                var base_inventory = inventory + system_id + "_" + client_id;
                //console.log("data is " + JSON.stringify(data));
                if (data.instances.length > 0) {
                    console.log("app id is " + app_id);
                    for (var i = 0; i < data.instances.length; i++) {
                        var instance = data.instances[i];
                        if (instance.app_id !== undefined){
                            app_id = instance.app_id;
                        }
                        var to_add = base_inventory + "_" + app_id + "_" + instance.id;
                        if (items > 0) {
                            selection += "&";
                        }
                        items++;
                        selection += to_add;
                    }
                }
            }
            if (data.clients != undefined && data.clients != null && data.clients.length > 0) {
                added = true;
                base_inventory = inventory + system_id;
                for (i = 0; i < data.clients.length; i++) {
                    client_id = data.clients[i];
                    to_add = base_inventory + "_" + client_id; // + "_" + "1" + instance.id;
                    if (items > 0) {
                        selection += "&";
                    }
                    items++;
                    selection += to_add;
                }
            }
            if (!added) {
                selection = inventory;
            }
            selections += selection;
        }
        console.log("job edit selections: " + selections);
        return selections;
    };

    var getSelectedBackups = function(tab) {
        var selections_obj = {};
        selections_obj.backups = [];
        var backups = [];
        if (tab.TreeTable !== undefined) {
            var selectedBackups = tab.TreeTable.getSelectionsExtra();
            if (selectedBackups.length > 0) {
                //console.log("selected backups = " + JSON.stringify(selectedBackups));
                var backups_per_instance = new Array();
                selectedBackups.forEach(function (obj, index) {
                    if (obj.backup_id !== undefined) {
                        var instance_id = obj.instance_id;
                        if (!$.isArray(backups_per_instance[instance_id])) {
                            backups_per_instance[instance_id] = new Array();
                        }
                        backups_per_instance[instance_id].push(obj.backup_id);
                    }
                });
                for (var instance in backups_per_instance) {
                    var b = backups_per_instance[instance];
                    b.sort();
                    backups.push(b[b.length - 1]);
                }
            }
            selections_obj.backups = backups;
        }
        //console.log("in: getSelectedBackups: selections object = " + JSON.stringify(selections_obj));
        return selections_obj;
    };

    PHD.buttonLogic = function ($but, $row, tab, id, row_data, analytics, ids, connectionsvc) {
        var rowData = row_data || null;
        var editpath = "", view = "", action = "";
        var $analytics = analytics ? analytics.provider : false;
        var analyticsSource = analytics ? analytics.source : null;
        var selectedIDs = ids || [];
        var elem = angular.element(document.querySelector('[ng-controller=GlobalCtrl]'));
        var $scope = elem.scope();

        $but.off("click click.autoblur").on("click.autoblur", function () {
            $(this).blur();
        });

        if ($but.is(".btn-add-backup")) {
            $but.attr("href", "app/joborders/backup/backup.html")
                .on("click", function (event) {
                    event.preventDefault();
                    $analytics && $analytics.eventTrack('Add Backup Job', {  category: 'Backup', label: analyticsSource});
                    PHD.wizard($but.attr("href"), "add-job", {
                        title: gettext("Create Backup Job"),
                        dialogClass: "dialog-add-job dialog-modal-form",
                        helpArticle: PHD.App.getHelpLink("backup", true),
                        // lmc
                        open: function () {
                            jobEdit = false;
                            jobID = 0;
                            preSelected = $but.data('selections');
                            var dateFormat = $.datepicker.formatDate(DATE_FORMAT, new Date());
                            $("#id_start_date").val(dateFormat);
                            $("#id_name").val("Backup Job");
                        },
                        beforeClose: function (event, ui) {
                            $(".backup-job-popover").remove();
                            $(".datepicker input:text").datepicker("destroy");
                            $(".ui-spinner-input").spinner("destroy");
                            
                            if(resetTour){
                                refreshPage();
                                setTimeout(function(){
                                    var $scope = $("[ng-controller=GlobalCtrl]").scope();
                                    $scope.user.showTour = true;
                                    window.location.href = "#!/dashboard/";
                                    resetTour = false;
                                    $(".btn-pref-cancel").trigger('click');
                                }, 1000);
                            }
                        }
                    });
                });
            return true;
        }

        if ($but.is(".btn-edit-job")) {
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot edit jobs");
                return true;
            }
            if ($row !== null && $row.length && selectedIDs.length < 2) {
                editpath = jobTypeRegex.exec($row.attr("class"))[1];
                var is_replication_job = false;
                if (editpath == "replication") {
                    editpath = "archive";
                    is_replication_job = true;
                }

                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));

                $but.attr("href", "app/joborders/" + editpath + "/" + editpath + ".html")
                    .removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Edit Job', {  category: 'Jobs', label: analyticsSource });

                        /* lmc - don't edit backups with this */
                        if (editpath !== "replication" && editpath !== "archive") {
                            return true;
                        }

                        if(rowData.application_name != "file-level"){
                            $url = "/api/joborders/" + $row.data("dbid") + url_sid;
                            // lmc - need get to fill in html
                            var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                            PHD.Ajax.get($url, load)
                                .done(function (data) {
                                    //PHD.hideLoadingIndicator(load);
                                    var data = data.data[0];    // API returns 1 object.
                                    var selections = jobEditSelections(rowData, data);
                                    var options = rowData;


                                    PHD.wizard($but.attr("href"), "edit-job", {
                                        title: gettext("Edit Job"),
                                        width: 900,
                                        height: 700,
                                        dialogClass: "dialog-add-job dialog-edit-job dialog-modal-form",
                                        helpArticle: PHD.App.getHelpLink(editpath, true),
                                        // lmc
                                        open: function () {
                                            jobEdit = true;
                                            jobID = id;
                                            jobOptions = data;
                                            jobReplication = is_replication_job;
                                            console.log('jobID is ' + jobID + " and name = " + data.name + " and is replication = " + is_replication_job);
                                            $("#id_name").val(data.name);
                                            $("#edit_object_id").text(jobID);
                                            preSelected = selections;
                                            useCopiedAssets = 0;
                                            if (data.copied_assets !== undefined && data.copied_assets === true) {
                                                useCopiedAssets = 1;
                                            }
                                            system_id = $row.data("sid");
                                        },
                                        beforeClose: function (event, ui) {
                                            if (editpath === "backup") {
                                                $(".backup-job-popover").remove();
                                                $(".datepicker input:text").datepicker("destroy").datepicker("widget").remove();
                                                $(".ui-spinner-input").spinner("destroy");
                                            }
                                        }
                                    }, true);

                                });
                        }

                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-edit-policy")) {
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot edit SLA policies");
                return true;
            }
            if ($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot edit SLA policies");
                return true;
            }
            if ($row !== null && $row.length && selectedIDs.length < 2) {
                $but.attr("href", "app/sla-policies/" + selectedIDs[0])
                    .removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-delete-policy")) {
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot delete SLA Policies");
                return true;
            }
            if ($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot delete SLA Policies");
                return true;
            }
            if ($row !== null && $row.length && selectedIDs.length < 2) {
                selectedJob = rowData;
                $but.removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                        if (selectedJob.type == 'policy') {
                            var loadSelector = $row.find("[data-source=\"name\"]");
                            ngDialogService.open({
                                template: 'app/protect/sla-policy/delete-policy.html',
                                scope: $scope,
                                overlay:true,
                                data: {'policyId': $row.data("dbid"), 'selectedPolicy': selectedJob, 'loadSelector': loadSelector, 'analyticsSource': analyticsSource, tab: tab },
                                modelDialogId: 'delete-sla-dialog',
                                name: 'delete-sla-dialog',
                                ngDialogStyle: 'width:500px;height:250px',
                                preCloseCallback: function (value) {
                                }
                            });
                            return true;
                        }
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-create-job")) {
            if($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Create Jobs");
                return true;
            }
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot Create Jobs");
                return true;
            }
            return true;
        }

        if ($but.is(".btn-pause-job")) {
            if($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Pause Jobs");
                return true;
            }
            if ($row !== null && $row.length && !$row.is(".system-true")) {
                $but.removeClass("disabled")
                    .off("click");
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                var resumeURL = "/api/jobs/resume/" + $row.data("dbid") + url_sid;
                var suspendURL = "/api/jobs/suspend/" + $row.data("dbid") + url_sid;
                var analyticsEvent;
                if ($row.is(".job-status-paused")) {
                    $but.attr("href", resumeURL)
                        .html('<i class="icon-play icon-large"></i>Resume');
                    analyticsEvent = "Resume Job";
                } else if ($row.is(".job-status-queued")) {
                    $but.attr("href", suspendURL)
                        .html('<i class="icon-pause icon-large"></i>Pause');
                    analyticsEvent = "Pause Job";
                } else if ($row.is(".job-status-running") && $row.is(".job-type-certification")){
                    $but.attr("href", suspendURL)
                        .html('<i class="icon-pause icon-large"></i>Pause');
                    analyticsEvent = "Pause Job";
                }
                else {
                    $but.disableButton();
                    return true;
                }
                $but.on("click", function (event) {
                    event.preventDefault();
                    $analytics && $analytics.eventTrack(analyticsEvent, {  category: 'Jobs', label: analyticsSource });
                    var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));

                    var request = connectionsvc.createApplianceRequest($but.attr("href"), "PUT");
                    httpService(request).then(function (response) {
                        PHD.hideLoadingIndicator(load);
                        tab.update();
                    }, function(response) {
                        PHD.hideLoadingIndicator(load);
                        var data = response.data;
                        if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                            var message = data.result[0].message;
                            ngDialogService.open({
                                dialogType: 'Error',
                                modelDialogId: 'job-error-dialog',
                                dialogMessage: message
                            });
                        }
                    });
                });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-cancel-job")) {
            if($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Cancel Jobs");
                return true;
            }
            //if ($row !== null && $row.length && !$row.is(".job-state-canceling") && !$row.is(".system-true")) {
            if ($row !== null && $row.length && !$row.is(".job-status-finished") && !$row.is(".job-status-canceled")) {
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                var job_is_remote = selectedJob.remote;
                var url = "/api/jobs/" + $row.data("dbid") + url_sid;
                $but.removeClass("disabled")
                    .off("click")
                    .attr("href", url)
                    .on("click", function (event) {
                        event.preventDefault();
                        var load = PHD.showLoadingIndicator($row.find("[data-source='status']"));
                        var payload = null;
                        if (job_is_remote === true) {
                            payload = JSON.stringify({remote:true});
                        }
                        var request = connectionsvc.createApplianceRequest($but.attr("href"), "DELETE", payload);
                        httpService(request).then(function (response) {
                            PHD.hideLoadingIndicator(load);
                            tab.update();
                        }, function(response) {
                            PHD.hideLoadingIndicator(load);
                            var data = response.data;
                            if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                                var message = data.result[0].message;
                                ngDialogService.open({
                                    dialogType: 'Error',
                                    modelDialogId: 'job-error-dialog',
                                    dialogMessage: message
                                });
                            }
                        });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-view-details")) {
            if ($row !== null && $row.length && selectedIDs.length < 2) {
                $but.removeClass("disabled");
                $but.off("click");
                if (!tab.inventoryOpen) {
                    $but
                        .html('<i class="icon-tasks icon-large"></i>' + gettext('View Details'))
                        .on("click", function (event) {
                            event.preventDefault();
                            tab.showJobDetails();
                        });
                } else {
                    $but
                        .html('<i class="icon-eyeclose icon-large"></i>' + gettext('Hide Details'))
                        .on("click", function (event) {
                            event.preventDefault();
                            tab.hideJobDetails();
                        });
                }
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-pop-out")) {
            if ($row !== null && $row.length && tab.index == JOB_TAB_HISTORY) {
                $but.removeClass("disabled")
                    .off("click")
                    .on("click", function (event) {
                        event.preventDefault();
                        var sid = rowData.system_id;
                        var bids = rowData.backup_id;
                        var size = rowData.size;
                        var name = rowData.name;

                        if(rowData.type === "certification"){
                            if(rowData.status === "AlreadyRunning"){
                                var message = "Same job was already in progress";
                                PHD.throwNotice({"result": [{"message": message}]});
                            }
                            else{
                                var dcaSessionId =  parseInt(rowData.id.slice(0, rowData.id.indexOf(".")));
                                var url = "/api/certification/history/"+ dcaSessionId + "/?sid="+ sid;
                                var load = PHD.showLoadingIndicator("body", true, "Loading...");
                                var resp = PHD.Ajax.get(url, load);
                                resp.done(function(data) {
                                    PHD.hideLoadingIndicator(load);
                                    var setTopvalue='top:75px;';
                                    if (screen.height >1024){
                                        setTopvalue='top:15%;'
                                    }else if(screen.height == 1024){
                                        setTopvalue='top:10%;'
                                    }
                                    var scope = angular.element(document.getElementById("idJobsPage")).scope();
                                    scope.DCAActiveJobDetails = false;
                                    var dimensions = 'width:1200px;height:500px;';
                                    if (data.history != undefined && data.history != null && data.history.detail != undefined && data.history.detail != null) {
                                        ngDialogService.open({
                                            template: 'app/joborders/certify-backups/dcaJobDetails.html',
                                            modelDialogId: 'dcaJobDetails-dialog',
                                            name: 'dcaJobDetails-dialog',
                                            scope: scope,
                                            ngDialogPostionStyle: setTopvalue,
                                            ngDialogStyle: dimensions,
                                            data: data
                                        });
                                    } else {
                                        var message = "There are no details for this job.";
                                        PHD.throwNotice({"result": [{"message": message}]});
                                    }
                                });
                            }
                        }
                        else if (!isNaN(bids) && rowData.status != 'Cancelled') {
                            // These variables MUST BE set to use status.html.
                            PHD.appliance_sid = sid;
                            backup_ids = bids.toString();
                            copies = (rowData.type == 'replication');
                            PHD.wizard("app/inventory/details/status.html", "view-status", {
                                title: gettext("Backup Statuses for Job: " + name),
                                dialogClass: "standalone",
                                height: 500,
                                helpArticle: PHD.App.getHelpLink("backup", true)
                            });
                        } else {
                            if (rowData.failed === false) {
                                copy_ids = rowData.id;
                                copies = true;
                                PHD.wizard("app/inventory/backup-copy-details/status.html", "view-status", {
                                    title: gettext("Backup Copy Statuses for Job: " + name),
                                    dialogClass: "standalone",
                                    height: 390,
                                    helpArticle: PHD.App.getHelpLink("backup", true)
                                });
                            } else {
                                var message = "";
                                message += (rowData.expanded_error !== undefined) ? rowData.expanded_error : rowData.message;
                                console.log("message " + rowData.message);
                                console.log("expanded_error " + rowData.expanded_error);
                                PHD.throwNotice({"result": [{"message": message}]});
                            }
                        }
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-hide-system-jobs")) {
            $but.on("click", function (event) {
                event.preventDefault();
                if ($but.data("hide") === undefined || $but.data("hide") === true) {
                    $but.data("hide", false);
                    $but.html('<i class="icon-eye icon-large"></i> ' + gettext('Show System Jobs'));
                    tab.toggleSystemJobs(false);
                } else {
                    $but.data("hide", true);
                    $but.html('<i class="icon-eyeclose icon-large"></i> ' + gettext('Hide System Jobs'));
                    tab.$table.find(".system-true").toggleClass("hidden");
                    tab.toggleSystemJobs(true);
                }
            });
            return true;
        }

        if ($but.is(".btn-disable-job")) {
            if ($scope.backupDisabled) {
                $but.disableButton();
                $but.prop("title", "Recovery operators cannot enable/disable jobs");
                return true;
            }
            if (selectedIDs.length > 0 || $row !== null && $row.length && !$row.is(".save-status-pending") && !$row.is(".save-status-errors") &&
                !$row.is(".schedule-type-on-demand") && !$row.is(".job-type-replication")) {

                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));

                var elem = angular.element(document.getElementById("idJobsPage"));
                var $jobScope = elem.scope();

                var disableURL = "/api/joborders/disable/" + $row.data("dbid") + url_sid;
                var enableURL = "/api/joborders/enable/" + $row.data("dbid") + url_sid;

                if (selectedIDs.length === 0) {
                    if ($row.is(".job-enabled-true")) {
                        $but.attr("href", disableURL)
                            .html('<i class="icon-off icon-large"></i> ' + gettext('Disable'));
                        if(selectedJob.type === "certification" ) {
                            $jobScope.currentDCAJobDisabled = true;
                            if(selectedJob.status === 'InstantLab' || selectedJob.status === 'Failover') {
                                $jobScope.currentDCAJobIsFailover = true;
                            }
                        }
                    } else if ($row.is(".job-enabled-false")) {
                        $but.attr("href", enableURL)
                            .html('<i class="icon-restart icon-large"></i> ' + gettext('Enable'));
                        if(selectedJob.type === "certification" ) {
                            $jobScope.currentDCAJobDisabled = false;
                            if(selectedJob.status === 'InstantLab' || selectedJob.status === 'Failover') {
                                $jobScope.currentDCAJobIsFailover = false;
                            }
                        }
                    }
                } else if (selectedIDs.length === 1) {
                    var selectedJob = selectedIDs[0];
                    var jobID = selectedJob.id;
                    disableURL = "/api/joborders/disable/" + jobID + url_sid;
                    enableURL = "/api/joborders/enable/" + jobID + url_sid;
                    var enabled = selectedJob.enabled;
                    if (enabled) {
                        $but.attr("href", disableURL)
                            .html('<i class="icon-off icon-large"></i> ' + gettext('Disable'));
                        if(selectedJob.type === "certification" ) {
                            $jobScope.currentDCAJobDisabled = true;
                            if(selectedJob.status === 'InstantLab' || selectedJob.status === 'Failover') {
                                $jobScope.currentDCAJobIsFailover = true;
                            }
                        }
                    } else {
                        $but.attr("href", enableURL)
                            .html('<i class="icon-restart icon-large"></i> ' + gettext('Enable'));
                        if(selectedJob.type === "certification" ) {
                            $jobScope.currentDCAJobDisabled = false;
                            if(selectedJob.status === 'InstantLab' || selectedJob.status === 'Failover') {
                                $jobScope.currentDCAJobIsFailover = false;
                            }
                        }
                    }                  
                } else {
                    $but.attr("href", "/api/joborders/")
                        .html('<i class="icon-off icon-large"></i> ' + gettext('Toggle'));                   
                }
                $but.removeClass("disabled")
                    .on("click", function (event) {
                        event.preventDefault();
                        if (selectedIDs.length === 0) {
                            var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                            $analytics && $analytics.eventTrack('Enable/Disable Job', {  category: 'Jobs', label: analyticsSource });

                            var request = connectionsvc.createApplianceRequest($but.attr("href"), "PUT");
                            httpService(request).then(function (response) {
                                PHD.hideLoadingIndicator(load);
                                tab.update();
                            }, function(response) {
                                PHD.hideLoadingIndicator(load);
                                var data = response.data;
                                if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                                    var message = data.result[0].message;
                                    ngDialogService.open({
                                        dialogType: 'Error',
                                        modelDialogId: 'job-error-dialog',
                                        dialogMessage: message
                                    });
                                }
                            });
                        } else {
                            var load = PHD.showLoadingIndicator('body', true, 'Enabling/Disabling Jobs....');
                            $analytics && $analytics.eventTrack('Enable/Disable Multiple Jobs', {  category: 'Jobs', label: analyticsSource });
                            var remainingCalls = selectedIDs.length;
                            selectedIDs.forEach (function(jobObj, index) {
                                var id = jobObj.id;
                                var a = id.split('.');
                                var enabled = jobObj.enabled;
                                var toggleURL = $but.attr("href") + (enabled ? "disable" : "enable") + "/" + a[0] + "/sid=" + a[1];

                                var request = connectionsvc.createApplianceRequest(toggleURL, "PUT");
                                httpService(request).then(function (response) {
                                    --remainingCalls;
                                    checkRemaining(remainingCalls);
                                }, function(response) {
                                    --remainingCalls;
                                    checkRemaining(remainingCalls);
                                });

                                function checkRemaining(remainingCalls) {
                                    if(remainingCalls <= 0) {
                                        PHD.hideLoadingIndicator(load);
                                        // Once complete, uncheck all and update.
                                        tab.$checkSelectAll.prop("checked", false);
                                        tab.selectedIDs = [];
                                        tab.updateButtons();
                                        tab.update(true);
                                    }
                                }
                            });
                        }
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-delete-job")) {
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot Delete Jobs");
                return true;
            }
            if ($row !== null && $row.length && selectedIDs.length < 2) {
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                $but.removeClass("disabled")
                    .attr("href", "/api/joborders/" + $row.data("dbid") + url_sid)
                    .on("click", function (event) {
                        event.preventDefault();
                        var formattedJobName = selectedJob !== null && selectedJob.name !== undefined ? (" " +selectedJob.name) : "";
                        $analytics && $analytics.eventTrack('Delete Job', {  category: 'Jobs', label: analyticsSource });
                        var url = $but.attr("href"),
                            confirm = PHD.confirmDialog(
                                "<p>" + gettext("Are you sure you want to delete the job") + formattedJobName + "?</p>",
                                { title: gettext("Confirm Job Deletion") }
                            );

                        confirm.done(function () {
                            var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                            var jobID = $row.data("dbid");
                            PHD.Ajax.delete($but.attr("href"), null, load)
                                .done(function (data) {
                                    //PHD.hideLoadingIndicator(load);
                                    tab.deleteRow(jobID);
                                    tab.update();
                                });
                        });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-run-job")) {
            if ($scope.backupDisabled) {
                $but.disableButton();
                $but.prop("title", "Recovery operators cannot run jobs");
                return true;
            }
            if(rowData !== null && rowData.type == "certification" && rowData.status != "Idle"){
                $but.disableButton();
                return true;
            }
            if ($row !== null && $row.length && !$row.is(".save-status-pending") && !$row.is(".save-status-errors") &&
                !$row.is(".job-enabled-false") && !$row.is(".job-type-replication") && selectedIDs.length < 2) {
                var selectedJob = rowData;
                var url_sid = (selectedJob === null) ? "/" : ("/?sid=" + (selectedJob.system_id !== undefined ? selectedJob.system_id : selectedJob.sid));
                var url = selectedJob.type === "certification" ? "/api/joborders/test/" : "/api/joborders/run/";

                $but.removeClass("disabled")
                    .attr("href", url + $row.data("dbid") + url_sid)
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Run Job', {  category: 'Jobs', label: analyticsSource });
                        var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));

                        var request = connectionsvc.createApplianceRequest($but.attr("href"), "PUT");
                        httpService(request).then(function (response) {
                            PHD.hideLoadingIndicator(load);
                            tab.update();
                        }, function(response) {
                            PHD.hideLoadingIndicator(load);
                            var data = response.data;
                            if (data !== undefined && data.result !== undefined && data.result[0] !== undefined) {
                                var message = data.result[0].message;
                                ngDialogService.open({
                                    dialogType: 'Error',
                                    modelDialogId: 'job-error-dialog',
                                    dialogMessage: message
                                });
                            }
                        });
                    });
            } else {
                $but.disableButton();
            }
            return true;
        }

        if ($but.is(".btn-run-jobs")) {
            if ($scope.backupDisabled) {
                $but.disableButton();
                $but.prop("title", "Recovery operators cannot run jobs");
                return true;
            }
        }

        if ($but.is(".btn-search-files") || $but.is(".btn-windows-recovery")){
            // monitor users cannot search files or start windows instant recovery
            if (!$scope.user.monitor){
                $but.removeClass("disabled");
            } else {
                var msg = "";
                if ($but.is(".btn-search-files")) {
                    msg = "Search Files";
                } else if ($but.is(".btn-windows-recovery")) {
                    msg = "start Windows Instant Recovery";
                }
                $but.addClass("disabled");
                $but.prop("title", "Monitor users cannot " + msg);
            }
        }

        if ($but.is(".btn-recover-files")) {
            var multipleSelected = ($row && $row.filter("[data-type=backup]").length > 1);
            if (multipleSelected) {
                $but.attr('title', 'Multiple backups selected.  Only one backup of this type can be recovered at a time.');
            } else {
                $but.removeAttr('title');
            }
            if ($row && (multipleSelected || $row.hasClass('backup-type-archive'))) {
                $but.disableButton();
                return true;
            } else {
                var catalogView = $but.data("catalog-view") || BACKUP_CATALOG_VIEW_VM;
                var selections_obj = getSelectedBackups(tab);

                $but.removeClass("disabled")
                    .attr("href", "app/recover/flr/filerecovery.html")
                    .on("click", function (event) {
                        event.preventDefault();
                        $analytics && $analytics.eventTrack('Recover Files', {  category: 'Recover', label: analyticsSource });
                        PHD.wizard($but.attr("href"), "recover-files", {
                            title: gettext("Create File-Level Recovery Job"),
                            helpArticle: PHD.App.getHelpLink("filerecovery", true),
                            dialogClass: "dialog-recover-files dialog-modal-form",
                            // lmc
                            open: function (event, ui) {
                                backupInfo = {"backups": []};
                                isIRJob = false;
                                selections = selections_obj;
                                objects = $but.data('objects');
                                view = catalogView;
                            }
                        })
                    });
            }
            return true;
        }

        if ($but.is(".btn-delete-backup")) {
            // monitor users cannot delete backups
            if (!$scope.user.monitor) {
                if ($scope.configureDisabled) {
                    $but.disableButton();
                    $but.prop("title", "Operators cannot delete backups");
                    return true;
                }
                if ($row !== null && $row.length && $row.is("[data-type=backup]")) {
                    $but.removeClass("disabled")
                        .attr("href", "/api/backups/related/")
                        .on("click", function (event) {
                            event.preventDefault();
                            if (!$row.hasClass('backup-type-archive') && !$row.data('remote')) {
                                var load = PHD.showLoadingIndicator($but);
                                var related_url;
                                var selections = tab.TreeTable.getSelectionObject();
                                var bid_string = selections.backups.toString();
                                //we'll need this when we allow hbc on target delete
                                /*if ($row.data('remote')) {
                                 related_url = "/api/backups/target-related/" + bid_string + "/?sid=" + selections.system_id;
                                 PHD.Ajax.get(related_url, load)
                                 .done(function (data) {
                                 $analytics && $analytics.eventTrack('Delete Backup on Target', {  category: 'Backup', label: analyticsSource });
                                 tab.confirmDelete(data, true);
                                 });
                                 } else {*/
                                related_url = $but.attr("href") + bid_string + "/?sid=" + selections.system_id + '&original';
                                PHD.Ajax.get(related_url, load)
                                    .done(function (data) {
                                        $analytics && $analytics.eventTrack('Delete Backup', {
                                            category: 'Backup',
                                            label: analyticsSource
                                        });
                                        tab.confirmDelete(data);
                                    });
                                //   }
                            } else if ($row.hasClass('backup-type-archive')) {
                                var message = "Individual Backup Copies cannot be deleted." + "<br/>" + "<br/>" +
                                    "Go to the appliance configuration page for the target to purge all backup copies.";
                                PHD.throwNotice({"result": [{"message": message}]}, {});
                            } else if ($row.data('remote')) {
                                var message = "Hot Backup Copies on the target cannot be deleted.";
                                PHD.throwNotice({"result": [{"message": message}]}, {});
                            }
                        });
                } else {
                    $but.disableButton();
                }
                return true;
            } else {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Delete backups");
                return true;
            }
        }

        if ($but.is(".btn-flr-details")) {
            // monitor users cannot view details
            if (!$scope.user.monitor) {
                if ($row !== null && $row.length) {
                    $but.removeClass("disabled")
                        .attr("href", "/api/jobs/active/flr/" + $row.attr("flr_id") + "/?sid=" + $row.attr('system-id'))
                        .on("click", function (event) {
                            event.preventDefault();
                            tab.showFLRDetails($row.attr("flr_id"), $row.attr('system-id'));
                        });
                } else {
                    $but.disableButton();
                }
                return true;
            } else {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot view details");
                return true;
            }
        }

        if ($but.is(".btn-delete-flr")) {
            //monitor users cannot delete FLR objects
            if (!$scope.user.monitor) {
                if ($row !== null && $row.length) {
                    var flr_id = $row.attr("flr_id");
                    var flr_is_remote = $row.data("remote");
                    $but.removeClass("disabled")
                        .attr("href", "/api/jobs/" + $row.attr("flr_id") + "/?sid=" + $row.attr('system-id'))
                        .on("click", function (event) {
                            event.preventDefault();
                            $analytics && $analytics.eventTrack('Delete FLR', {
                                category: 'Recover',
                                label: analyticsSource
                            });
                            var url = $but.attr("href"),
                                confirm = PHD.confirmDialog(
                                    "<p>" + gettext("Are you sure you want to delete this FLR Object?") + "</p>",
                                    {title: gettext("Confirm FLR Deletion")}
                                );

                            confirm.done(function () {
                                var load = PHD.showLoadingIndicator($row.find("[data-source=\"status\"]"));
                                $row.find("[data-source=\"status\"]").html("Deleting");
                                var payload = null;
                                if (flr_is_remote === true) {
                                    payload = JSON.stringify({remote: true});
                                }
                                PHD.Ajax.delete(url, payload, load)
                                    .done(function (data) {
                                        //PHD.hideLoadingIndicator(load);
                                        tab.deleteFLR(flr_id);
                                    });
                            });
                        });
                } else {
                    $but.disableButton();
                }
                return true;
            } else {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot delete FLR");
                return true;
            }
        }

        if ($but.is(".btn-archive")) {
            if ($scope.user.monitor) {
                $but.disableButton();
                $but.prop("title", "Monitor users cannot Create Jobs");
                return true;
            }
            if ($scope.configureDisabled) {
                $but.disableButton();
                $but.prop("title", "Operators cannot Create Jobs");
                return true;
            }
            $but.attr("href", "app/joborders/archive/archive.html")
                .on("click", function (event) {
                    event.preventDefault();
                    $analytics && $analytics.eventTrack('Add Backup Copy Job', {  category: 'Backup', label: analyticsSource });
                    PHD.wizard($but.attr("href"), "add-job", {
                        title: gettext("Create Backup Copy Job"),
                        width: 900,
                        height: 700,
                        dialogClass: "dialog-add-job dialog-modal-form",
                        helpArticle: PHD.App.getHelpLink("archive", true),
                        // lmc
                        open: function () {
                            jobEdit = false;
                            jobID = 0;
                            preSelected = $but.data('selections');
                            useCopiedAssets = 0;
                            if( tab !== null && tab.isCopyTab == true ) {
                                useCopiedAssets = 1;
                            }
                            var dateFormat = $.datepicker.formatDate(DATE_FORMAT, new Date());
                            $("#id_start_date").val(dateFormat);
                            $("#id_name").val("Backup Copy Job");
                            $("#id_encrypt").prop('disabled', false);
                        },
                        beforeClose: function (event, ui) {
                            $(".backup-job-popover").remove();
                            $(".datepicker input:text").datepicker("destroy");
                            $(".ui-spinner-input").spinner("destroy");
                        }
                    });
                });
            return true;
        }

        if ($but.is(".btn-export-csv")) {
            $but.attr("href", tab.getUpdateUrl(true));
            return true;
        }

        if ($but.is(".btn-show-filter")) {
            $but.on("click", function (event) {
                event.preventDefault();
                if (tab) {
                    tab.showFilter();
                }
            });
            return true;
        }

        if ($but.parent().is(".btn-group")) {
            $but
                .off("click.btngroup")
                .on("click.btngroup", function (event) {
                    event.preventDefault();
                    event.stopPropagation();
                    var $menu = $but.parent().toggleClass("open");

                    if ($menu.is(".open")) {
                        $('body').one("click.btngroupoff", function (event) {
                            event.preventDefault();
                            $menu.removeClass("open");
                        });
                    }
                })
                .next("ul").find("a")
                .off("click.btngroup")
                .on("click.btngroup", function () {
                    event.preventDefault();
                    $but.parent().removeClass("open");
                });
            return true;
        }

        $but.on("click", function (event) {
            if (!$but.is(".plain-link") && !$but.closest(".note").length)
                event.preventDefault();
        });

        return false;
    };
	
})(jQuery, window.PHD || (window.PHD = {}), window);
